package com.shixing.edit;

import android.content.Context;
import android.content.Intent;
import android.media.MediaScannerConnection;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.view.View;
import android.widget.MediaController;
import android.widget.Toast;
import android.widget.VideoView;

import androidx.appcompat.app.AppCompatActivity;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

public class VideoPlayActivity extends AppCompatActivity {

    public static void start(Context context, String path) {
        Intent starter = new Intent(context, VideoPlayActivity.class);
        starter.putExtra("path", path);
        context.startActivity(starter);
    }

    private String mVideoPath = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_preview);

        VideoView videoView = findViewById(R.id.video_view);
        mVideoPath = getIntent().getStringExtra("path");
        videoView.setVideoPath(mVideoPath);

        final MediaController controller = new MediaController(this);
        videoView.setMediaController(controller);
        controller.setMediaPlayer(videoView);
        videoView.start();

        videoView.post(new Runnable() {
            @Override
            public void run() {
                controller.show();
            }
        });
    }

    public void back(View view) {
        finish();
    }

    public void save(View view) {
        File folder = new File(Environment.getExternalStorageDirectory() + File.separator + Environment.DIRECTORY_DCIM + File.separator + "Camera");
        if (!folder.exists()) {
            boolean ret = folder.mkdirs();
            if (!ret) {
                Toast.makeText(this, "创建文件夹失败", Toast.LENGTH_SHORT).show();
                return;
            }
        }

        File src = new File(mVideoPath);
        File dest = new File(folder, "SXEditDemo" + System.currentTimeMillis() + ".mp4");
        copyFile(src, dest);

        MediaScannerConnection.scanFile(this, new String[]{dest.getAbsolutePath()}, null, new MediaScannerConnection.OnScanCompletedListener() {
            @Override
            public void onScanCompleted(String path, Uri uri) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Toast.makeText(VideoPlayActivity.this, "已保存到相册", Toast.LENGTH_SHORT).show();
                    }
                });
            }
        });
    }

    public static void copyFile(File src, File dest) {
        if (!src.getAbsolutePath().equals(dest.getAbsolutePath())) {
            try {
                InputStream in = new FileInputStream(src);
                FileOutputStream out = new FileOutputStream(dest);
                byte[] buf = new byte[1024];

                int len;
                while ((len = in.read(buf)) >= 0) {
                    out.write(buf, 0, len);
                }
                in.close();
                out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
