package com.shixing.edit.adapter;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.DiffUtil;
import androidx.recyclerview.widget.ListAdapter;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.R;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;

import java.util.List;

public class ActionAdapter extends ListAdapter<ActionItem, ActionAdapter.ActionViewHolder> {

    private OnActionClickListener mListener;
    public List<ActionItem> mFirstLevelList;
    public List<ActionItem> mSecondLevelList;
    public List<ActionItem> mThreeLevelList;
    public List<ActionItem> mFourLevelList;
    public ActionAdapter() {
        super(new ActionDiffCallback());
    }

    @NonNull
    @Override
    public ActionViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ActionViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_action, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ActionViewHolder holder, int position) {
        holder.bind(getItem(position));
    }

    public void setOnActionClickListener(OnActionClickListener listener) {
        mListener = listener;
    }

    class ActionViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        private final ImageView mIcon;
        private final TextView mName;

        ActionViewHolder(@NonNull View itemView) {
            super(itemView);
            mIcon = itemView.findViewById(R.id.action_icon);
            mName = itemView.findViewById(R.id.action_name);
            itemView.setOnClickListener(this);
        }

        void bind(ActionItem item) {
            mIcon.setImageResource(item.iconResId);
            mName.setText(item.actionName);
        }

        @Override
        public void onClick(View v) {
            if (mListener != null) {
                mListener.onActionClick(getItem(getLayoutPosition()));
            }
        }
    }

    static class ActionDiffCallback extends DiffUtil.ItemCallback<ActionItem> {

        @Override
        public boolean areItemsTheSame(@NonNull ActionItem oldItem, @NonNull ActionItem newItem) {
            return oldItem.actionName.equals(newItem.actionName);
        }

        @Override
        public boolean areContentsTheSame(@NonNull ActionItem oldItem, @NonNull ActionItem newItem) {
            return oldItem.actionName.equals(newItem.actionName);
        }
    }


}
