package com.shixing.edit.adapter;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.R;

import java.util.ArrayList;
import java.util.List;

public class AlignmentAdapter extends RecyclerView.Adapter<AlignmentAdapter.AlignmentHolder> {

    private List<AlignmentData> data = new ArrayList<>();

    private AlignmentClickListener listener;

    public void setData(List<AlignmentData> data) {
        this.data = data;
        notifyDataSetChanged();
    }

    public void setListener(AlignmentClickListener listener) {
        this.listener = listener;
    }

    @NonNull
    @Override
    public AlignmentHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_alignment, parent, false);
        return new AlignmentHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull AlignmentHolder holder, int position) {
        AlignmentData alignmentData = data.get(position);
        holder.bind(alignmentData);
        holder.itemView.setTag(position);
    }

    @Override
    public int getItemCount() {
        return data == null ? 0 : data.size();
    }

    class AlignmentHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        private ImageView icon;
        private TextView name;

        public AlignmentHolder(@NonNull View itemView) {
            super(itemView);

            icon = itemView.findViewById(R.id.iv_icon);
            name = itemView.findViewById(R.id.tv_name);
            itemView.setOnClickListener(this);
        }

        void bind(AlignmentData alignmentData) {
            icon.setImageResource(alignmentData.iconResId);
            name.setText(alignmentData.name);
        }

        @Override
        public void onClick(View v) {
            if (listener != null) {
                listener.onAlignmentClick(data.get((Integer) itemView.getTag()));
            }
        }
    }

    public static class AlignmentData {
        public String name;
        public int iconResId;

        public AlignmentData(String name, int iconResId) {
            this.name = name;
            this.iconResId = iconResId;
        }
    }

    public interface AlignmentClickListener {
        void onAlignmentClick(AlignmentData data);
    }
}
