package com.shixing.edit.adapter;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.shixing.edit.R;
import com.shixing.sxedit.track.SXTrack;

import java.util.ArrayList;
import java.util.List;

public class BlendAdapter extends RecyclerView.Adapter<BlendAdapter.BlendViewHolder> {

    private List<BlendData> data = new ArrayList<>();
    private SXTrack.SXBlendMode selectedMode = SXTrack.SXBlendMode.Normal;

    public void setBlendModes(List<BlendData> data) {
        this.data = data;
        notifyDataSetChanged();
    }

    @NonNull
    @Override
    public BlendViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new BlendViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_blend, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull BlendViewHolder holder, int position) {
        BlendData blendData = data.get(position);
        holder.name.setText(blendData.name);
        Glide.with(holder.thumb).load(blendData.thumbUrl).into(holder.thumb);
        holder.thumb.setBackgroundResource(selectedMode == blendData.blendMode ? R.drawable.bg_blend_image : 0);
    }

    @Override
    public int getItemCount() {
        return data == null ? 0 : data.size();
    }

    private OnBlendModeSelectedListener listener;

    public void setOnSelectedListener(OnBlendModeSelectedListener listener) {
        this.listener = listener;
    }

    public void setSelectedMode(SXTrack.SXBlendMode blendMode) {
        selectedMode = blendMode;
        notifyDataSetChanged();
    }

    public interface OnBlendModeSelectedListener {
        void onBlendModeSelected(SXTrack.SXBlendMode mode);
    }

    class BlendViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        private final ImageView thumb;
        private final TextView name;

        public BlendViewHolder(@NonNull View itemView) {
            super(itemView);

            thumb = itemView.findViewById(R.id.iv_thumb);
            name = itemView.findViewById(R.id.tv_name);

            itemView.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {
            BlendData blendData = data.get(getLayoutPosition());
            if (blendData.blendMode != selectedMode) {
                selectedMode = blendData.blendMode;
                notifyDataSetChanged();
                if (listener != null) {
                    listener.onBlendModeSelected(selectedMode);
                }
            }
        }
    }

    public static class BlendData {
        String name;
        String thumbUrl;
        SXTrack.SXBlendMode blendMode;

        public BlendData(String name, String thumbUrl, SXTrack.SXBlendMode blendMode) {
            this.name = name;
            this.thumbUrl = thumbUrl;
            this.blendMode = blendMode;
        }
    }

}
