package com.shixing.edit.adapter;

import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.shixing.edit.R;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;

import java.util.ArrayList;
import java.util.List;

public class FilterAdapter extends RecyclerView.Adapter<FilterAdapter.TextViewHolder> {
    private Context mContext;
    private List<ActionItem> mActionList;
    private OnActionClickListener mListener;
    private String mSelectedItemName;

    public FilterAdapter(Context mContext) {
        this.mContext = mContext;
    }

    public void updateData(List<ActionItem> mTitleList) {
        if (mTitleList == null)
            mTitleList = new ArrayList<>();

        this.mActionList = mTitleList;
        notifyDataSetChanged();
    }

    public void setActionClickListener(OnActionClickListener listener) {
        this.mListener = listener;
    }

    public void setSelectedItemName(String selectedItemName) {
        mSelectedItemName = selectedItemName;
        notifyDataSetChanged();
    }

    @NonNull
    @Override
    public TextViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new TextViewHolder(LayoutInflater.from(mContext).inflate(R.layout.item_filter, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull final TextViewHolder holder, final int position) {
        final ActionItem actionItem = mActionList.get(position);
        holder.action_name.setText(actionItem.actionName);
        boolean isSelected = actionItem.actionName.equals(mSelectedItemName);
        holder.itemView.setSelected(isSelected);
        holder.action_name.setTextColor(isSelected ? mContext.getResources().getColor(R.color.text_selected_red) : mContext.getResources().getColor(R.color.text_white));

        Glide.with(holder.iv_view)
                .load(mActionList.get(position).srcPath)
                .centerCrop()
                .error(R.drawable.icon_wu)
                .into(holder.iv_view);

        holder.itemView.setOnClickListener(v -> {
            if (mListener != null && !isSelected) {
                mSelectedItemName = actionItem.actionName;
                notifyDataSetChanged();
                mListener.onActionClick(actionItem);
            }
        });
    }

    @Override
    public int getItemCount() {
        return mActionList.size();
    }

    public class TextViewHolder extends RecyclerView.ViewHolder {
        TextView action_name;
        ImageView action_icon;
        ImageView iv_view;
        public TextViewHolder(View view) {
            super(view);
            action_name = view.findViewById(R.id.tv_name);
            action_icon = view.findViewById(R.id.iv_wu);
            iv_view=view.findViewById(R.id.iv_view);
        }

    }


}
