package com.shixing.edit.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.shixing.edit.R;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;

import java.util.ArrayList;
import java.util.List;

public class TextStyleAdapter extends RecyclerView.Adapter<TextStyleAdapter.TextViewHolder> {
    private Context mContext;
    private List<ActionItem> mTitleList;
    private OnActionClickListener mListener;
    private int selectedPosition;
    private String style;
    public TextStyleAdapter(Context mContext,String style) {
        this.mContext = mContext;
        this.style=style;
    }

    public void updateData(List<ActionItem> mTitleList) {
        if (mTitleList == null)
            mTitleList = new ArrayList<>();

        this.mTitleList = mTitleList;
        notifyDataSetChanged();
    }

    public void setActionClickListener(OnActionClickListener listener) {
        this.mListener = listener;
    }

    @NonNull
    @Override
    public TextViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        TextViewHolder holder=null;
        switch (style){
            case "text_style_name":
                holder=new TextViewHolder(LayoutInflater.from(mContext).inflate(R.layout.item_text_style_name, parent, false));
                break;
            case "text_style":
                holder=new TextViewHolder(LayoutInflater.from(mContext).inflate(R.layout.item_text_style, parent, false));
                break;
        }
        return holder;
    }

    @Override
    public void onBindViewHolder(@NonNull final TextViewHolder holder, final int position) {
        switch (style){
            case "text_style_name":
                holder.textView.setText(mTitleList.get(position).actionName);
                holder.textView.setSelected(selectedPosition == position);
                if (selectedPosition == position) {
                    holder.textView.setTextColor(mContext.getResources().getColor(R.color.text_selected_red));
                } else {
                    holder.textView.setTextColor(mContext.getResources().getColor(R.color.text_white));
                }
                holder.textView.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        if (mListener != null) {
                            notifyItemChanged(selectedPosition);
                            selectedPosition = position;
                            notifyItemChanged(selectedPosition);
                            mListener.onActionClick(new ActionItem(holder.textView.getText().toString()));
                        }
                    }
                });
                break;
            case "text_style":
                ActionItem actionItem = mTitleList.get(position);
                if ("".equals(actionItem.actionName)) {
                    holder.iv_view.setImageResource(R.drawable.icon_wu);
                } else {
                    Glide.with(holder.iv_view).load(actionItem.actionName).into(holder.iv_view);
                }
                holder.iv_view.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        if (mListener != null) {
                            mListener.onActionClick(mTitleList.get(position));
                        }
                    }
                });
                break;
        }


    }

    @Override
    public int getItemCount() {
        return mTitleList.size();
    }

    public class TextViewHolder extends RecyclerView.ViewHolder {
        TextView textView;
        ImageView iv_download;
        ImageView iv_view;
        public TextViewHolder(View view) {
            super(view);
            switch (style){
                case "text_style_name":
                    textView = view.findViewById(R.id.tv_text);
                    iv_download=view.findViewById(R.id.iv_download);
                    break;
                case "text_style":
                    iv_view = view.findViewById(R.id.iv_view);
                    break;
            }

        }

    }


}
