package com.shixing.edit.adapter;

import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.R;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;

import java.util.ArrayList;
import java.util.List;

public class TitleAdapter extends RecyclerView.Adapter<TitleAdapter.TextViewHolder> {
    private Context mContext;
    private List<String> mTitleList;
    private OnActionClickListener mListener;
    private int selectedPosition;

    public TitleAdapter(Context mContext) {
        this.mContext = mContext;
    }

    public void updateData(List<String> mTitleList) {
        if (mTitleList == null)
            mTitleList = new ArrayList<>();

        this.mTitleList = mTitleList;
        notifyDataSetChanged();
    }

    public void setActionClickListener(OnActionClickListener listener) {
        this.mListener = listener;
    }

    @NonNull
    @Override
    public TextViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new TextViewHolder(LayoutInflater.from(mContext).inflate(R.layout.item_menu_title, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull final TextViewHolder holder, final int position) {
        holder.textView.setText(mTitleList.get(position));
        holder.textView.setSelected(selectedPosition == position);
        if (selectedPosition == position) {
            holder.textView.setTextColor(mContext.getResources().getColor(R.color.text_selected_red));
        } else {
            holder.textView.setTextColor(mContext.getResources().getColor(R.color.text_white));
        }
        holder.textView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mListener != null) {
                    notifyItemChanged(selectedPosition);
                    selectedPosition = position;
                    notifyItemChanged(selectedPosition);
                    mListener.onActionClick(new ActionItem(holder.textView.getText().toString()));
                }
            }
        });
    }

    public void setSelectedPosition(int position) {
        selectedPosition = position;
        notifyDataSetChanged();
        if (mListener != null)
            mListener.onActionClick(new ActionItem(mTitleList.get(selectedPosition)));
    }

    @Override
    public int getItemCount() {
        return mTitleList.size();
    }

    public class TextViewHolder extends RecyclerView.ViewHolder {
        TextView textView;

        public TextViewHolder(View view) {
            super(view);
            textView = view.findViewById(R.id.tv_text);
        }

    }


}
