package com.shixing.edit.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.shixing.edit.R;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;

import java.util.ArrayList;
import java.util.List;

public class TransitionItemAdapter extends RecyclerView.Adapter<TransitionItemAdapter.TextViewHolder> {
    private Context mContext;
    private List<ActionItem> mActionList;
    private OnActionClickListener mListener;
    private int selectedPosition;

    public TransitionItemAdapter(Context mContext) {
        this.mContext = mContext;
    }

    public void updateData(List<ActionItem> mTitleList) {
        if (mTitleList == null)
            mTitleList = new ArrayList<>();

        this.mActionList = mTitleList;
        notifyDataSetChanged();
    }

    public void setActionClickListener(OnActionClickListener listener) {
        this.mListener = listener;
    }

    @NonNull
    @Override
    public TextViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new TextViewHolder(LayoutInflater.from(mContext).inflate(R.layout.item_menu_transition, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull final TextViewHolder holder, final int position) {
        ActionItem actionItem = mActionList.get(position);
        holder.action_name.setText(actionItem.actionName);
        if ("".equals(actionItem.srcPath)) {
            holder.action_icon.setImageResource(R.drawable.icon_wu);
        } else {
            Glide.with(holder.action_icon).load(actionItem.srcPath).into(holder.action_icon);
        }
        holder.itemView.setSelected(selectedPosition == position);
//        if (selectedPosition == position) {
//            holder.action_name.setTextColor(mContext.getResources().getColor(R.color.text_selected_red));
//        } else {
            holder.action_name.setTextColor(mContext.getResources().getColor(R.color.text_white));
//        }
        holder.itemView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mListener != null) {
                    notifyItemChanged(selectedPosition);
                    selectedPosition = position;
                    notifyItemChanged(selectedPosition);
                    mListener.onActionClick(mActionList.get(position));
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        return mActionList.size();
    }

    public class TextViewHolder extends RecyclerView.ViewHolder {
        TextView action_name;
        ImageView action_icon;

        public TextViewHolder(View view) {
            super(view);
            action_name = view.findViewById(R.id.action_name);
            action_icon = view.findViewById(R.id.action_icon);
        }

    }


}
