package com.shixing.edit.adjust;

import android.graphics.SurfaceTexture;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.util.Range;
import android.view.MotionEvent;
import android.view.Surface;
import android.view.View;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.EditActivity;
import com.shixing.edit.R;
import com.shixing.edit.adapter.AdjustAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.utils.ToastUtil;
import com.shixing.sxedit.effect.SXColorAdjustEffect;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.ColorAdjustEffect;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.FilterEffect;
import com.shixing.sxedit.internal.GenericEffect;
import com.shixing.sxedit.internal.MainTrackGroup;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AdjustFragment extends BaseFragment implements OnActionClickListener {
    private static Map<SXColorAdjustEffect.SXColorSettings, Range<Float>> valueRangeMap = new HashMap<>();

    static {
        for (SXColorAdjustEffect.SXColorSettings setting : SXColorAdjustEffect.SXColorSettings.values()) {
            float[] range = ColorAdjustEffect.getValueRangeForColorSetting(setting);
            valueRangeMap.put(setting, Range.create(range[0], range[1]));
        }
    }
    private RecyclerView mItem1_recycler;
    private ArrayList<ActionItem> itemArrayList;
    private GenericEffect colorAdjustEffect;
    private SeekBar seekBar;

    private SXColorAdjustEffect.SXColorSettings currentSetting = SXColorAdjustEffect.SXColorSettings.Brightness;

    private TextView lowerText;
    private TextView upperText;
    private boolean isMainGroup = false;
    private View btnApplyToAll;

    @Override
    protected int initRootView() {
        return R.layout.fragment_adjust;
    }

    @Override
    protected void initArguments(Bundle arguments) {

    }

    @Override
    protected void initView() {
        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        showItem1(R.drawable.icon_lvjing);

        seekBar = (SeekBar) findViewById(R.id.seekBar);
        lowerText = (TextView) findViewById(R.id.tv_seekbar_start);
        upperText = (TextView) findViewById(R.id.tv_seekbar_end);
        btnApplyToAll = findViewById(R.id.apply_all);

        Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
        if (track == null) {
            isMainGroup = true;
            btnApplyToAll.setVisibility(View.GONE);
        }

        if (isMainGroup) {
            Effect effect = TrackActionManager.getInstance().getCurrentSelectEffect();
            if (effect == null) {
                double currentTime = ActionManager.getInstance().getListener().getCurrentTime();
                MainTrackGroup mainGroup = ActionManager.getInstance().getListener().getMainGroup();
                colorAdjustEffect = mainGroup.addGenericEffect(GenericEffect.GenericEffectType.ColorAdjustment);
                colorAdjustEffect.setStartTime(currentTime);
                if (TrackActionManager.getInstance().canBeAddEffect(this.colorAdjustEffect)){
                    TrackActionManager.getInstance().addEffect(this.colorAdjustEffect);
                    TrackActionManager.getInstance().onEffectSelect(this.colorAdjustEffect, false);
                }else {
                    this.colorAdjustEffect =null;
                    ToastUtil.showToast(getContext(),"已经超出最大层级，不能在当前位置添加");
                    new Handler().post(() -> TrackActionManager.getInstance().hideDetailFragment());
//                    TrackActionManager.getInstance().hideDetailFragment();
                }
            } else {
                colorAdjustEffect = (GenericEffect) effect;
            }
        } else {
            MediaTrack mediaTrack = (MediaTrack) track;
            List<GenericEffect> effects = mediaTrack.getColorAdjusts();
            if (effects != null && effects.size() > 0) {
                colorAdjustEffect = effects.get(0);
            } else {
                colorAdjustEffect = ((MediaTrack) track).addGenericEffect(GenericEffect.GenericEffectType.ColorAdjustment);
                colorAdjustEffect.setEffectFollowType(SXEffect.SXEffectTimeFollowType.FollowWholeProcess);
            }
        }

//        colorAdjustEffect = (ColorAdjustEffect) effect;


        setSeekbar(currentSetting);
    }

    @Override
    protected void onLazyLoad() {
    }

    @Override
    protected void initListener() {
        seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (fromUser) {
                    colorAdjustEffect.setAttribute(currentSetting.name, progressToValue((float)progress / seekBar.getMax()));
                    ActionManager.getInstance().getListener().updateCurrentFrame();
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        findViewById(R.id.tv_bottom_right).setOnClickListener(v -> {
            colorAdjustEffect.resetAttributes();
            setSeekbar(currentSetting);
            ActionManager.getInstance().getListener().updateCurrentFrame();
        });

        findViewById(R.id.iv_submit).setOnClickListener(v -> TrackActionManager.getInstance().hideDetailFragment());

        btnApplyToAll.setOnClickListener(v -> {
            ActionManager.getInstance().getListener().applyColorAdjustToAllMediaTrack(colorAdjustEffect);
            ActionManager.getInstance().getListener().updateCurrentFrame();
        });
    }

    private float progressToValue(float per) {
        Range<Float> range = valueRangeMap.get(currentSetting);
        return range.getLower() + (range.getUpper() - range.getLower()) * per;
    }

    @Override
    public void onActionClick(ActionItem action) {
        switch (action.actionType) {
            case ActionItem.resId:
                switch (action.iconResId) {
                    case R.drawable.icon_liangdu:
                        setSeekbar(SXColorAdjustEffect.SXColorSettings.Brightness);
                        break;
                    case R.drawable.icon_duibidu:
                        setSeekbar(SXColorAdjustEffect.SXColorSettings.Contrast);
                        break;
                    case R.drawable.icon_baohedu:
                        setSeekbar(SXColorAdjustEffect.SXColorSettings.Saturation);
                        break;
                    case R.drawable.icon_ruihua:
                        setSeekbar(SXColorAdjustEffect.SXColorSettings.Sharpen);
                        break;
                    case R.drawable.icon_gaoguang:
                        setSeekbar(SXColorAdjustEffect.SXColorSettings.Highlight);
                        break;
                    case R.drawable.icon_yinying:
                        setSeekbar(SXColorAdjustEffect.SXColorSettings.Shadow);
                        break;
                    case R.drawable.icon_baoguangdu:
                        setSeekbar(SXColorAdjustEffect.SXColorSettings.Exposure);
                        break;
                    case R.drawable.icon_sexiang:
                        setSeekbar(SXColorAdjustEffect.SXColorSettings.Hue);
                        break;
                    default:
                        break;
                }
                break;
            default:
                break;
        }
    }

    private void setSeekbar(SXColorAdjustEffect.SXColorSettings setting) {
        if (colorAdjustEffect == null) return;
        currentSetting = setting;
        Range<Float> range = valueRangeMap.get(setting);
        seekBar.setMax((int) ((range.getUpper() - range.getLower()) * 100));
        float value = colorAdjustEffect.getAttributeFloat(setting.name);
        seekBar.setProgress((int) ((value - range.getLower()) * 100));
        lowerText.setText("" + range.getLower());
        upperText.setText("" + range.getUpper());
    }

    private void showItem1(int resId) {
        itemArrayList = new ArrayList<>();
        itemArrayList.add(new ActionItem("亮度", R.drawable.icon_liangdu, ActionItem.resId));
        itemArrayList.add(new ActionItem("对比度", R.drawable.icon_duibidu, ActionItem.resId));
        itemArrayList.add(new ActionItem("饱和度", R.drawable.icon_baohedu, ActionItem.resId));
        itemArrayList.add(new ActionItem("锐化", R.drawable.icon_ruihua, ActionItem.resId));
        itemArrayList.add(new ActionItem("高光", R.drawable.icon_gaoguang, ActionItem.resId));
        itemArrayList.add(new ActionItem("阴影", R.drawable.icon_yinying, ActionItem.resId));
        itemArrayList.add(new ActionItem("曝光度", R.drawable.icon_baoguangdu, ActionItem.resId));
        itemArrayList.add(new ActionItem("色相", R.drawable.icon_sexiang, ActionItem.resId));

        AdjustAdapter adapter = new AdjustAdapter(this.mActivity);
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);
    }
}
