package com.shixing.edit.audio;

import android.util.Log;

import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.audio.SXAudioThumbnail;
import com.shixing.sxedit.audio.SXAudioThumbnailCache;

import java.util.HashMap;

public class AudioThumbManager {
    private static final String TAG = "AudioThumbManager";
    private final SXAudioThumbnailCache mCache;
    private HashMap<String, SXAudioThumbnail> mThumbs = new HashMap<>();

    private AudioThumbManager() {
        mCache = new SXAudioThumbnailCache(10);
    }

    public static AudioThumbManager getInstance() {
        return InstanceHolder.instance;
    }

    public void addAudio(String path, String audioId) {
        SXAudioThumbnail thumbnail = new SXAudioThumbnail(path, mCache);
        mThumbs.put(audioId, thumbnail);
        if (thumbnail.isValid()) {
            thumbnail.setOnAnalysisProgressChangedListener(progress -> {
                if (thumbnail.isFullyLoaded()) {
                    thumbnail.setOnAnalysisProgressChangedListener(null);
                    int[] thumbData = thumbnail.getThumbData(0, 0, thumbnail.duration());
                    TrackActionManager.getInstance().onAudioThumbChanged(audioId, thumbData);
                }
            });
        }
    }

    public void removeAudio(String audioId) {
        SXAudioThumbnail thumbnail = mThumbs.get(audioId);
        if (thumbnail != null) {
            thumbnail.release();
        }
    }

    private static class InstanceHolder {
         private static AudioThumbManager instance = new AudioThumbManager();
    }
}
