package com.shixing.edit.audio;

import android.Manifest;
import android.content.pm.PackageManager;
import android.media.MediaRecorder;
import android.os.Bundle;
import android.transition.TransitionManager;
import android.util.Log;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.EditActivity;
import com.shixing.edit.R;
import com.shixing.edit.adapter.BackgroundAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.utils.Logger;
import com.shixing.edit.utils.ToastUtil;
import com.shixing.sxedit.internal.AudioItem;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class MusicRecordFragment extends BaseFragment<OnActionClickListener> implements OnActionClickListener {

    private String type;//color/style/blurred
    private ImageButton ib_record;

    private long mStartTime;
    //按下开始录音的灵敏度，单位毫秒
    private final int mPrepareTime=500;
    private int mRecordStatus;//录音状态
    private final int STATUS_START=0;
    private final int STATUS_RECORDING=1;
    private final int STATUS_FINISH=2;

    @Override
    protected int initRootView() {
        return R.layout.fragment_record;
    }

    @Override
    protected void initArguments(Bundle arguments) {
    }

    @Override
    protected void initView() {
        ib_record=findViewById(R.id.ib_record);

    }

    @Override
    protected void onLazyLoad() {
        if (ContextCompat.checkSelfPermission(mActivity, Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED) {
            requestPermissions(new String[]{Manifest.permission.RECORD_AUDIO}, 500);
        }
    }

    @SuppressWarnings("all")
    @Override
    protected void initListener() {
        mRootView.findViewById(R.id.iv_submit).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (listener!=null){
                    ActionItem actionItem=new ActionItem("完成",R.drawable.icon_wancheng);
                    actionItem.id="background_color_selected";
                    listener.onActionClick(actionItem);
                }
            }
        });
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                ToastUtil.showToast(getContext(),"请按住录音按键录音");
            }
        });

        GestureDetector gestureDetector = new GestureDetector(getContext(), new GestureDetector.SimpleOnGestureListener() {
            @Override
            public void onLongPress(MotionEvent e) {
                    startRecord();
            }
        });
        ib_record.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if (event.getActionMasked() == MotionEvent.ACTION_UP) {
                    if (isRecording) {
                        stopRecord();
                    }
                }
                gestureDetector.onTouchEvent(event);
                return false;
            }
        });

    }

    private boolean isRecording = false;
    private MediaRecorder recorder;
    private String outputFile;

    private void startRecord() {
        recorder = new MediaRecorder();
        recorder.setAudioSource(MediaRecorder.AudioSource.MIC);
        recorder.setOutputFormat(MediaRecorder.OutputFormat.AAC_ADTS);
        recorder.setAudioEncoder(MediaRecorder.AudioEncoder.AAC);
        outputFile = getOutputFilePath();
        recorder.setOutputFile(outputFile);
        try {
            recorder.prepare();
            recorder.start();
            isRecording = true;
            Toast.makeText(mActivity, "开始录音", Toast.LENGTH_SHORT).show();
        } catch (IOException e) {
            e.printStackTrace();
            Toast.makeText(mActivity, "录制失败", Toast.LENGTH_SHORT).show();
        }
    }

    private void stopRecord() {
        if (isRecording) {
            isRecording = false;
            try {
                recorder.stop();
                recorder.release();
                AudioItem audioItem = ActionManager.getInstance().getListener().addAudioTrack(outputFile);
                if (audioItem != null) {
                    List<List<AudioItem>> audioGroups = ActionManager.getInstance().getListener().getAudioGroups();
                    TrackActionManager.getInstance().setAudioGroup(audioGroups);
                }
                ((EditActivity) mActivity).hideDetailFragment();
            } catch (Exception e) {
                e.printStackTrace();
                Toast.makeText(mActivity, "录制失败", Toast.LENGTH_SHORT).show();
            }
        }
    }

    private String getOutputFilePath() {
        return mActivity.getExternalCacheDir() + "/" + System.currentTimeMillis() + ".aac";
    }

    @Override
    public void onActionClick(ActionItem action) {
        switch (action.actionType) {
            case ActionItem.resId:
                switch (action.iconResId) {

                    default:
                        break;
                }
                break;
            default:
                break;
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == 500 && grantResults.length == 1) {
            if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {

            } else {
                Toast.makeText(mActivity, "需要录音权限", Toast.LENGTH_SHORT).show();
                ((EditActivity) mActivity).hideDetailFragment();
            }
        } else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }
}
