package com.shixing.edit.background;

import android.os.Bundle;
import android.view.View;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.EditActivity;
import com.shixing.edit.R;
import com.shixing.edit.adapter.BackgroundAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.data.EditData;
import com.shixing.edit.helper.MediaHelper;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;

import java.util.ArrayList;

public class BackgroundFragment extends BaseFragment<OnActionClickListener> implements OnActionClickListener {

    private RecyclerView mItem1_recycler;
    private ArrayList<ActionItem> itemArrayList;
    private String type;//color/style/blurred

    @Override
    protected int initRootView() {
        return R.layout.fragment_background;
    }

    @Override
    protected void initArguments(Bundle arguments) {
        type=arguments.getString("type");
    }

    @Override
    protected void initView() {
        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        showItem1(R.drawable.icon_lvjing);
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        mRootView.findViewById(R.id.iv_submit).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (listener!=null){
                    ActionItem actionItem=new ActionItem("完成",R.drawable.icon_wancheng);
                    actionItem.id="background_color_selected";
                    listener.onActionClick(actionItem);
                }
            }
        });
    }
    @Override
    public void onActionClick(ActionItem action) {
        switch (action.actionType) {
            case ActionItem.resId:
                switch (action.iconResId) {

                    default:
                        break;
                }
                break;
            default:
                break;
        }

        if ("color".equals(type)) {
            applyBgColor(action);
        } else if ("style".equals(type)) {
            onStyleClick(action);
        }
    }

    private void onStyleClick(ActionItem action) {
        switch (action.iconResId) {
            case R.drawable.icon_wu:
                ActionManager.getInstance().getListener().removeBackground();
                ActionManager.getInstance().getListener().updateCurrentFrame();
                break;
            case R.drawable.icon_add:
                MediaHelper.chooseOneVideoOrPicture(mActivity, EditActivity.CODE_SEL_BG_MEDIA);
                TrackActionManager.getInstance().hideDetailFragment();
                break;
        }
    }

    private void applyBgColor(ActionItem action) {
        int color = getResources().getColor(action.iconResId);

        EditData.getInstance().editManager.setBackgroundColor(color);
        EditData.getInstance().editManager.updateCurrentFrame();
    }

    private void showItem1(int resId) {
        itemArrayList = new ArrayList<>();
        switch (type){
            case "color":
                itemArrayList.add(new ActionItem("color0", R.color.bg_color_alpha, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_1, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_2, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_3, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_4, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_5, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_6, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_7, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_8, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_9, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_10, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_11, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_12, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_13, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_14, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_15, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_16, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.color.bg_color_17, ActionItem.resId));
                break;
            case "style":
                itemArrayList.add(new ActionItem("color0", R.drawable.icon_wu, ActionItem.resId));
                itemArrayList.add(new ActionItem("color1", R.drawable.icon_add, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_2, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_3, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_4, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_5, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_6, ActionItem.resId));
                break;
            case "blurred":
                itemArrayList.add(new ActionItem("color0", R.drawable.icon_wu, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_1, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_2, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_3, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_4, ActionItem.resId));
//                itemArrayList.add(new ActionItem("color1", R.color.bg_color_5, ActionItem.resId));
                break;
            default:
                break;
        }


        BackgroundAdapter adapter = new BackgroundAdapter(this.mActivity,type);
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);
    }
}
