package com.shixing.edit.clip;

import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.R;
import com.shixing.edit.adapter.FilterAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.data.AnimationModel;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.utils.ToastUtil;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.AnimationEffect;
import com.shixing.sxedit.internal.MediaTrack;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AnimaFragment extends BaseFragment implements OnActionClickListener {

    private static Map<String, AnimationModel> animations = new HashMap<>();

    private RecyclerView mItem1_recycler;
    private ArrayList<ActionItem> itemArrayList;

    private static final int ID_VIDEO_ANIM_1 = 101;
    private static final int ID_VIDEO_ANIM_2 = 102;
    private MediaTrack track;
    private AnimationEffect animationEffect;
    private SeekBar seekBar;
    private View seekBarGroup;

    private @AnimationModel.Type int mAnimType;
    private String mFolder;
    private String mNamePrefix;
    private TextView mTvStart;
    private TextView mTvEnd;

    private AnimaFragment() {
    }

    public static AnimaFragment newInstance(int animType) {
        AnimaFragment fragment = new AnimaFragment();
        Bundle bundle = new Bundle();
        bundle.putInt("type", animType);
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    protected int initRootView() {
        return R.layout.fragment_animation;
    }

    @Override
    protected void initArguments(Bundle arguments) {
        mAnimType = arguments.getInt("type", 0);

        if (mAnimType == AnimationModel.ENTER) {
            mFolder = "入场动画";
            mNamePrefix = "ve";
        } else if (mAnimType == AnimationModel.EXIT) {
            mFolder = "出场动画";
            mNamePrefix = "vx";
        } else if (mAnimType == AnimationModel.LOOP) {
            mFolder = "循环动画";
            mNamePrefix = "vl";
        }
    }

    @Override
    protected void initView() {
        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        seekBar = mRootView.findViewById(R.id.seekBar);
        seekBarGroup = findViewById(R.id.seekbar_group);
        mTvStart = (TextView) findViewById(R.id.tv_seekbar_start);
        mTvEnd = (TextView) findViewById(R.id.tv_seekbar_end);

        track = (MediaTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        showItem1(R.drawable.icon_lvjing);

        AnimationModel animationModel = animations.get(track.getTrackId());
        if (animationModel != null) {
            List<AnimationEffect> animations = track.getAnimations();
            if (animations != null && animations.size() > 0) {
                animationEffect = animations.get(0);

                if (animationModel.type == mAnimType) {
                    seekBarGroup.setVisibility(View.VISIBLE);
                    seekBar.setProgress((int) (animationEffect.duration() / track.duration() * 100));
                }
            }
        }
        mTvStart.setText(String.format("%.1fs", 0f));
        mTvEnd.setText(String.format("%.1fs", track.duration()));
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        findViewById(R.id.iv_submit).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                TrackActionManager.getInstance().hideDetailFragment();
            }
        });

        seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (fromUser) {
                    if (animationEffect != null) {
                        double duration = (double) progress / seekBar.getMax() * track.duration();
                        animationEffect.setDuration(duration);
                        if (mAnimType != AnimationModel.LOOP) {
                            animationEffect.setDurationOfOneCycle(duration);
                        }
                        TrackActionManager.getInstance().updateTrackAnimation();
                    }
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });
    }

    @Override
    public void onActionClick(ActionItem action) {
        removeAnim();
        if (TextUtils.isEmpty(action.srcPath)) {
            TrackActionManager.getInstance().updateTrackAnimation();
            seekBarGroup.setVisibility(View.GONE);
            return;
        }

        seekBarGroup.setVisibility(View.VISIBLE);
        animationEffect = track.addAnimation(action.actionType);
        if (mAnimType == AnimationModel.LOOP) {
            animationEffect.setDuration(track.duration());
        }
        seekBar.setProgress((int) (animationEffect.duration() / track.duration() * 100));

        if (mAnimType == AnimationModel.EXIT) {
            animationEffect.setEffectFollowType(SXEffect.SXEffectTimeFollowType.FollowEnd);
        } else {
            animationEffect.setEffectFollowType(SXEffect.SXEffectTimeFollowType.FollowStart);
        }

        AnimationModel model = new AnimationModel();
        model.id = animationEffect.getEffectId();
        model.type = mAnimType;
        model.resName = action.actionName;
        animations.put(track.getTrackId(), model);

        TrackActionManager.getInstance().updateTrackAnimation();
    }

    private void removeAnim() {
        if (animationEffect != null) {
            track.removeAnimation(animationEffect.getEffectId());
            animationEffect = null;
        }
    }

    private void showItem1(int resId) {
        String assetPath = Constants.assetsPath + "/视频动画/" + mFolder;
        String resPath = Constants.resourcePath + "/视频动画/" + mFolder;

        itemArrayList = new ArrayList<>();
        itemArrayList.add(new ActionItem("无", "", ""));
        itemArrayList.add(new ActionItem("动画1", assetPath + "/" + mNamePrefix + "1.gif", resPath + "/" + mNamePrefix + "1"));
        itemArrayList.add(new ActionItem("动画2", assetPath + "/" + mNamePrefix + "2.gif", resPath + "/" + mNamePrefix + "2"));
        itemArrayList.add(new ActionItem("动画3", assetPath + "/" + mNamePrefix + "3.gif", resPath + "/" + mNamePrefix + "3"));
        itemArrayList.add(new ActionItem("动画4", assetPath + "/" + mNamePrefix + "4.gif", resPath + "/" + mNamePrefix + "4"));
        itemArrayList.add(new ActionItem("动画5", assetPath + "/" + mNamePrefix + "5.gif", resPath + "/" + mNamePrefix + "5"));

        FilterAdapter adapter = new FilterAdapter(this.mActivity);
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);

        AnimationModel animationModel = animations.get(track.getTrackId());
        if (animationModel != null && animationModel.type == mAnimType) {
            adapter.setSelectedItemName(animationModel.resName);
        }
    }

    public static AnimationModel getAnimation(String trackId) {
        return animations.get(trackId);
    }
}
