package com.shixing.edit.clip;

import android.os.Bundle;
import android.view.View;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.EditActivity;
import com.shixing.edit.R;
import com.shixing.edit.adapter.BlendAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.track.SXTrack;

import java.util.ArrayList;
import java.util.List;

public class BlendFragment extends BaseFragment {

    private View btnSubmit;
    private RecyclerView listBlend;
    private BlendAdapter blendAdapter;
    private MediaTrack track;

    @Override
    protected int initRootView() {
        return R.layout.fragment_blend;
    }

    @Override
    protected void initArguments(Bundle arguments) {

    }

    @Override
    protected void initView() {
        btnSubmit = findViewById(R.id.iv_submit);
        listBlend = (RecyclerView) findViewById(R.id.list_blend);

        track = (MediaTrack) TrackActionManager.getInstance().getCurrentSelectTrack();

        blendAdapter = new BlendAdapter();
        blendAdapter.setBlendModes(getBlendList());
        blendAdapter.setSelectedMode(track.blendMode());
        listBlend.setLayoutManager(new LinearLayoutManager(mActivity, RecyclerView.HORIZONTAL, false));
        listBlend.setAdapter(blendAdapter);
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        btnSubmit.setOnClickListener(v -> ((EditActivity) mActivity).hideDetailFragment());
        blendAdapter.setOnSelectedListener(blendMod -> {
            track.setBlendMode(blendMod);
            ActionManager.getInstance().getListener().updateCurrentFrame();
        });
    }

    private List<BlendAdapter.BlendData> getBlendList() {
        String thumbUrlPrefix = Constants.assetsPath + "/blend/";
        ArrayList<BlendAdapter.BlendData> blendData = new ArrayList<>();
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_normal), thumbUrlPrefix + "Normal.png", SXTrack.SXBlendMode.Normal));
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_lighten), thumbUrlPrefix + "Lighten.png", SXTrack.SXBlendMode.Lighten));
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_darken), thumbUrlPrefix + "Darken.png", SXTrack.SXBlendMode.Darken));
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_multiply), thumbUrlPrefix + "Multiply.png", SXTrack.SXBlendMode.Multiply));
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_average), thumbUrlPrefix + "Average.png", SXTrack.SXBlendMode.Average));
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_add), thumbUrlPrefix + "Add.png", SXTrack.SXBlendMode.Add));
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_subtract), thumbUrlPrefix + "Subtract.png", SXTrack.SXBlendMode.Subtract));
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_difference), thumbUrlPrefix + "Difference.png", SXTrack.SXBlendMode.Difference));
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_screen), thumbUrlPrefix + "Screen.png", SXTrack.SXBlendMode.Screen));
        blendData.add(new BlendAdapter.BlendData(getString(R.string.blend_overlay), thumbUrlPrefix + "Overlay.png", SXTrack.SXBlendMode.Overlay));
        return blendData;
    }
}
