package com.shixing.edit.clip;


import android.graphics.Bitmap;
import android.media.MediaMetadataRetriever;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import androidx.fragment.app.Fragment;

import com.shixing.edit.EditActivity;
import com.shixing.edit.R;

import com.shixing.edit.adjust.AdjustFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.data.AnimationModel;
import com.shixing.edit.effects.TrackEffectsFragment;
import com.shixing.edit.filter.MediaFilterFragment;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.multitrack.TrackUtil;
import com.shixing.edit.utils.ToastUtil;
import com.shixing.edit.widget.ReversingDialog;
import com.shixing.edit.widget.SurfaceOverlay;
import com.shixing.sxedit.internal.EditManager;
import com.shixing.sxedit.internal.MainTrackGroup;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;
import com.shixing.sxedit.tools.SXReverseCompositor;
import com.shixing.sxedit.track.SXTrack;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/*** 剪辑子列表 */
public class ClipMenu  implements OnActionClickListener {

    private  ArrayList<ActionItem> mEditActions;
    private ArrayList<ActionItem> mChannelEditActions;
    private ArrayList<ActionItem> mAnimActions;

    private ReversingDialog mReversingDialog;

    private EditActivity mActivity;
    private static ClipMenu instance;
    private ActionItem playbackItem;

    private ClipMenu(EditActivity editActivity){
        this.mActivity=editActivity;
    }
    public static ClipMenu getInstance(EditActivity editActivity){
        if (instance==null)
            instance= new ClipMenu(editActivity);
        return instance;
    }
    public  ArrayList<ActionItem> getActionList(){
        Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
        boolean staticImage = ((MediaTrack) track).isStaticImage();
        boolean inMainGroup = ((MediaTrack) track).inMainTrackGroup();

        mEditActions = new ArrayList<>();
        mEditActions.add(new ActionItem("编辑", R.drawable.icon_bianji));
        if (!staticImage) {
            mEditActions.add(new ActionItem("变速", R.drawable.icon_biansu));
            mEditActions.add(new ActionItem("音量", R.drawable.icon_yinliang));
        }
        mEditActions.add(new ActionItem("滤镜", R.drawable.icon_lvjing));
        mEditActions.add(new ActionItem("动画", R.drawable.icon_donghua));
        mEditActions.add(new ActionItem("删除", R.drawable.icon_shanchu));
        mEditActions.add(new ActionItem("蒙版", R.drawable.icon_mengban));
        mEditActions.add(new ActionItem("复制", R.drawable.icon_fuzhi));
        mEditActions.add(new ActionItem("调节", R.drawable.icon_tiaojie));
        mEditActions.add(new ActionItem("分割", R.drawable.icon_fenge));
        if (!inMainGroup) {
            mEditActions.add(new ActionItem("混合模式", R.drawable.icon_hunhemoshi));
            mEditActions.add(new ActionItem("层级", R.drawable.icon_cengji));
        } else {
//            mEditActions.add(new ActionItem("左"));
//            mEditActions.add(new ActionItem("右"));
        }
        mEditActions.add(new ActionItem("特效", R.drawable.icon_texiao));

        TrackReverseState trackReverseState = sReverseStateMap.get(track.getTrackId());
        if (trackReverseState != null && trackReverseState.mReversing) {
            playbackItem = new ActionItem("取消倒放", R.drawable.icon_daofang);
        } else {
            playbackItem = new ActionItem("倒放", R.drawable.icon_daofang);
        }
        mEditActions.add(playbackItem);

        return mEditActions;
    }

    public  void onActionClick(ActionItem action) {

            switch (action.iconResId) {
            case R.drawable.icon_bianji:
                showBianji();
                break;
            case R.drawable.icon_donghua:
//                showFragment(new VideoAnimFragment());
//                showFragment(new AnimaFragment());
                showAnimSubMenu();
                break;
            case R.drawable.icon_yinliang:
                if (TrackActionManager.getInstance().getCurrentSelectTrack()!=null){
                    VolumeFragment fragment=new VolumeFragment();
                    Bundle bundle=new Bundle();
                    bundle.putSerializable("track",TrackActionManager.getInstance().getCurrentSelectTrack());
                    fragment.setArguments(bundle);
                    showFragment(fragment);
                }
                break;
            case R.drawable.icon_lvjing:
                showFragment(new MediaFilterFragment());
                break;
            case R.drawable.icon_mengban: {
                Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
                mActivity.overlay.showMask(track);
                mActivity.overlay.setCurrentMod(SurfaceOverlay.MOD_MASK);
                showFragment(new MaskFragment());
            }
                break;
            case R.drawable.icon_tiaojie:
                showFragment(new AdjustFragment());
                break;
            case R.drawable.icon_biansu:
                showBiansu();
                break;

            case R.drawable.icon_jingxiang:
                //发送镜像指令给引擎
            {
                ToastUtil.showToast(mActivity, "镜像");

                Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
                ActionManager.getInstance().getListener().horizontalFlip(track);
            }
                break;
            case R.drawable.icon_xuanzhuan: {
                //发送旋转指令给引擎
                Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
                ActionManager.getInstance().getListener().rotate(track);
            }
                break;
            case R.drawable.icon_jiancai:
                //发送剪裁指令给引擎
                ToastUtil.showToast(mActivity,"剪裁");
                showCropFragment();
                break;
            case R.drawable.icon_butouming:
                //发送不透明指令给引擎
                ToastUtil.showToast(mActivity,"不透明");
                showFragment(new OpacityFragment());
                break;
            case R.drawable.icon_changgui:
                //发送常规变速指令给引擎
                showFragment(new ChangeSpeedFragment());
                break;
            case R.drawable.icon_quxian:
                showFragment(new CurveSpeedFragment());
                break;
            case R.drawable.icon_dingge:
                //发送定格指令给引擎
                ToastUtil.showToast(mActivity,"定格");//罗彩
            {
                Track track=TrackActionManager.getInstance().getCurrentSelectTrack();
//                ActionManager.getInstance().getListener().freeze((MediaTrack) track);

                ActionListener listener = ActionManager.getInstance().getListener();
                EditManager editManager = listener.getEditManager();
                double currentTime = listener.getCurrentTime();

//                Track newTrack = editManager.cloneTrack(track.getTrackId());
                listener.splitTrack(track);
                MainTrackGroup group = editManager.mainGroup();
//                group.addTrack(newTrack, currentTime);
//                group.setTrackInPoint(newTrack.getTrackId(), currentTime);
//                group.setTrackDuration(newTrack.getTrackId(), 3);
//                ((MediaTrack) newTrack).freeze(currentTime);


                String filePath = ((MediaTrack) track).getFilePath();
                MediaMetadataRetriever retriever = new MediaMetadataRetriever();
                retriever.setDataSource(filePath);

                Bitmap bitmap = retriever.getFrameAtTime((long) ((currentTime - track.startTime()) * 1000000), MediaMetadataRetriever.OPTION_CLOSEST);
                File dir = mActivity.getExternalFilesDir("dingge");
                String path = dir + "/" + System.currentTimeMillis() + ".png";
                TrackUtil.saveBitmapToPng(bitmap, path);
                listener.addMainTrack(path);

                TrackActionManager.getInstance().addMainTrack(listener.getMainTracks());
//                TrackActionManager.getInstance().onTrackSelect(newTrack, true);

            }
                break;
            case R.drawable.icon_shanchu: //删除
                Track track1=TrackActionManager.getInstance().getCurrentSelectTrack();
                if (track1!=null&&track1 instanceof MediaTrack){
                    MediaTrack mediaTrack=(MediaTrack) track1;
                    if (mediaTrack.inMainTrackGroup()) {
                        if (ActionManager.getInstance().getListener().getMainTracks().size() <= 1) {
                            ToastUtil.showToast(mActivity, "最后一个不能删除");
                        } else {
                            deleteCurrentTrack();
                        }
                    }else {
                        deleteCurrentTrack();
                    }
                }


                break;
            case R.drawable.icon_fuzhi: //复制
                copyCurrentTrack();
                break;
            case R.drawable.icon_fenge:
                splitCurrentTrack();
                break;

            case R.drawable.icon_ruchangdonghua:
                showFragment(AnimaFragment.newInstance(AnimationModel.ENTER));
                break;
            case R.drawable.icon_chuchangdonghua:
                showFragment(AnimaFragment.newInstance(AnimationModel.EXIT));
                break;
            case R.drawable.icon_xunhuandonghua:
                showFragment(AnimaFragment.newInstance(AnimationModel.LOOP));
                break;
            case R.drawable.icon_hunhemoshi:
                showFragment(new BlendFragment());
                    break;
            case R.drawable.icon_cengji:
                showFragment(new PIPLevelFragment());
                break;
            case R.drawable.icon_texiao:
                showFragment(new TrackEffectsFragment());
                break;
            case R.drawable.icon_daofang: {
                MediaTrack track = (MediaTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
                EditManager editManager = ActionManager.getInstance().getListener().getEditManager();
                TrackGroup group = editManager.group(track.getGroupId());
                double duration = track.duration();
                double inPoint = track.inPoint();
                double startTime = track.startTime();
                double newStartTime = inPoint - (startTime + track.resourceDuration() - (inPoint + duration));

                TrackReverseState trackReverseState = sReverseStateMap.get(track.getTrackId());
                if (trackReverseState != null && trackReverseState.mReversing) {
                    track.setResource(trackReverseState.mPath);
                    group.setTrackTime(track, newStartTime, inPoint, duration);

                    trackReverseState.mReversing = false;
                    ActionManager.getInstance().getListener().updateCurrentFrame();
                    playbackItem.actionName = "倒放";
                    mActivity.subMenuItemChanged();
                    TrackActionManager.getInstance().addMainTrack(ActionManager.getInstance().getListener().getMainTracks());
                } else {
                    if (trackReverseState == null) {
                        trackReverseState = new TrackReverseState();
                        sReverseStateMap.put(track.getTrackId(), trackReverseState);
                        trackReverseState.mPath = track.getFilePath();
                    }

                    if (trackReverseState.mReversePath == null) {
                        File reverseDir = mActivity.getExternalFilesDir("reverse");
                        String reversePath = reverseDir + "/" + UUID.randomUUID() + ".mp4";
                        trackReverseState.mReversePath = reversePath;

                        TrackReverseState finalTrackReverseState = trackReverseState;

                        mReversingDialog = new ReversingDialog();
                        mReversingDialog.show(mActivity.getSupportFragmentManager(), "reversing");
                        runReverseTask(trackReverseState.mPath, reversePath, new Runnable() {
                            @Override
                            public void run() {
                                Log.d("TAG", "reverse finish " + reversePath);
                                finalTrackReverseState.mReversing = true;
                                track.setResource(reversePath);
                                group.setTrackTime(track, newStartTime, inPoint, duration);
                                playbackItem.actionName = "取消倒放";
                                mActivity.subMenuItemChanged();
                                ActionManager.getInstance().getListener().updateCurrentFrame();
                                TrackActionManager.getInstance().addMainTrack(ActionManager.getInstance().getListener().getMainTracks());
                                mReversingDialog.dismiss();
                            }
                        });
                    } else {
                        trackReverseState.mReversing = true;
                        track.setResource(trackReverseState.mReversePath);
                        group.setTrackTime(track, newStartTime, inPoint, duration);
                        playbackItem.actionName = "取消倒放";
                        mActivity.subMenuItemChanged();
                        ActionManager.getInstance().getListener().updateCurrentFrame();
                        TrackActionManager.getInstance().addMainTrack(ActionManager.getInstance().getListener().getMainTracks());
                    }
                }
            }
            break;
            default:
                EditManager editManager = ActionManager.getInstance().getListener().getEditManager();
                MainTrackGroup mainGroup = editManager.mainGroup();
                Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
                if (action.actionName.equals("左")) {
                    ToastUtil.showToast(mActivity, "左");
                    Track lastTrack = mainGroup.findLastTrack(track.getTrackId());
                    if (lastTrack != null) {
                        mainGroup.offsetTrack(track.getTrackId(), lastTrack.inPoint());
                    }
                } else if (action.actionName.equals("右")) {
                    ToastUtil.showToast(mActivity, "右");
                    Track nextTrack = mainGroup.findNextTrack(track.getTrackId());
                    if (nextTrack != null) {
                        mainGroup.offsetTrack(track.getTrackId(), nextTrack.inPoint() + nextTrack.duration());
                    }
                }
                List<MediaTrack> mainTracks = ActionManager.getInstance().getListener().getMainTracks();
                TrackActionManager.getInstance().addMainTrack(mainTracks);

                ActionManager.getInstance().getListener().updateCurrentFrame();
                break;
        }
    }

    private void runReverseTask(String path, String reversePath, Runnable callback) {
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    long start = System.currentTimeMillis();
                    SXReverseCompositor.reverseVideo(path, reversePath);
                    Log.d("TEST", "reverse cost " + (System.currentTimeMillis() - start) + " ms");
                } catch (IOException e) {
                    e.printStackTrace();
                }
                mActivity.runOnUiThread(callback::run);
            }
        });
        thread.setPriority(Thread.MIN_PRIORITY);
        thread.start();
    }

    private void deleteCurrentTrack() {
        Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
        if (((MediaTrack) track).inMainTrackGroup()) {
            ActionManager.getInstance().getListener().removeTrack(track);
            TrackActionManager.getInstance().cancelSelectTrackByClick();
            TrackActionManager.getInstance().addMainTrack(ActionManager.getInstance().getListener().getMainTracks());
        } else {
            ActionManager.getInstance().getListener().removeTrack(track);
            TrackActionManager.getInstance().cancelSelectTrackByClick();
            TrackActionManager.getInstance().addTrackGroup(ActionManager.getInstance().getListener().getPIPGroups(), SXTrack.SX_TRACK_TYPE_MEDIA);
        }
    }

    private void copyCurrentTrack() {
        MediaTrack track = (MediaTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        if (track.inMainTrackGroup()) {
            ActionManager.getInstance().getListener().copyMainTrack(track);
            TrackActionManager.getInstance().addMainTrack(ActionManager.getInstance().getListener().getMainTracks());
        } else {
            Track newTrack = ActionManager.getInstance().getListener().copyTrack(track);
            if (newTrack != null) {
                TrackActionManager.getInstance().addTrackGroup(ActionManager.getInstance().getListener().getPIPGroups(), SXTrack.SX_TRACK_TYPE_MEDIA);
                TrackActionManager.getInstance().onTrackSelect(newTrack, true);
            }
        }
    }

    private void splitCurrentTrack() {
        MediaTrack track = (MediaTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        if (track.inMainTrackGroup()) {
            Track newTrack = ActionManager.getInstance().getListener().splitTrack(track);
            TrackActionManager.getInstance().addMainTrack(ActionManager.getInstance().getListener().getMainTracks());
        } else {
            Track newTrack = ActionManager.getInstance().getListener().splitTrack(track);
            TrackActionManager.getInstance().addTrackGroup(ActionManager.getInstance().getListener().getPIPGroups(), SXTrack.SX_TRACK_TYPE_MEDIA);
            if (newTrack != null) {
                TrackActionManager.getInstance().onTrackSelect(newTrack, true);
            }
        }
    }


    private void showBianji(){

        mChannelEditActions = new ArrayList<>();
        mChannelEditActions.add(new ActionItem("镜像", R.drawable.icon_jingxiang));
        mChannelEditActions.add(new ActionItem("旋转", R.drawable.icon_xuanzhuan));
        mChannelEditActions.add(new ActionItem("剪裁", R.drawable.icon_jiancai));
        mChannelEditActions.add(new ActionItem("不透明", R.drawable.icon_butouming));
        mActivity.showSubMenu(mChannelEditActions,this,"Bianji",1);
    }

    private void showAnimSubMenu() {
        if (mAnimActions == null) {
            mAnimActions = new ArrayList<>();
            mAnimActions.add(new ActionItem("入场动画", R.drawable.icon_ruchangdonghua));
            mAnimActions.add(new ActionItem("出场动画", R.drawable.icon_chuchangdonghua));
            mAnimActions.add(new ActionItem("循环动画", R.drawable.icon_xunhuandonghua));
        }

        mActivity.showSubMenu(mAnimActions, this, "Anim", 1);
        TrackActionManager.getInstance().showTrackAnimationMask(true);
    }

    private void showBiansu(){
        Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
        boolean staticImage = ((MediaTrack) track).isStaticImage();

        mChannelEditActions = new ArrayList<>();
        mChannelEditActions.add(new ActionItem("常规变速", R.drawable.icon_changgui));
        mChannelEditActions.add(new ActionItem("曲线变速", R.drawable.icon_quxian));
        if (!staticImage) {
//            mChannelEditActions.add(new ActionItem("定格", R.drawable.icon_dingge));
        }

        mActivity.showSubMenu(mChannelEditActions,this,"Biansu",1);
    }

    private void showFragment(Fragment fragment){
        mActivity.mMenu_container.setVisibility(View.VISIBLE);
        mActivity.getSupportFragmentManager().beginTransaction()
                .replace(R.id.menu_container, fragment)
                .commit();
        mActivity.getSupportFragmentManager().beginTransaction().show(fragment);
    }
    private void showCropFragment(){
//        CropFragment fragment=new CropFragment();
        MyUCropFragment fragment=new MyUCropFragment();
        Bundle bundle=new Bundle();
        bundle.putSerializable("track",TrackActionManager.getInstance().getCurrentSelectTrack());
        fragment.setArguments(bundle);
        mActivity.crop_container.setVisibility(View.VISIBLE);
        mActivity.getSupportFragmentManager().beginTransaction()
                .replace(R.id.crop_container, fragment)
                .commit();
        mActivity.getSupportFragmentManager().beginTransaction().show(fragment);
    }

    private static class TrackReverseState {
        private String mPath;
        private String mReversePath;
        private boolean mReversing;
    }

    private static final Map<String, TrackReverseState> sReverseStateMap = new HashMap<>();
}
