package com.shixing.edit.clip;

import android.graphics.PointF;
import android.graphics.RectF;
import android.os.Bundle;
import android.util.Log;
import android.widget.TextView;

import com.shixing.edit.EditActivity;
import com.shixing.edit.R;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.widget.CurveSpeedView;
import com.shixing.sxedit.SXShape;
import com.shixing.sxedit.internal.EditManager;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.TrackGroup;

import java.util.List;

public class CurveSpeedFragment extends BaseFragment {
    private static final String TAG = "CurveSpeedFragment";
    private SXShape shape;

    private CurveSpeedView curveSpeedView;
    private TextView btnAddPoint;
    private MediaTrack mediaTrack;
    private EditManager editManager;
    private TrackGroup group;

    @Override
    protected int initRootView() {
        return R.layout.panel_curve_speed;
    }

    @Override
    protected void initArguments(Bundle arguments) {

    }

    @Override
    protected void initView() {
        shape = new SXShape();
        curveSpeedView = (CurveSpeedView) findViewById(R.id.curve_speed_view);
        btnAddPoint = (TextView) findViewById(R.id.add_point);

        mediaTrack = (MediaTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        editManager = ActionManager.getInstance().getListener().getEditManager();
        group = editManager.group(mediaTrack.getGroupId());
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        findViewById(R.id.tv_bottom_left).setOnClickListener(v -> curveSpeedView.reset());
        findViewById(R.id.iv_submit_edit).setOnClickListener(v -> {
            TrackActionManager.getInstance().addMainTrack(ActionManager.getInstance().getListener().getMainTracks());
            ((EditActivity) mActivity).hideDetailFragment();
        });

        curveSpeedView.setActionListener(new CurveSpeedView.ActionListener() {
            @Override
            public void onPathChanged(List<CurveSpeedView.ControlPoint> points) {
                shape.clear();
                RectF frame = curveSpeedView.getFrame();

                CurveSpeedView.ControlPoint point = points.get(0);

                PointF lastP = convert(frame, point);
                PointF p;
                shape.moveTo(lastP.x, lastP.y);
                for (int i = 1; i < points.size(); i++) {
                    point = points.get(i);

                    p = convert(frame, point);
                    shape.bezierTo(lastP.x + (p.x - lastP.x) / 2, lastP.y, p.x - (p.x - lastP.x) / 2, p.y, p.x, p.y);

                    lastP = p;
                }

                boolean b = group.setMediaTrackSpeed(mediaTrack, shape);
            }

            @Override
            public void onSeek(double time) {

            }

            @Override
            public void onSelect(boolean select, boolean canBeRemove) {
                btnAddPoint.setText(select ? "删除点" : "添加点");
                btnAddPoint.setEnabled(!select || canBeRemove);
            }
        });

        btnAddPoint.setOnClickListener(v -> {

        });
    }

    private PointF convert(RectF frame, CurveSpeedView.ControlPoint point) {
        float halfHeight = frame.height() / 2;
        float midY = halfHeight + frame.top;

        PointF pointF = new PointF();
        if (point.getY() <= frame.top + frame.height() / 2) {
            pointF.y = 1 +  (midY - point.getY()) / halfHeight * 9;
        } else {
            pointF.y = 0.1f + (frame.bottom - point.getY()) / halfHeight * 0.9f;
        }
        pointF.x = point.getX() - frame.left;
        return pointF;
    }
}
