package com.shixing.edit.clip;

import android.graphics.PointF;
import android.os.Bundle;
import android.view.View;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.EditActivity;
import com.shixing.edit.R;
import com.shixing.edit.adapter.MaskAdapter;
import com.shixing.edit.adapter.TransitionItemAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.utils.ToastUtil;
import com.shixing.sxedit.SXShape;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Shape;
import com.shixing.sxedit.internal.Track;

import java.util.ArrayList;

public class MaskFragment extends BaseFragment implements OnActionClickListener {

    private RecyclerView mItem1_recycler;
    private ArrayList<ActionItem> itemArrayList;

    @Override
    protected int initRootView() {
        return R.layout.fragment_mask;
    }

    @Override
    protected void initArguments(Bundle arguments) {

    }

    @Override
    protected void initView() {
        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        showItem1(R.drawable.icon_lvjing);
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });
        findViewById(R.id.tv_bottom_right).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
                track.setInverseMaskShape(!track.isInverseMaskShape());
                ActionManager.getInstance().getListener().updateCurrentFrame();
            }
        });

        findViewById(R.id.iv_submit).setOnClickListener(v -> {
            TrackActionManager.getInstance().hideDetailFragment();
        });
    }
    @Override
    public void onActionClick(ActionItem action) {
        Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
        int[] trackSize = track.getTrackSize();
        int width = trackSize[0];
        int height = trackSize[1];
        float centerX = width / 2f;
        float centerY = height / 2f;
        switch (action.actionType) {
            case ActionItem.resId:
                switch (action.iconResId) {
                    case R.drawable.icon_wu: {
                        track.setMaskShape((SXShape) null);
                    }
                        break;
                    case R.drawable.icon_yuanxing: {
                        SXShape shape = new SXShape();
                        shape.circle(centerX, centerY, Math.min(width, height) * 0.4f);
                        track.setMaskShape(shape);
                    }
                        break;
                    case R.drawable.icon_juxing: {
                        float rectWidth = width * 0.8f;
                        float rectHeight = rectWidth * 0.6f;
                        SXShape shape = new SXShape();
                        shape.rect(centerX - rectWidth / 2, centerY - rectHeight / 2, rectWidth, rectHeight);
                        track.setMaskShape(shape);
                        break;
                    }
                    case R.drawable.icon_duobianxing: {
                        float l = width * 0.4f / 2;
                        SXShape shape = new SXShape();
                        shape.moveTo(centerX + 2 * l, centerY);
                        shape.lineTo(centerX + l, centerY + (float) (Math.sqrt(3)) * l);
                        shape.lineTo(centerX - l, centerY + (float) (Math.sqrt(3)) * l);
                        shape.lineTo(centerX - 2 * l, centerY);
                        shape.lineTo(centerX - l, centerY - (float) (Math.sqrt(3)) * l);
                        shape.lineTo(centerX + l, centerY - (float) (Math.sqrt(3)) * l);
                        shape.closePath();
                        track.setMaskShape(shape);
                        break;
                    }
                    case R.drawable.icon_xing: {
                        SXShape shape = new SXShape();

                        PointF[] points = pentagram(width * 0.45f, centerX, centerY, 0);
                        shape.moveTo(points[0].x, points[0].y);
                        for (int i = 1; i < points.length; i++) {
                            shape.lineTo(points[i].x, points[i].y);
                        }
                        shape.closePath();

                        track.setMaskShape(shape);
                        break;
                    }
                    default:
                        break;
                }
                break;
            default:
                break;
        }
        ActionManager.getInstance().getListener().updateCurrentFrame();
        ((EditActivity) mActivity).overlay.invalidate();
    }

    PointF[] pentagram(float R, float x, float y, float yDegree) {
        float rad = (float) Math.PI / 180;
        float r = R * (float) (Math.sin(18 * rad) / Math.cos(36 * rad));
        PointF[] RPoints = new PointF[5];
        PointF[] rPoints = new PointF[5];
        for (int k = 0; k < 5; k++) {
            RPoints[k] = new PointF(x - (R * cos((90 + k * 72 + yDegree) * rad)), y - (R * sin((90 + k * 72 + yDegree) * rad)));
            rPoints[k] = new PointF(x - (r * cos((90 + 36 + k * 72 + yDegree) * rad)), y - (r * sin((90 + 36 + k * 72 + yDegree) * rad)));
        }

        PointF[] points = new PointF[10];
        for (int i = 0; i < 5; i++) {
            points[2 * i] = RPoints[i];
            points[2 * i + 1] = rPoints[i];
        }
        return points;
    }

    float sin(float a) {
        return (float) Math.sin(a);
    }

    float cos(float a) {
        return (float) Math.cos(a);
    }

    private void showItem1(int resId) {
        itemArrayList = new ArrayList<>();
        itemArrayList.add(new ActionItem("无", R.drawable.icon_wu, ActionItem.resId));
        itemArrayList.add(new ActionItem("圆形", R.drawable.icon_yuanxing, ActionItem.resId));
        itemArrayList.add(new ActionItem("矩形", R.drawable.icon_juxing, ActionItem.resId));
        itemArrayList.add(new ActionItem("多边形", R.drawable.icon_duobianxing, ActionItem.resId));
        itemArrayList.add(new ActionItem("星型", R.drawable.icon_xing, ActionItem.resId));
//        itemArrayList.add(new ActionItem("叠化", resId, ActionItem.resId));
//        itemArrayList.add(new ActionItem("叠化", resId, ActionItem.resId));
//        itemArrayList.add(new ActionItem("叠化", resId, ActionItem.resId));

        MaskAdapter adapter = new MaskAdapter(this.mActivity);
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);
    }
}
