package com.shixing.edit.clip;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.graphics.PorterDuff;
import android.graphics.RectF;
import android.media.MediaMetadataRetriever;
import android.net.Uri;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AccelerateInterpolator;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.annotation.ColorInt;
import androidx.annotation.IdRes;
import androidx.annotation.IntDef;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDelegate;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.transition.AutoTransition;
import androidx.transition.Transition;
import androidx.transition.TransitionManager;

import com.shixing.edit.R;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.utils.AppExecutors;
import com.shixing.edit.utils.Logger;
import com.shixing.edit.utils.Utils;
import com.shixing.edit.widget.CropView;
import com.shixing.edit.widget.CustomSeekBar;
import com.shixing.edit.widget.TestView;
import com.shixing.sxedit.internal.AnimationEffect;
import com.shixing.sxedit.internal.MediaTrack;
import com.yalantis.ucrop.UCrop;
import com.yalantis.ucrop.UCropActivity;

import com.yalantis.ucrop.callback.BitmapCropCallback;
import com.yalantis.ucrop.model.AspectRatio;
import com.yalantis.ucrop.util.SelectedStateListDrawable;
import com.yalantis.ucrop.view.CropImageView;
import com.yalantis.ucrop.view.GestureCropImageView;
import com.yalantis.ucrop.view.OverlayView;
import com.yalantis.ucrop.view.TransformImageView;
import com.yalantis.ucrop.view.UCropView;
import com.yalantis.ucrop.view.widget.AspectRatioTextView;
import com.yalantis.ucrop.view.widget.HorizontalProgressWheelView;

import java.io.File;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import static android.app.Activity.RESULT_OK;
import static com.yalantis.ucrop.UCrop.EXTRA_INPUT_URI;
import static com.yalantis.ucrop.UCrop.EXTRA_OUTPUT_URI;
import static com.yalantis.ucrop.UCrop.of;

@SuppressWarnings("ConstantConditions")
public class MyUCropFragment extends Fragment implements UCropFragmentCallback, View.OnClickListener {
    ImageView iv_916;
    ImageView iv_34;
    ImageView iv_11;
    ImageView iv_43;
    ImageView iv_169;
    ImageView iv_free;
    TextView tv_916;
    TextView tv_34;
    TextView tv_11;
    TextView tv_43;
    TextView tv_169;
    TextView tv_free;
    LinearLayout ll_916;
    LinearLayout ll_34;
    LinearLayout ll_11;
    LinearLayout ll_43;
    LinearLayout ll_169;
    LinearLayout ll_free;
    TextView currentSelectTv;
    CustomSeekBar customSeekBar;
    //    TrackBean trackBean;
    MediaTrack mediaTrack;
    SeekBar seekBar_progress;
    int sourceWidth,sourceHeight;
    Matrix matrix;
    ConstraintLayout cl_progress;
    TextView tv_seekbar_start;
    TextView tv_seekbar_end;

    public static final int DEFAULT_COMPRESS_QUALITY = 90;
    public static final Bitmap.CompressFormat DEFAULT_COMPRESS_FORMAT = Bitmap.CompressFormat.JPEG;

    public static final int NONE = 0;
    public static final int SCALE = 1;
    public static final int ROTATE = 2;
    public static final int ALL = 3;

    @Override
    public void loadingProgress(boolean showLoader) {

    }

    @Override
    public void onCropFinish(UCropResult result) {
        Matrix matrix=mGestureCropImageView.getCurrentImageMatrix();
        RectF cropViewRect = mOverlayView.getCropViewRect();
        float initialMinScale = mGestureCropImageView.getInitialMinScale();

        int leftp = mOverlayView.getPaddingLeft();
        int topp = mOverlayView.getPaddingTop();

        matrix.postScale(1 / initialMinScale, 1/ initialMinScale);
        matrix.postTranslate(- (cropViewRect.left - leftp) / initialMinScale, - (cropViewRect.top - topp) / initialMinScale);

        mediaTrack.cropMediaWithSize((int) (cropViewRect.width() / initialMinScale), (int) (cropViewRect.height() / initialMinScale), matrix);
        ActionManager.getInstance().getListener().updateCurrentFrame();
        TrackActionManager.getInstance().hideDetailFragment();
    }


    @IntDef({NONE, SCALE, ROTATE, ALL})
    @Retention(RetentionPolicy.SOURCE)
    public @interface GestureTypes {
    }

    public static final String TAG = "UCropFragment";
    Ratio currentRatio=Ratio.RATIO_FREE;
    private static final long CONTROLS_ANIMATION_DURATION = 50;
    private static final int TABS_COUNT = 3;
    private static final int SCALE_WIDGET_SENSITIVITY_COEFFICIENT = 15000;
    private static final int ROTATE_WIDGET_SENSITIVITY_COEFFICIENT = 42;
    private UCropFragmentCallback callback;

    private int mActiveControlsWidgetColor;

    @ColorInt
    private int mRootViewBackgroundColor;
    private int mLogoColor;

    private boolean mShowBottomControls;

    private Transition mControlsTransition;

    private UCropView mUCropView;
    private GestureCropImageView mGestureCropImageView;
    private OverlayView mOverlayView;
    private ViewGroup mWrapperStateAspectRatio, mWrapperStateRotate, mWrapperStateScale;
    private ViewGroup mLayoutAspectRatio, mLayoutRotate, mLayoutScale;
    private List<ViewGroup> mCropAspectRatioViews = new ArrayList<>();
    private TextView mTextViewRotateAngle, mTextViewScalePercent;
    private View mBlockingView;
    private View mRootView;
    private Bitmap.CompressFormat mCompressFormat = DEFAULT_COMPRESS_FORMAT;
    private int mCompressQuality = DEFAULT_COMPRESS_QUALITY;
    private int[] mAllowedGestures = new int[]{SCALE, ROTATE, ALL};
    private Matrix initMatrix;//初始化后的变动，在传给引擎的时候，需要做抵消。
    private boolean isInitBitmap;//是否是初始化图片
    private CropView cropView;

    public enum Ratio{
        RATIO_FREE(0),RATIO_916(9f/16),RATIO_34(3f/4),RATIO_11(1f),RATIO_43(4f/3),RATIO_169(16f/9);
        private float ratio;
        private Ratio(float ratio){
            this.ratio=ratio;
        }
        public float getRatio(){
            return this.ratio;
        }
    }
    static {
        AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
    }

    public static MyUCropFragment newInstance(Bundle uCrop) {
        MyUCropFragment fragment = new MyUCropFragment();
        fragment.setArguments(uCrop);
        return fragment;
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        setCallback(this);
//        if (getParentFragment() instanceof UCropFragmentCallback)
//            callback = (UCropFragmentCallback) getParentFragment();
//        else if (context instanceof UCropFragmentCallback)
//            callback = (UCropFragmentCallback) context;
//        else
//            throw new IllegalArgumentException(context.toString()
//                    + " must implement UCropFragmentCallback");
    }

    public void setCallback(UCropFragmentCallback callback) {
        this.callback = callback;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_crop, container, false);
        mRootView=rootView;
        Bundle args = getArguments();

        initView(rootView,args);
//        addBlockingView(rootView);
        initListener(rootView);

        return rootView;
    }

    protected void initView(View mRootView,Bundle args) {
        cropView=mRootView.findViewById(R.id.cropView);
        iv_free=mRootView.findViewById(R.id.iv_free);
        iv_916=mRootView.findViewById(R.id.iv_916);
        iv_34=mRootView.findViewById(R.id.iv_34);
        iv_11=mRootView.findViewById(R.id.iv_11);
        iv_43=mRootView.findViewById(R.id.iv_43);
        iv_169=mRootView.findViewById(R.id.iv_169);
        tv_free=mRootView.findViewById(R.id.tv_free);
        tv_916=mRootView.findViewById(R.id.tv_916);
        tv_34=mRootView.findViewById(R.id.tv_34);
        tv_11=mRootView.findViewById(R.id.tv_11);
        tv_43=mRootView.findViewById(R.id.tv_43);
        tv_169=mRootView.findViewById(R.id.tv_169);
        ll_free=mRootView.findViewById(R.id.ll_free);
        ll_916=mRootView.findViewById(R.id.ll_916);
        ll_34=mRootView.findViewById(R.id.ll_34);
        ll_11=mRootView.findViewById(R.id.ll_11);
        ll_43=mRootView.findViewById(R.id.ll_43);
        ll_169=mRootView.findViewById(R.id.ll_169);
        customSeekBar=mRootView.findViewById(R.id.customSeekBar);
        seekBar_progress=mRootView.findViewById(R.id.seekBar_progress);
        cl_progress=mRootView.findViewById(R.id.cl_progress);
        tv_seekbar_start=mRootView.findViewById(R.id.tv_seekbar_start);
        tv_seekbar_end=mRootView.findViewById(R.id.tv_seekbar_end);

        matrix=new Matrix();


        Bitmap b1;
        if (getArguments()!=null&&getArguments().getSerializable("track")!=null){
            mediaTrack=(MediaTrack)getArguments().getSerializable("track");
            if (mediaTrack.resourceDuration()<=0){
                b1=BitmapFactory.decodeFile(mediaTrack.getFilePath());
                args.putParcelable(EXTRA_INPUT_URI, Uri.fromFile(new File(mediaTrack.getFilePath())));
                args.putParcelable(EXTRA_OUTPUT_URI, Uri.fromFile(new File(this.getContext().getCacheDir().getAbsolutePath()+"/crop.jpeg")));
                args.putBoolean(UCrop.Options.EXTRA_HIDE_BOTTOM_CONTROLS,true);
                setupViews(mRootView, args);
                setImageData(args,b1);
                setInitialState();
                sourceWidth=b1.getWidth();
                sourceHeight=b1.getHeight();
                cl_progress.setVisibility(View.GONE);
            }else {
                cl_progress.setVisibility(View.VISIBLE);
                double currentTime=mediaTrack.inPoint();
                getFrameBitmap(currentTime);
                tv_seekbar_start.setText("0");
                tv_seekbar_end.setText((int)mediaTrack.resourceDuration()+"");
                cl_progress.setVisibility(View.GONE);
            }

        }
        seekBar_progress.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (mediaTrack.resourceDuration()>0){
                    getFrameBitmap(mediaTrack.resourceDuration()*progress/100);
                }

            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        currentSelectTv=tv_free;
        currentSelectTv.setTextColor(getResources().getColor(R.color.text_selected_red));
    }

    private void getFrameBitmap(double time){
        AppExecutors.getInstance().networkIO().execute(new Runnable() {

            @Override
            public void run() {
                MediaMetadataRetriever retriever = new MediaMetadataRetriever();
                retriever.setDataSource(mediaTrack.getFilePath());
                Bitmap tmp = retriever.getFrameAtTime((long) (time * 1000 * 1000));
                sourceWidth=tmp.getWidth();
                sourceHeight=tmp.getHeight();
                String savePath=MyUCropFragment.this.getContext().getCacheDir().getAbsolutePath();
                Uri uri=Utils.saveBitmap(tmp,savePath);

                mRootView.getHandler().post(new Runnable() {
                    @Override
                    public void run() {
                        Bundle args=new Bundle();
                        args.putParcelable(EXTRA_INPUT_URI, uri);
                        args.putParcelable(EXTRA_OUTPUT_URI, Uri.fromFile(new File(MyUCropFragment.this.getContext().getCacheDir().getAbsolutePath()+"/crop.jpeg")));
                        args.putBoolean(UCrop.Options.EXTRA_HIDE_BOTTOM_CONTROLS,true);
                        setupViews(mRootView, args);
                        setImageData(args,tmp);
                        setInitialState();
                    }
                });

                retriever.release();
            }
        });
    }

    protected void initListener(View mRootView) {
        ll_free.setOnClickListener(this);
        ll_916.setOnClickListener(this);
        ll_34.setOnClickListener(this);
        ll_11.setOnClickListener(this);
        ll_43.setOnClickListener(this);
        ll_169.setOnClickListener(this);
        mRootView.setOnClickListener(this);
        customSeekBar.setAngleChangeListener(new CustomSeekBar.onAngleChangeListener() {
            @Override
            public void onAngleChangeStart() {
                mGestureCropImageView.cancelAllAnimations();
            }

            @Override
            public void onAngleChangeEnd() {
                mGestureCropImageView.setImageToWrapCropBounds();
            }

            @Override
            public void onAngleChange(float angle) {
                float currentAngle=mGestureCropImageView.getCurrentAngle();
                mGestureCropImageView.postRotate(angle-currentAngle);
            }
        });
        mRootView.findViewById(R.id.iv_submit).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                cropAndSaveImage();


            }
        });
    }

    private void setRatio(Ratio ratio){
        currentRatio=ratio;
        mGestureCropImageView.setTargetAspectRatio(ratio.ratio);
        mGestureCropImageView.setImageToWrapCropBounds();
    }
    @Override
    public void onClick(View v) {
        switch (v.getId()){
            case R.id.ll_free:
                setRatio(Ratio.RATIO_FREE);
                currentSelectTv.setTextColor(getResources().getColor(R.color.white));
                currentSelectTv=tv_free;
                currentSelectTv.setTextColor(getResources().getColor(R.color.text_selected_red));
                break;
            case R.id.ll_916:
//                cropView.setRatio(CropView.Ratio.RATIO_916);
                setRatio(Ratio.RATIO_916);
                currentSelectTv.setTextColor(getResources().getColor(R.color.white));
                currentSelectTv=tv_916;
                currentSelectTv.setTextColor(getResources().getColor(R.color.text_selected_red));
                break;
            case R.id.ll_34:
//                cropView.setRatio(CropView.Ratio.RATIO_34);
                setRatio(Ratio.RATIO_34);
                currentSelectTv.setTextColor(getResources().getColor(R.color.white));
                currentSelectTv=tv_34;
                currentSelectTv.setTextColor(getResources().getColor(R.color.text_selected_red));
                break;
            case R.id.ll_11:
//                cropView.setRatio(CropView.Ratio.RATIO_11);
                setRatio(Ratio.RATIO_11);
                currentSelectTv.setTextColor(getResources().getColor(R.color.white));
                currentSelectTv=tv_11;
                currentSelectTv.setTextColor(getResources().getColor(R.color.text_selected_red));
                break;
            case R.id.ll_43:
//                cropView.setRatio(CropView.Ratio.RATIO_43);
                setRatio(Ratio.RATIO_43);
                currentSelectTv.setTextColor(getResources().getColor(R.color.white));
                currentSelectTv=tv_43;
                currentSelectTv.setTextColor(getResources().getColor(R.color.text_selected_red));
                break;
            case R.id.ll_169:
//                cropView.setRatio(CropView.Ratio.RATIO_169);
                setRatio(Ratio.RATIO_169);
                currentSelectTv.setTextColor(getResources().getColor(R.color.white));
                currentSelectTv=tv_169;
                currentSelectTv.setTextColor(getResources().getColor(R.color.text_selected_red));
                break;

        }
    }
    public void setupViews(View view, Bundle args) {
        mActiveControlsWidgetColor = args.getInt(UCrop.Options.EXTRA_UCROP_COLOR_CONTROLS_WIDGET_ACTIVE, ContextCompat.getColor(getContext(), R.color.ucrop_color_widget_active));
        mLogoColor = args.getInt(UCrop.Options.EXTRA_UCROP_LOGO_COLOR, ContextCompat.getColor(getContext(), R.color.ucrop_color_default_logo));
        mShowBottomControls = !args.getBoolean(UCrop.Options.EXTRA_HIDE_BOTTOM_CONTROLS, false);
        mRootViewBackgroundColor = args.getInt(UCrop.Options.EXTRA_UCROP_ROOT_VIEW_BACKGROUND_COLOR, ContextCompat.getColor(getContext(), R.color.ucrop_color_crop_background));

        initiateRootViews(view);
        callback.loadingProgress(true);

        if (mShowBottomControls) {

            ViewGroup wrapper = view.findViewById(R.id.controls_wrapper);
            wrapper.setVisibility(View.VISIBLE);
            LayoutInflater.from(getContext()).inflate(R.layout.ucrop_controls, wrapper, true);

            mControlsTransition = new AutoTransition();
            mControlsTransition.setDuration(CONTROLS_ANIMATION_DURATION);

            mWrapperStateAspectRatio = view.findViewById(R.id.state_aspect_ratio);
            mWrapperStateAspectRatio.setOnClickListener(mStateClickListener);
            mWrapperStateRotate = view.findViewById(R.id.state_rotate);
            mWrapperStateRotate.setOnClickListener(mStateClickListener);
            mWrapperStateScale = view.findViewById(R.id.state_scale);
            mWrapperStateScale.setOnClickListener(mStateClickListener);

            mLayoutAspectRatio = view.findViewById(R.id.layout_aspect_ratio);
            mLayoutRotate = view.findViewById(R.id.layout_rotate_wheel);
            mLayoutScale = view.findViewById(R.id.layout_scale_wheel);

            setupAspectRatioWidget(args, view);
            setupRotateWidget(view);
            setupScaleWidget(view);
            setupStatesWrapper(view);
        } else {
            ConstraintLayout.LayoutParams params = (ConstraintLayout.LayoutParams) view.findViewById(R.id.ucrop_frame).getLayoutParams();
            params.bottomMargin = 0;
            view.findViewById(R.id.ucrop_frame).requestLayout();
        }
    }

    private void setImageData(@NonNull Bundle bundle,Bitmap bitmap) {
        Uri inputUri = bundle.getParcelable(EXTRA_INPUT_URI);
        Uri outputUri = bundle.getParcelable(EXTRA_OUTPUT_URI);
        processOptions(bundle);
        if (inputUri != null && outputUri != null) {
            try {
                mGestureCropImageView.setImageUri(inputUri, outputUri);

            } catch (Exception e) {
                callback.onCropFinish(getError(e));
            }
        } else {
            callback.onCropFinish(getError(new NullPointerException(getString(R.string.ucrop_error_input_data_is_absent))));
        }
    }



    /**
     * This method extracts {@link UCrop.Options #optionsBundle} from incoming bundle
     * and setups fragment, {@link OverlayView} and {@link CropImageView} properly.
     */
    @SuppressWarnings("deprecation")
    private void processOptions(@NonNull Bundle bundle) {
        // Bitmap compression options
        String compressionFormatName = bundle.getString(UCrop.Options.EXTRA_COMPRESSION_FORMAT_NAME);
        Bitmap.CompressFormat compressFormat = null;
        if (!TextUtils.isEmpty(compressionFormatName)) {
            compressFormat = Bitmap.CompressFormat.valueOf(compressionFormatName);
        }

        mCompressFormat = (compressFormat == null) ? DEFAULT_COMPRESS_FORMAT : compressFormat;

        mCompressQuality = bundle.getInt(UCrop.Options.EXTRA_COMPRESSION_QUALITY, UCropActivity.DEFAULT_COMPRESS_QUALITY);

        // Gestures options
        int[] allowedGestures = bundle.getIntArray(UCrop.Options.EXTRA_ALLOWED_GESTURES);
        if (allowedGestures != null && allowedGestures.length == TABS_COUNT) {
            mAllowedGestures = allowedGestures;
        }

        // Crop image view options
        mGestureCropImageView.setMaxBitmapSize(bundle.getInt(UCrop.Options.EXTRA_MAX_BITMAP_SIZE, CropImageView.DEFAULT_MAX_BITMAP_SIZE));
        mGestureCropImageView.setMaxScaleMultiplier(bundle.getFloat(UCrop.Options.EXTRA_MAX_SCALE_MULTIPLIER, CropImageView.DEFAULT_MAX_SCALE_MULTIPLIER));
        mGestureCropImageView.setImageToWrapCropBoundsAnimDuration(bundle.getInt(UCrop.Options.EXTRA_IMAGE_TO_CROP_BOUNDS_ANIM_DURATION, CropImageView.DEFAULT_IMAGE_TO_CROP_BOUNDS_ANIM_DURATION));

        // Overlay view options
        mOverlayView.setFreestyleCropEnabled(bundle.getBoolean(UCrop.Options.EXTRA_FREE_STYLE_CROP, OverlayView.DEFAULT_FREESTYLE_CROP_MODE != OverlayView.FREESTYLE_CROP_MODE_DISABLE));

        mOverlayView.setDimmedColor(bundle.getInt(UCrop.Options.EXTRA_DIMMED_LAYER_COLOR, getResources().getColor(R.color.ucrop_color_default_dimmed)));
        mOverlayView.setCircleDimmedLayer(bundle.getBoolean(UCrop.Options.EXTRA_CIRCLE_DIMMED_LAYER, OverlayView.DEFAULT_CIRCLE_DIMMED_LAYER));

        mOverlayView.setShowCropFrame(bundle.getBoolean(UCrop.Options.EXTRA_SHOW_CROP_FRAME, OverlayView.DEFAULT_SHOW_CROP_FRAME));
        mOverlayView.setCropFrameColor(bundle.getInt(UCrop.Options.EXTRA_CROP_FRAME_COLOR, getResources().getColor(R.color.ucrop_color_default_crop_frame)));
        mOverlayView.setCropFrameStrokeWidth(bundle.getInt(UCrop.Options.EXTRA_CROP_FRAME_STROKE_WIDTH, getResources().getDimensionPixelSize(R.dimen.ucrop_default_crop_frame_stoke_width)));

        mOverlayView.setShowCropGrid(bundle.getBoolean(UCrop.Options.EXTRA_SHOW_CROP_GRID, OverlayView.DEFAULT_SHOW_CROP_GRID));
        mOverlayView.setCropGridRowCount(bundle.getInt(UCrop.Options.EXTRA_CROP_GRID_ROW_COUNT, OverlayView.DEFAULT_CROP_GRID_ROW_COUNT));
        mOverlayView.setCropGridColumnCount(bundle.getInt(UCrop.Options.EXTRA_CROP_GRID_COLUMN_COUNT, OverlayView.DEFAULT_CROP_GRID_COLUMN_COUNT));
        mOverlayView.setCropGridColor(bundle.getInt(UCrop.Options.EXTRA_CROP_GRID_COLOR, getResources().getColor(R.color.ucrop_color_default_crop_grid)));
        mOverlayView.setCropGridStrokeWidth(bundle.getInt(UCrop.Options.EXTRA_CROP_GRID_STROKE_WIDTH, getResources().getDimensionPixelSize(R.dimen.ucrop_default_crop_grid_stoke_width)));

        // Aspect ratio options
        float aspectRatioX = bundle.getFloat(UCrop.EXTRA_ASPECT_RATIO_X, 0);
        float aspectRatioY = bundle.getFloat(UCrop.EXTRA_ASPECT_RATIO_Y, 0);

        int aspectRationSelectedByDefault = bundle.getInt(UCrop.Options.EXTRA_ASPECT_RATIO_SELECTED_BY_DEFAULT, 0);
        ArrayList<AspectRatio> aspectRatioList = bundle.getParcelableArrayList(UCrop.Options.EXTRA_ASPECT_RATIO_OPTIONS);

        if (aspectRatioX > 0 && aspectRatioY > 0) {
            if (mWrapperStateAspectRatio != null) {
                mWrapperStateAspectRatio.setVisibility(View.GONE);
            }
            mGestureCropImageView.setTargetAspectRatio(aspectRatioX / aspectRatioY);
        } else if (aspectRatioList != null && aspectRationSelectedByDefault < aspectRatioList.size()) {
            mGestureCropImageView.setTargetAspectRatio(aspectRatioList.get(aspectRationSelectedByDefault).getAspectRatioX() /
                    aspectRatioList.get(aspectRationSelectedByDefault).getAspectRatioY());
        } else {
            mGestureCropImageView.setTargetAspectRatio(CropImageView.SOURCE_IMAGE_ASPECT_RATIO);
        }

        // Result bitmap max size options
        int maxSizeX = bundle.getInt(UCrop.EXTRA_MAX_SIZE_X, 0);
        int maxSizeY = bundle.getInt(UCrop.EXTRA_MAX_SIZE_Y, 0);

        if (maxSizeX > 0 && maxSizeY > 0) {
            mGestureCropImageView.setMaxResultImageSizeX(maxSizeX);
            mGestureCropImageView.setMaxResultImageSizeY(maxSizeY);
        }
    }

    private void initiateRootViews(View view) {
        mUCropView = view.findViewById(R.id.ucrop);
        mGestureCropImageView = mUCropView.getCropImageView();
        mOverlayView = mUCropView.getOverlayView();

        mGestureCropImageView.setTransformImageListener(mImageListener);

        ((ImageView) view.findViewById(R.id.image_view_logo)).setColorFilter(mLogoColor, PorterDuff.Mode.SRC_ATOP);

        view.findViewById(R.id.ucrop_frame).setBackgroundColor(mRootViewBackgroundColor);
    }
    boolean isInitBitmapScale=true;
    boolean isInitBitmapRotate=true;
    boolean isInitBitmapTrans=true;
    private TransformImageView.TransformImageListener mImageListener = new TransformImageView.TransformImageListener() {
        @Override
        public void onRotate(float currentAngle) {
            setAngleText(currentAngle);
            if (isInitBitmapRotate) {
                isInitBitmapRotate=false;
                initMatrix.postRotate(currentAngle);
            }
            Logger.e("cf onRotate currentAngle=" +currentAngle+"  "+ mGestureCropImageView.getCurrentImageMatrix().toShortString());
        }

        @Override
        public void onScale(float currentScale) {
            setScaleText(currentScale);
            if (isInitBitmapScale) {
                isInitBitmapScale=false;
                initMatrix.postScale(currentScale, currentScale);
            }
            Logger.e("cf onScale currentScale=" +currentScale+" "+ mGestureCropImageView.getCurrentImageMatrix().toShortString());
        }

        @Override
        public void onLoadComplete() {
            mUCropView.animate().alpha(1).setDuration(300).setInterpolator(new AccelerateInterpolator());
//            mBlockingView.setClickable(false);
            callback.loadingProgress(false);

                initMatrix=new Matrix();
            isInitBitmapScale=false;
            isInitBitmapRotate=false;
//                initMatrix.set(mGestureCropImageView.getCurrentImageMatrix());
            Logger.e("cf onLoadComplete initMatrix=" + initMatrix.toShortString());
            Logger.e("cf onLoadComplete Matrix=" + mGestureCropImageView.getCurrentImageMatrix().toShortString());
        }

        @Override
        public void onLoadFailure(@NonNull Exception e) {
            callback.onCropFinish(getError(e));
        }

    };

    /**
     * Use {@link #} for color filter
     */
    private void setupStatesWrapper(View view) {
        ImageView stateScaleImageView = view.findViewById(R.id.image_view_state_scale);
        ImageView stateRotateImageView = view.findViewById(R.id.image_view_state_rotate);
        ImageView stateAspectRatioImageView = view.findViewById(R.id.image_view_state_aspect_ratio);

        stateScaleImageView.setImageDrawable(new SelectedStateListDrawable(stateScaleImageView.getDrawable(), mActiveControlsWidgetColor));
        stateRotateImageView.setImageDrawable(new SelectedStateListDrawable(stateRotateImageView.getDrawable(), mActiveControlsWidgetColor));
        stateAspectRatioImageView.setImageDrawable(new SelectedStateListDrawable(stateAspectRatioImageView.getDrawable(), mActiveControlsWidgetColor));
    }

    private void setupAspectRatioWidget(@NonNull Bundle bundle, View view) {
        int aspectRationSelectedByDefault = bundle.getInt(UCrop.Options.EXTRA_ASPECT_RATIO_SELECTED_BY_DEFAULT, 0);
        ArrayList<AspectRatio> aspectRatioList = bundle.getParcelableArrayList(UCrop.Options.EXTRA_ASPECT_RATIO_OPTIONS);

        if (aspectRatioList == null || aspectRatioList.isEmpty()) {
            aspectRationSelectedByDefault = 2;

            aspectRatioList = new ArrayList<>();
            aspectRatioList.add(new AspectRatio(null, 1, 1));
            aspectRatioList.add(new AspectRatio(null, 3, 4));
            aspectRatioList.add(new AspectRatio(getString(R.string.ucrop_label_original).toUpperCase(),
                    CropImageView.SOURCE_IMAGE_ASPECT_RATIO, CropImageView.SOURCE_IMAGE_ASPECT_RATIO));
            aspectRatioList.add(new AspectRatio(null, 3, 2));
            aspectRatioList.add(new AspectRatio(null, 16, 9));
        }

        LinearLayout wrapperAspectRatioList = view.findViewById(R.id.layout_aspect_ratio);

        FrameLayout wrapperAspectRatio;
        AspectRatioTextView aspectRatioTextView;
        LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(0, ViewGroup.LayoutParams.MATCH_PARENT);
        lp.weight = 1;
        for (AspectRatio aspectRatio : aspectRatioList) {
            wrapperAspectRatio = (FrameLayout) getLayoutInflater().inflate(R.layout.ucrop_aspect_ratio, null);
            wrapperAspectRatio.setLayoutParams(lp);
            aspectRatioTextView = ((AspectRatioTextView) wrapperAspectRatio.getChildAt(0));
            aspectRatioTextView.setActiveColor(mActiveControlsWidgetColor);
            aspectRatioTextView.setAspectRatio(aspectRatio);

            wrapperAspectRatioList.addView(wrapperAspectRatio);
            mCropAspectRatioViews.add(wrapperAspectRatio);
        }

        mCropAspectRatioViews.get(aspectRationSelectedByDefault).setSelected(true);

        for (ViewGroup cropAspectRatioView : mCropAspectRatioViews) {
            cropAspectRatioView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    mGestureCropImageView.setTargetAspectRatio(
                            ((AspectRatioTextView) ((ViewGroup) v).getChildAt(0)).getAspectRatio(v.isSelected()));
                    mGestureCropImageView.setImageToWrapCropBounds();
                    if (!v.isSelected()) {
                        for (ViewGroup cropAspectRatioView : mCropAspectRatioViews) {
                            cropAspectRatioView.setSelected(cropAspectRatioView == v);
                        }
                    }
                }
            });
        }
    }

    private void setupRotateWidget(View view) {
        mTextViewRotateAngle = view.findViewById(R.id.text_view_rotate);
        ((HorizontalProgressWheelView) view.findViewById(R.id.rotate_scroll_wheel))
                .setScrollingListener(new HorizontalProgressWheelView.ScrollingListener() {
                    @Override
                    public void onScroll(float delta, float totalDistance) {
                        mGestureCropImageView.postRotate(delta / ROTATE_WIDGET_SENSITIVITY_COEFFICIENT);
                    }

                    @Override
                    public void onScrollEnd() {
                        mGestureCropImageView.setImageToWrapCropBounds();
                    }

                    @Override
                    public void onScrollStart() {
                        mGestureCropImageView.cancelAllAnimations();
                    }
                });

        ((HorizontalProgressWheelView) view.findViewById(R.id.rotate_scroll_wheel)).setMiddleLineColor(mActiveControlsWidgetColor);


        view.findViewById(R.id.wrapper_reset_rotate).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                resetRotation();
            }
        });
        view.findViewById(R.id.wrapper_rotate_by_angle).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                rotateByAngle(90);
            }
        });
        setAngleTextColor(mActiveControlsWidgetColor);
    }

    private void setupScaleWidget(View view) {
        mTextViewScalePercent = view.findViewById(R.id.text_view_scale);
        ((HorizontalProgressWheelView) view.findViewById(R.id.scale_scroll_wheel))
                .setScrollingListener(new HorizontalProgressWheelView.ScrollingListener() {
                    @Override
                    public void onScroll(float delta, float totalDistance) {
                        if (delta > 0) {
                            mGestureCropImageView.zoomInImage(mGestureCropImageView.getCurrentScale()
                                    + delta * ((mGestureCropImageView.getMaxScale() - mGestureCropImageView.getMinScale()) / SCALE_WIDGET_SENSITIVITY_COEFFICIENT));
                        } else {
                            mGestureCropImageView.zoomOutImage(mGestureCropImageView.getCurrentScale()
                                    + delta * ((mGestureCropImageView.getMaxScale() - mGestureCropImageView.getMinScale()) / SCALE_WIDGET_SENSITIVITY_COEFFICIENT));
                        }
                    }

                    @Override
                    public void onScrollEnd() {
                        mGestureCropImageView.setImageToWrapCropBounds();
                    }

                    @Override
                    public void onScrollStart() {
                        mGestureCropImageView.cancelAllAnimations();
                    }
                });
        ((HorizontalProgressWheelView) view.findViewById(R.id.scale_scroll_wheel)).setMiddleLineColor(mActiveControlsWidgetColor);

        setScaleTextColor(mActiveControlsWidgetColor);
    }

    private void setAngleText(float angle) {
        if (mTextViewRotateAngle != null) {
            mTextViewRotateAngle.setText(String.format(Locale.getDefault(), "%.1f°", angle));
        }
    }

    private void setAngleTextColor(int textColor) {
        if (mTextViewRotateAngle != null) {
            mTextViewRotateAngle.setTextColor(textColor);
        }
    }

    private void setScaleText(float scale) {
        if (mTextViewScalePercent != null) {
            mTextViewScalePercent.setText(String.format(Locale.getDefault(), "%d%%", (int) (scale * 100)));
        }
    }

    private void setScaleTextColor(int textColor) {
        if (mTextViewScalePercent != null) {
            mTextViewScalePercent.setTextColor(textColor);
        }
    }

    private void resetRotation() {
        mGestureCropImageView.postRotate(-mGestureCropImageView.getCurrentAngle());
        mGestureCropImageView.setImageToWrapCropBounds();
    }

    private void rotateByAngle(int angle) {
        mGestureCropImageView.postRotate(angle);
        mGestureCropImageView.setImageToWrapCropBounds();
    }

    private final View.OnClickListener mStateClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            if (!v.isSelected()) {

                setWidgetState(v.getId());

            }
        }
    };

    private void setInitialState() {
        if (mShowBottomControls) {
            if (mWrapperStateAspectRatio.getVisibility() == View.VISIBLE) {
                setWidgetState(R.id.state_aspect_ratio);
            } else {
                setWidgetState(R.id.state_scale);
            }
        } else {
            setAllowedGestures(0);
        }
    }

    private void setWidgetState(@IdRes int stateViewId) {
        if (!mShowBottomControls) return;

        mWrapperStateAspectRatio.setSelected(stateViewId == R.id.state_aspect_ratio);
        mWrapperStateRotate.setSelected(stateViewId == R.id.state_rotate);
        mWrapperStateScale.setSelected(stateViewId == R.id.state_scale);

        mLayoutAspectRatio.setVisibility(stateViewId == R.id.state_aspect_ratio ? View.VISIBLE : View.GONE);
        mLayoutRotate.setVisibility(stateViewId == R.id.state_rotate ? View.VISIBLE : View.GONE);
        mLayoutScale.setVisibility(stateViewId == R.id.state_scale ? View.VISIBLE : View.GONE);

        changeSelectedTab(stateViewId);

        if (stateViewId == R.id.state_scale) {
            setAllowedGestures(0);
        } else if (stateViewId == R.id.state_rotate) {
            setAllowedGestures(1);
        } else {
            setAllowedGestures(2);
        }
    }

    private void changeSelectedTab(int stateViewId) {
        if (getView() != null) {
            TransitionManager.beginDelayedTransition((ViewGroup) getView().findViewById(R.id.ucrop_photobox), mControlsTransition);
        }
        mWrapperStateScale.findViewById(R.id.text_view_scale).setVisibility(stateViewId == R.id.state_scale ? View.VISIBLE : View.GONE);
        mWrapperStateAspectRatio.findViewById(R.id.text_view_crop).setVisibility(stateViewId == R.id.state_aspect_ratio ? View.VISIBLE : View.GONE);
        mWrapperStateRotate.findViewById(R.id.text_view_rotate).setVisibility(stateViewId == R.id.state_rotate ? View.VISIBLE : View.GONE);
    }

    private void setAllowedGestures(int tab) {
        mGestureCropImageView.setScaleEnabled(mAllowedGestures[tab] == ALL || mAllowedGestures[tab] == SCALE);
        mGestureCropImageView.setRotateEnabled(mAllowedGestures[tab] == ALL || mAllowedGestures[tab] == ROTATE);
    }

    /**
     * Adds view that covers everything below the Toolbar.
     * When it's clickable - user won't be able to click/touch anything below the Toolbar.
     * Need to block user input while loading and cropping an image.
     */
    private void addBlockingView(View view) {
        if (mBlockingView == null) {
            mBlockingView = new View(getContext());
            RelativeLayout.LayoutParams lp = new RelativeLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
            mBlockingView.setLayoutParams(lp);
            mBlockingView.setClickable(true);
        }

        ((RelativeLayout) view.findViewById(R.id.ucrop_photobox)).addView(mBlockingView);
    }

    public void cropAndSaveImage() {
//        mBlockingView.setClickable(true);
        callback.loadingProgress(true);

        mGestureCropImageView.cropAndSaveImage(mCompressFormat, mCompressQuality, new BitmapCropCallback() {

            @Override
            public void onBitmapCropped(@NonNull Uri resultUri, int offsetX, int offsetY, int imageWidth, int imageHeight) {
                callback.onCropFinish(getResult(resultUri, mGestureCropImageView.getTargetAspectRatio(), offsetX, offsetY, imageWidth, imageHeight));
                callback.loadingProgress(false);
            }

            @Override
            public void onCropFailure(@NonNull Throwable t) {
                callback.onCropFinish(getError(t));
            }
        });
    }

    protected UCropResult getResult(Uri uri, float resultAspectRatio, int offsetX, int offsetY, int imageWidth, int imageHeight) {
        return new UCropResult(RESULT_OK, new Intent()
                .putExtra(EXTRA_OUTPUT_URI, uri)
                .putExtra(UCrop.EXTRA_OUTPUT_CROP_ASPECT_RATIO, resultAspectRatio)
                .putExtra(UCrop.EXTRA_OUTPUT_IMAGE_WIDTH, imageWidth)
                .putExtra(UCrop.EXTRA_OUTPUT_IMAGE_HEIGHT, imageHeight)
                .putExtra(UCrop.EXTRA_OUTPUT_OFFSET_X, offsetX)
                .putExtra(UCrop.EXTRA_OUTPUT_OFFSET_Y, offsetY)
        );
    }

    protected UCropResult getError(Throwable throwable) {
        return new UCropResult(UCrop.RESULT_ERROR, new Intent().putExtra(UCrop.EXTRA_ERROR, throwable));
    }

    public class UCropResult {

        public int mResultCode;
        public Intent mResultData;

        public UCropResult(int resultCode, Intent data) {
            mResultCode = resultCode;
            mResultData = data;
        }

    }

}

