package com.shixing.edit.clip;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.R;
import com.shixing.edit.adapter.FilterAdapter;
import com.shixing.edit.adapter.TitleAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.AnimationEffect;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.AnimationEffect;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class VideoAnimFragment extends BaseFragment implements OnActionClickListener {

    private RecyclerView mTitle_recycler;
    private RecyclerView mItem1_recycler;
    private ArrayList<String> titleList;

    private SeekBar seekbar;
    private TextView tvDuration;
    private View seekbarGroup;

    private FilterAdapter adapter;
    private List<ActionItem> list1;
    private List<ActionItem> list2;
    private List<ActionItem> list3;

    private MediaTrack mediaTrack;
    private AnimationEffect animationEffect;
    private double trackDuration;
    private double maxDuration;
    private SXEffect.SXEffectTimeFollowType followType = SXEffect.SXEffectTimeFollowType.FollowStart;

    private String[] categories = {"入场动画", "出场动画", "循环动画"};

    @Override
    protected int initRootView() {
        return R.layout.fragment_video_anim;
    }

    @Override
    protected void initArguments(Bundle arguments) {

    }

    @Override
    protected void initView() {
        mTitle_recycler = mRootView.findViewById(R.id.title_recycler);
        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        seekbar = mRootView.findViewById(R.id.seekBar);
        tvDuration = mRootView.findViewById(R.id.tv_seekbar_num);
        seekbarGroup = mRootView.findViewById(R.id.group_seekbar);
        seekbar.setMax(100);

        titleList = new ArrayList<>();
        titleList.addAll(Arrays.asList(categories));

        TitleAdapter typeAdapter = new TitleAdapter(this.mActivity);
        typeAdapter.updateData(titleList);
        typeAdapter.setActionClickListener(this);
        mTitle_recycler.setAdapter(typeAdapter);
        LinearLayoutManager typeLayoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mTitle_recycler.setLayoutManager(typeLayoutManager);

        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);
        adapter = new FilterAdapter(this.mActivity);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);

        initList();
        adapter.updateData(list1);

        mediaTrack = (MediaTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        trackDuration = mediaTrack.duration();
        maxDuration = trackDuration / 2;
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mRootView.setOnClickListener(v -> {});

        mRootView.findViewById(R.id.iv_submit).setOnClickListener(v -> TrackActionManager.getInstance().hideDetailFragment());

        seekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                double duration = progress / 100f * maxDuration;
                tvDuration.setText(String.format("%.1fs", duration));
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {
                double duration = seekBar.getProgress() / 100f * maxDuration;
                if (animationEffect.followType() != SXEffect.SXEffectTimeFollowType.FollowWholeProcess) {
                    animationEffect.setDuration(duration);
                }

                animationEffect.setDurationOfOneCycle(duration);
            }
        });
    }

    @Override
    public void onActionClick(ActionItem action) {
        if (categories[0].equals(action.actionName)) {
            maxDuration = trackDuration / 2;
            adapter.updateData(list1);
            followType = SXEffect.SXEffectTimeFollowType.FollowStart;
            seekbarGroup.setVisibility(View.GONE);
        } else if (categories[1].equals(action.actionName)) {
            maxDuration = trackDuration / 2;
            adapter.updateData(list2);
            followType = SXEffect.SXEffectTimeFollowType.FollowEnd;
            seekbarGroup.setVisibility(View.GONE);
        } else if (categories[2].equals(action.actionName)) {
            maxDuration = trackDuration;
            adapter.updateData(list3);
            followType = SXEffect.SXEffectTimeFollowType.FollowWholeProcess;
            seekbarGroup.setVisibility(View.GONE);
        } else {
            applyAnim(action);
        }
    }

    private void applyAnim(ActionItem action) {
        if (TextUtils.isEmpty(action.srcPath)) {
            seekbarGroup.setVisibility(View.GONE);
            List<AnimationEffect> animations = mediaTrack.getAnimations();
            if (animations != null) {
                for (AnimationEffect animation : animations) {
                    if (animation.followType() == followType) {
                        mediaTrack.removeAnimation(animation.getEffectId());
                    }
                }
            }
        } else {
            String animPath = action.actionType;
            AnimationEffect anim = null;
            List<AnimationEffect> animations = mediaTrack.getAnimations();
            if (animations != null) {
                for (AnimationEffect animation : animations) {
                    if ((followType == SXEffect.SXEffectTimeFollowType.FollowStart || followType == SXEffect.SXEffectTimeFollowType.FollowEnd) && animation.followType() == SXEffect.SXEffectTimeFollowType.FollowWholeProcess) {
                        mediaTrack.removeAnimation(animation.getEffectId());
                    } else if (followType == SXEffect.SXEffectTimeFollowType.FollowWholeProcess && (animation.followType() == SXEffect.SXEffectTimeFollowType.FollowStart || animation.followType() == SXEffect.SXEffectTimeFollowType.FollowEnd)) {
                        mediaTrack.removeAnimation(animation.getEffectId());
                    } else if (animation.followType() == followType) {
                        animation.setResource(animPath);
                        animation.setDurationOfOneCycle(animation.duration());
                        anim = animation;
                    }
                }
            }

            if (anim == null) {
                anim = mediaTrack.addAnimation(animPath);
                if (anim != null) {
                    anim.setEffectFollowType(followType);
                    if (anim.resourceDuration() > maxDuration) {
                        anim.setDuration(maxDuration);
                        anim.setDurationOfOneCycle(maxDuration);
                    }
                }
            }

            if (anim != null) {
                animationEffect = anim;
                seekbarGroup.setVisibility(View.VISIBLE);
                seekbar.setProgress((int) (anim.duration() / maxDuration * 100));
                tvDuration.setText(String.format("%.1fs", anim.duration()));
            }
        }
    }

    private void initList() {
        list1 = initList("入场动画", "ve");
        list2 = initList("出场动画", "vx");
        list3 = initList("循环动画", "vl");
    }

    private List<ActionItem> initList(String subFolderName, String name) {
        List<ActionItem> animList = new ArrayList<>();
        String inAinmSubFolder = "视频动画" + "/" + subFolderName;
        animList.add(new ActionItem("无", "", ""));
        animList.add(new ActionItem("动画1", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "1.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "1"));
        animList.add(new ActionItem("动画2", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "2.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "2"));
        animList.add(new ActionItem("动画3", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "3.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "3"));
        animList.add(new ActionItem("动画4", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "4.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "4"));
        animList.add(new ActionItem("动画5", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "5.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "5"));
        return animList;
    }
}
