package com.shixing.edit.clip.transition;

import android.os.Bundle;
import android.view.View;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.R;
import com.shixing.edit.adapter.TitleAdapter;
import com.shixing.edit.adapter.TransitionItemAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.internal.MainTrackGroup;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;

import java.util.ArrayList;
import java.util.List;

public class TransitionFragment extends BaseFragment implements OnActionClickListener {

    private final String folderName = "视频转场";
    private RecyclerView mTitle_recycler;
    private RecyclerView mItem1_recycler;
    private SeekBar mSeekBar;
    private ImageView mIv_submit;

    private ArrayList<String> titleList;
    private ArrayList<ActionItem> itemArrayList;
    private String trackId;
    private View seekbarGroup;
    private MainTrackGroup mainGroup;
    private double maxDuration;
    private TextView tvDuration;

    @Override
    protected int initRootView() {
        return R.layout.fragment_edit_transition;
    }

    @Override
    protected void initArguments(Bundle arguments) {
        trackId = arguments.getString("id");
    }

    @Override
    protected void initView() {
        mTitle_recycler = mRootView.findViewById(R.id.title_recycler);
        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        mSeekBar = mRootView.findViewById(R.id.seekBar);
        mIv_submit = mRootView.findViewById(R.id.iv_submit);

        titleList = new ArrayList<>();
        titleList.add("基础转场");
//        titleList.add("基础运营");
//        titleList.add("运镜转场");
//        titleList.add("幻灯片");
//        titleList.add("特效转场");
//        titleList.add("基础运营");
//        titleList.add("运镜转场");
//        titleList.add("幻灯片");
//        titleList.add("特效转场");

        TitleAdapter adapter = new TitleAdapter(this.mActivity);
        adapter.updateData(titleList);
        adapter.setActionClickListener(this);
        mTitle_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mTitle_recycler.setLayoutManager(layoutManager);
        showItem1(R.drawable.icon_add_tiezhi);

        seekbarGroup = findViewById(R.id.group_seekbar);
        tvDuration = (TextView) findViewById(R.id.tv_duration);

        mainGroup = ActionManager.getInstance().getListener().getMainGroup();
        maxDuration = mainGroup.getTransitionMaxDuration(trackId);
        if (mainGroup.hasTransition(trackId)) {
            double duration = mainGroup.getTransitionDuration(trackId);

            seekbarGroup.setVisibility(View.VISIBLE);
            updateSeekBar(duration);
        }
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        mSeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (fromUser) {
                    double duration = progress / 100f * maxDuration;
                    tvDuration.setText(String.format("%.1fs", duration));
                    mainGroup.setTransitionDuration(trackId, duration);
                    ActionManager.getInstance().getListener().updateCurrentFrame();
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        mIv_submit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                updateMainTracks();
                TrackActionManager.getInstance().hideDetailFragment();
            }
        });

        findViewById(R.id.tv_applyall).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String resourcePath = null;
                double transitionDuration = 0;

                boolean hasTransition = mainGroup.hasTransition(trackId);
                if (hasTransition) {
                    resourcePath = mainGroup.getTransitionResourcePath(trackId);
                    transitionDuration = mainGroup.getTransitionDuration(trackId);
                }
                List<Track> tracks = mainGroup.tracks();
                for (int i = 0; i < tracks.size() - 1; i++) {
                    if (!hasTransition) {
                        mainGroup.removeTransition(tracks.get(i).getTrackId());
                    } else {
                        mainGroup.addTransition(resourcePath, tracks.get(i).getTrackId(), transitionDuration);
                    }
                }
            }
        });
    }

    private void showItem1(int resId) {
        itemArrayList = new ArrayList<>();
        itemArrayList.add(new ActionItem("无", "", ""));
//        itemArrayList.add(new ActionItem("闪黑", resId, ActionItem.resId));
//        itemArrayList.add(new ActionItem("闪白", resId, ActionItem.resId));
//        itemArrayList.add(new ActionItem("色彩溶解", resId, ActionItem.resId));
//        itemArrayList.add(new ActionItem("叠化", resId, ActionItem.resId));
//        itemArrayList.add(new ActionItem("叠化", resId, ActionItem.resId));
        itemArrayList.add(new ActionItem("转场1", Constants.assetsPath + "/" + folderName + "/tr1.gif", Constants.resourcePath + "/" + folderName + "/tr1"));
        itemArrayList.add(new ActionItem("转场2", Constants.assetsPath + "/" + folderName + "/tr2.gif", Constants.resourcePath + "/" + folderName + "/tr2"));
        itemArrayList.add(new ActionItem("转场3", Constants.assetsPath + "/" + folderName + "/tr3.gif", Constants.resourcePath + "/" + folderName + "/tr3"));
        itemArrayList.add(new ActionItem("转场4", Constants.assetsPath + "/" + folderName + "/tr4.gif", Constants.resourcePath + "/" + folderName + "/tr4"));
        itemArrayList.add(new ActionItem("转场5", Constants.assetsPath + "/" + folderName + "/tr5.gif", Constants.resourcePath + "/" + folderName + "/tr5"));

        TransitionItemAdapter adapter = new TransitionItemAdapter(this.mActivity);
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);
    }

    @Override
    public void onActionClick(ActionItem action) {
        switch (action.actionType) {
            case ActionItem.text:
                switch (action.actionName) {
                    case "基础运营":
                        showItem1(R.drawable.icon_add_tiezhi);
                        break;
                    case "运镜转场":
                        showItem1(R.drawable.icon_baoguangdu);
                        break;
                    case "幻灯片":
                        showItem1(R.drawable.icon_bianji);
                        break;
                    case "特效转场":
                        showItem1(R.drawable.icon_biansu);
                        break;
                    default:
                        break;
                }
                break;
//            case ActionItem.resId:
//                applyTransition(action);
            default:
                applyTransition(action);
                break;
        }
    }

    private void applyTransition(ActionItem action) {
        if ("无".equals(action.actionName)) {
            mainGroup.removeTransition(trackId);
            seekbarGroup.setVisibility(View.GONE);
            updateMainTracks();
        } else {
            seekbarGroup.setVisibility(View.VISIBLE);
            double duration = maxDuration / 2;
            mainGroup.addTransition(action.actionType, trackId, duration);
            updateSeekBar(duration);
            updateMainTracks();
        }
    }

    private void updateMainTracks() {
        List<MediaTrack> mainTracks = ActionManager.getInstance().getListener().getMainTracks();
        TrackActionManager.getInstance().addMainTrack(mainTracks);
        ActionManager.getInstance().getListener().updateCurrentFrame();
    }

    private void updateSeekBar(double duration) {
        mSeekBar.setProgress((int) (duration / maxDuration * 100));
        tvDuration.setText(String.format("%.1fs", duration));
    }

}
