package com.shixing.edit.data;

import android.text.TextUtils;
import android.util.Log;
import android.util.Size;

import com.shixing.edit.widget.SurfaceOverlay;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.EditManager;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.FilterEffect;
import com.shixing.sxedit.internal.MainTrackGroup;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.StickerTrack;
import com.shixing.sxedit.internal.TextTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;
import com.shixing.sxedit.internal.VideoEffect;

import java.util.ArrayList;
import java.util.List;

public class EditData {
    private static final String TAG = "EditData";
    private static final double TEXT_TRACK_DURATION = 3;
    private static final double FILTER_DURATION = 3;
    private static final double TRACK_OUT_POINT_THRESHOLD = 0.01;

    public double mCurrentTime;

    public EditManager editManager;
    public ArrayList<MainTrack> mainTracks = new ArrayList<>();
    public ArrayList<TextTrackGroup> textGroups = new ArrayList<>();
    public ArrayList<PIPTrackGroup> pipGroups = new ArrayList<>();
    public ArrayList<FilterAndAdjustGroup> filterGroups = new ArrayList<>();

    public MainTrack currentMainTrack;
    public TextTrack currentTextTrack;
    public StickerTrack currentStickerTrack;
    public MediaTrack currentPIPTrack;
    public FilterAndAdjustEffect currentFilterOrEffect;

    private SurfaceOverlay overlay;

    private boolean clipMod = false;

    private EditData() {

    }

    public MainTrack addMainTrack(MediaTrack mediaTrack) {
        MainTrack track = new MainTrack(mediaTrack);
        mainTracks.add(track);
        return track;
    }

    public static EditData getInstance() {
        return Holder.data;
    }

    public void removeCurrentMainTrack() {
        if (mainTracks.size() <= 1) return;

        editManager.mainGroup().removeTrack(currentMainTrack.mediaTrack.getTrackId());
        int currentIndex = mainTracks.indexOf(currentMainTrack);
        double lastTrackTime = currentIndex == 0 ? 0 : mainTracks.get(currentIndex - 1).getOutPoint();
        mainTracks.remove(currentMainTrack);
        for (int i = currentIndex; i < mainTracks.size(); i++) {
            mainTracks.get(i).updateInPoint();
        }

        if (currentIndex == mainTracks.size()) {
            lastTrackTime -= TRACK_OUT_POINT_THRESHOLD;
        }

        editManager.seek((float) lastTrackTime);
        if (mEditListener != null) {
            mEditListener.onMainDurationChanged();
        }
        exitClipMod(true);
    }

    public void checkClipTimeRange() {
        if (clipMod) {
            if (mCurrentTime < currentMainTrack.inPoint || mCurrentTime >= currentMainTrack.getOutPoint()) {
                exitClipMod(true);
            }
        }
    }

    public void updateCurrentMainTrack() {
        for (EditData.MainTrack mainTrack : mainTracks) {
            if (mainTrack.getOutPoint() > mCurrentTime) {
                currentMainTrack = mainTrack;
                return;
            }
        }
        currentMainTrack = mainTracks.get(mainTracks.size() - 1);
    }

    public void addNewMainTrack(List<String> paths) {
        exitClipMod(true);
        updateCurrentMainTrack();
        MainTrackGroup mainTrackGroup = editManager.mainGroup();

        double startTime = currentMainTrack.getOutPoint();
        int indexToAdd = mainTracks.indexOf(currentMainTrack) + 1;

        for (String path : paths) {
            MediaTrack mediaTrack = editManager.createMediaTrack(path);
            if (mediaTrack != null && mediaTrack.isValid()) {
                mediaTrack.fitToEditContext(true, 0);
                boolean success = mainTrackGroup.addTrack(mediaTrack, startTime);
                if (success) {
                    MainTrack mainTrack = new MainTrack((mediaTrack));
                    mainTracks.add(indexToAdd, mainTrack);
                    startTime = mainTrack.getOutPoint();
                    indexToAdd++;
                }
            }
        }

        for (int i = indexToAdd; i < mainTracks.size(); i++) {
            mainTracks.get(i).updateInPoint();
        }

        seekTo(currentMainTrack.getOutPoint(), false);
        exitClipMod(true);
        mEditListener.onMainDurationChanged();
    }

    public void copyCurrentMainTrack() {
        MediaTrack newTrack = (MediaTrack) editManager.cloneTrack(currentMainTrack.mediaTrack.getTrackId());
        if (newTrack != null && newTrack.isValid()) {
            MainTrackGroup mainGroup = editManager.mainGroup();

            double startTime = currentMainTrack.getOutPoint();
            int currentIndex = mainTracks.indexOf(currentMainTrack);
            boolean success = mainGroup.addTrack(newTrack, startTime);
            if (success) {
                MainTrack mainTrack = new MainTrack(newTrack);
                mainTracks.add(currentIndex + 1, mainTrack);

                for (int i = currentIndex + 2; i < mainTracks.size(); i++) {
                    mainTracks.get(i).updateInPoint();
                }

                editManager.seek(currentMainTrack.getOutPoint() - TRACK_OUT_POINT_THRESHOLD);
                mEditListener.onMainDurationChanged();
            }
        }
    }

    public void splitCurrentMainTrack() {
        MainTrackGroup mainGroup = editManager.mainGroup();
        MediaTrack newTrack = (MediaTrack) mainGroup.splitTrack(currentMainTrack.mediaTrack.getTrackId(), mCurrentTime - currentMainTrack.mediaTrack.inPoint());
        if (newTrack != null) {
            int currentIndex = mainTracks.indexOf(currentMainTrack);
            MainTrack mainTrack = new MainTrack(newTrack);
            mainTracks.add(currentIndex + 1, mainTrack);

            currentMainTrack.updateDuration();
            for (int i = currentIndex + 2; i < mainTracks.size(); i++) {
                mainTracks.get(i).updateInPoint();
            }

            mEditListener.onMainDurationChanged();
        }
    }

    public boolean editing = false;
    public void setEditing(boolean editing) {
        overlay.showTextFrame(currentTextTrack);
    }

    public void createTextTrack() {
        final TextTrack textTrack = editManager.createTextTrack(TEXT_TRACK_DURATION);
        if (textTrack != null && textTrack.isValid()) {
            textTrack.setFontFile("/system/fonts/NotoSansCJK-Regular.ttc");
            textTrack.setFontSize(100);
            textTrack.setContent("输入文字");

            double inPoint = mCurrentTime;
            double duration = textTrack.duration();

            TextTrackGroup group = null;
            for (TextTrackGroup textGroup : textGroups) {
                boolean canAddToThis = true;

                for (Track track : textGroup.tracks) {
                    if (!(track.inPoint() >= inPoint + duration || track.inPoint() + track.duration() <= mCurrentTime)) {
                        canAddToThis = false;
                        break;
                    }
                }

                if (canAddToThis) {
                    group = textGroup;
                    break;
                }
            }

            if (group == null) {
                TrackGroup textGroup = editManager.addNewGroup();
                TextTrackGroup textTrackGroup = new TextTrackGroup();
                textTrackGroup.group = textGroup;
                textGroups.add(textTrackGroup);
                group = textTrackGroup;
            }

            Size size = editManager.getSize();
            textTrack.setPosition(new float[]{size.getWidth() / 2f, size.getHeight() / 2f});
            group.group.addTrack(textTrack, inPoint);

            int addIndex = 0;
            for (int i = group.tracks.size() - 1; i >= 0; i--) {
                Track track = group.tracks.get(i);
                if (track.inPoint() + track.duration() <= inPoint) {
                    addIndex = i + 1;
                    break;
                }
            }

            group.tracks.add(addIndex, textTrack);
            currentTextTrack = textTrack;

            editManager.updateCurrentFrame(new EditManager.UpdateListener() {
                @Override
                public void onUpdateFinish() {
                    editing = true;
                    overlay.showTextFrame(textTrack);
                }
            });
        }
    }

    public void createStickerTrack(String stickerPath) {
        StickerTrack stickerTrack = editManager.createStickerTrack(stickerPath, 0);
        if (stickerTrack != null && stickerTrack.isValid()) {
            double inPoint = mCurrentTime;
            double duration = stickerTrack.duration();

            TextTrackGroup group = null;
            for (TextTrackGroup textGroup : textGroups) {
                boolean canAddToThis = true;

                for (Track track : textGroup.tracks) {
                    if (!(track.inPoint() >= inPoint + duration || track.inPoint() + track.duration() <= mCurrentTime)) {
                        canAddToThis = false;
                        break;
                    }
                }

                if (canAddToThis) {
                    group = textGroup;
                    break;
                }
            }

            if (group == null) {
                TrackGroup textGroup = editManager.addNewGroup();
                TextTrackGroup textTrackGroup = new TextTrackGroup();
                textTrackGroup.group = textGroup;
                textGroups.add(textTrackGroup);
                group = textTrackGroup;
            }

            Size size = editManager.getSize();
            stickerTrack.setPosition(new float[]{size.getWidth() / 2f, size.getHeight() / 2f});
            group.group.addTrack(stickerTrack, inPoint);

            int addIndex = 0;
            for (int i = group.tracks.size() - 1; i >= 0; i--) {
                Track track = group.tracks.get(i);
                if (track.inPoint() + track.duration() <= inPoint) {
                    addIndex = i + 1;
                    break;
                }
            }

            group.tracks.add(addIndex, stickerTrack);
            currentStickerTrack = stickerTrack;

            editManager.updateCurrentFrame();
        }

    }

    public void createPIPTrack(String path) {
        MediaTrack mediaTrack = editManager.createMediaTrack(path);
        if (mediaTrack != null && mediaTrack.isValid()) {
            double inPoint = mCurrentTime;
            double duration = mediaTrack.duration();

            PIPTrackGroup group = null;
            for (PIPTrackGroup pipGroup : pipGroups) {
                boolean canAddToThis = true;

                for (Track track : pipGroup.tracks) {
                    if (!(track.inPoint() >= inPoint + duration || track.inPoint() + track.duration() <= mCurrentTime)) {
                        canAddToThis = false;
                        break;
                    }
                }

                if (canAddToThis) {
                    group = pipGroup;
                    break;
                }
            }

            if (group == null) {
                TrackGroup textGroup = editManager.addNewGroup();
                PIPTrackGroup pipGroup = new PIPTrackGroup();
                pipGroup.group = textGroup;
                pipGroups.add(pipGroup);
                group = pipGroup;
            }

            mediaTrack.fitToEditContext(false, 40);
            group.group.addTrack(mediaTrack, inPoint);

            int addIndex = 0;
            for (int i = group.tracks.size() - 1; i >= 0; i--) {
                Track track = group.tracks.get(i);
                if (track.inPoint() + track.duration() <= inPoint) {
                    addIndex = i + 1;
                    break;
                }
            }

            group.tracks.add(addIndex, mediaTrack);
            currentPIPTrack = mediaTrack;

            editManager.updateCurrentFrame();

            mEditListener.onMainDurationChanged();
        }
    }

    public void addVideoEffect(String effectPath) {
        VideoEffect videoEffect = currentMainTrack.mediaTrack.addVideoEffect(effectPath);
        videoEffect.setStartTime(mCurrentTime);

    }

    public void setOverlay(SurfaceOverlay overlay) {
        this.overlay = overlay;
    }

    public void testSelectMainTrack(int index) {
        int i = index % mainTracks.size();
        String trackId = mainTracks.get(i).mediaTrack.getTrackId();
        Log.d(TAG, "testSelectMainTrack: " + i + ", id: " + trackId);
        selectMainTrack(trackId);
    }

    public void selectMainTrack(String trackId) {
        if (!TextUtils.isEmpty(trackId)) {
            MainTrack selected = null;
            for (MainTrack mainTrack : mainTracks) {
                if (mainTrack.mediaTrack.getTrackId().equals(trackId)) {
                    selected = mainTrack;
                    break;
                }
            }

            if (selected != null) {
                if (selected != currentMainTrack) {
                    if (currentMainTrack == null || selected.inPoint > currentMainTrack.inPoint) {
                        currentMainTrack = selected;
                        seekTo(selected.inPoint, false);
                    } else {
                        seekTo(selected.getOutPoint() - TRACK_OUT_POINT_THRESHOLD, false);
                    }
                }
                enterClipMode();
            }
        }
    }

    public void seekTo(double time) {
        seekTo(time, true);
    }

    public void seekTo(double time, boolean checkClipRange) {
        mCurrentTime = time;
        editManager.seek(time);
        if (checkClipRange) {
            checkClipTimeRange();
        }
    }

    public void enterClipMode() {
        clipMod = true;
        updateCurrentMainTrack();
        overlay.showTrackFrame(currentMainTrack.mediaTrack);
    }

    public void exitClipMod() {
        exitClipMod(false);
    }

    private void exitClipMod(boolean notifyUI) {
        if (clipMod) {
            clipMod = false;

            if (notifyUI && mEditListener != null) {
                mEditListener.onExitClipMod();
            }
        }
    }

    public boolean setMainTrackInPoint(MainTrack track, double inPoint) {
        int index = mainTracks.indexOf(track);
        if (index < 0 || track.inPoint == inPoint) {
            return false;
        }

        MainTrackGroup mainTrackGroup = editManager.mainGroup();
        boolean success = mainTrackGroup.setTrackInPoint(track.mediaTrack.getTrackId(), inPoint);
        if (success) {
            track.updateInPoint();
            track.updateDuration();

            updateMainTracksInPoint(index + 1);
            seekTo(track.inPoint, false);
            mEditListener.onMainDurationChanged();
        }
        return success;
    }

    public boolean setMainTrackDuration(MainTrack track, double duration) {
        int index = mainTracks.indexOf(track);
        if (index < 0 || track.duration == duration) {
            return false;
        }

        MainTrackGroup mainTrackGroup = editManager.mainGroup();
        boolean success = mainTrackGroup.setTrackDuration(track.mediaTrack.getTrackId(), duration);
        if (success) {
            track.updateDuration();

            updateMainTracksInPoint(index + 1);
            seekTo(track.getOutPoint() - TRACK_OUT_POINT_THRESHOLD, false);
            mEditListener.onMainDurationChanged();
        }
        return success;
    }

    private void updateMainTracksInPoint(int fromIndex) {
        for (int i = Math.max(0, fromIndex); i < mainTracks.size(); i++) {
            mainTracks.get(i).updateInPoint();
        }
    }

    public void selectTextOrStickerTrack(Track track) {
        if (track == null) {
            overlay.showTextFrame(null);
            return;
        }

        if (track instanceof TextTrack) {
            currentTextTrack = (TextTrack) track;
            overlay.showTextFrame((TextTrack) track);
        } else {
            currentStickerTrack = (StickerTrack) track;
        }
    }

    public void changeTrackLeftBound(Track track, double time) {
        double outPoint = track.inPoint() + track.duration();
        String groupId = track.getGroupId();
        TrackGroup group = editManager.group(groupId);
        group.offsetTrack(track.getTrackId(), time);
        group.setTrackDuration(track.getTrackId(), outPoint - time);
    }

    public void changeTrackRightBound(Track track, double time) {
        String groupId = track.getGroupId();
        TrackGroup group = editManager.group(groupId);
        group.setTrackDuration(track.getTrackId(), time - track.startTime());
    }

    public void applyFilter(String path, String filterName, boolean isGlobalFilter) {
        if (isGlobalFilter) {
            if (currentFilterOrEffect == null) {
                FilterAndAdjustGroup group = getFilterGroupCanAddTo(mCurrentTime, FILTER_DURATION);
                if (group == null) {
                    group = new FilterAndAdjustGroup();
                    filterGroups.add(group);
                }

                FilterEffect filter = editManager.mainGroup().addFilter(path);
                filter.setStartTime(mCurrentTime);
                filter.setDuration(FILTER_DURATION);

                FilterAndAdjustEffect effect = new FilterAndAdjustEffect();
                effect.effect = filter;
                effect.effectName = filterName;
                effect.effectId = filter.getEffectId();
                effect.type = filter.type();
                currentFilterOrEffect = effect;

                group.effects.add(effect);
            } else {
                FilterAndAdjustGroup filterGroup = null;
                FilterAndAdjustEffect filterEffect = currentFilterOrEffect;
                for (FilterAndAdjustGroup group : filterGroups) {
                    for (FilterAndAdjustEffect effect : group.effects) {
                        if (effect.effectName.equals(currentFilterOrEffect.effectName)) {
                            filterGroup = group;
                            break;
                        }
                    }
                    if (filterGroup != null)
                        break;
                }

                Effect old = filterEffect.effect;
                double startTime = old.startTime();
                double duration = old.duration();
                editManager.mainGroup().removeFilter(filterEffect.effectId);

                FilterEffect newFilter = editManager.mainGroup().addFilter(path);
                newFilter.setStartTime(startTime);
                newFilter.setDuration(duration);

                filterEffect.effect = newFilter;
                filterEffect.effectName = filterName;
                filterEffect.effectId = newFilter.getEffectId();
                filterEffect.type = newFilter.type();
            }
        }
    }

    private FilterAndAdjustGroup getFilterGroupCanAddTo(double startTime, double duration) {
        for (FilterAndAdjustGroup group : filterGroups) {
            boolean canAddToThis = true;
            for (FilterAndAdjustEffect effect : group.effects) {
                if (startTime >= effect.effect.startTime() || startTime + duration < effect.effect.startTime()) {
                    continue;
                }
                canAddToThis = false;
                break;
            }

            if (canAddToThis) {
                return group;
            }
        }
        return null;
    }

    private static class Holder {
        private static EditData data = new EditData();
    }

    public static class MainTrack {
        public MediaTrack mediaTrack;
        public double startTime;
        public double inPoint;
        public double duration;
        public double resourceDuration; //素材时长，图片为0
        public FilterEffect filter;
        public String transitionId;//转场路径

        public MainTrack(MediaTrack mediaTrack) {
            this.mediaTrack = mediaTrack;
            startTime = mediaTrack.startTime();
            inPoint = mediaTrack.inPoint();
            duration = mediaTrack.duration();
            resourceDuration = mediaTrack.resourceDuration();
        }

        public void updateInPoint() {
            startTime = mediaTrack.startTime();
            inPoint = mediaTrack.inPoint();
        }

        public void updateDuration() {
            duration = mediaTrack.duration();
        }

        public double getOutPoint() {
            return inPoint + duration;
        }

        public String getTransitionId(){
            return transitionId;
        }
    }

    public static class TextTrackGroup {
        public TrackGroup group;
        public ArrayList<Track> tracks = new ArrayList<>();
    }

    public static class PIPTrackGroup {
        public TrackGroup group;
        public ArrayList<MediaTrack> tracks = new ArrayList<>();
    }

    public static class FilterAndAdjustEffect {
        public Effect effect;
        public String effectId;
        public String effectName;
        public SXEffect.SXEffectType type;
    }

    public static class FilterAndAdjustGroup {
        public List<FilterAndAdjustEffect> effects = new ArrayList<>();
    }

    public interface EditListener {
        void onMainDurationChanged();

        void onExitClipMod();
    }

    private EditListener mEditListener;

    public void setEditListener(EditListener editListener) {
        mEditListener = editListener;
    }
}
