package com.shixing.edit.effects;

import android.os.Bundle;
import android.view.View;
import android.widget.ImageView;

import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.R;
import com.shixing.edit.adapter.StickerAdapter;
import com.shixing.edit.adapter.TitleAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.utils.ToastUtil;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.VideoEffect;

import java.util.ArrayList;
import java.util.List;

public class TrackEffectsFragment extends BaseFragment<OnActionClickListener> implements OnActionClickListener {

    private final String folderName = "视频特效";
    private RecyclerView mTitle_recycler;
    private RecyclerView mItem1_recycler;
    private ArrayList<String> titleList;
    private ArrayList<ActionItem> itemArrayList;
    private ImageView iv_submit;
    private ImageView iv_cancel;
    private VideoEffect videoEffect;

    @Override
    protected int initRootView() {
        return R.layout.fragment_effect;
    }

    @Override
    protected void initArguments(Bundle arguments) {

    }

    @Override
    protected void initView() {
        mTitle_recycler = mRootView.findViewById(R.id.title_recycler);
        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        iv_submit=mRootView.findViewById(R.id.iv_submit);
        iv_cancel=mRootView.findViewById(R.id.iv_cancel);

        titleList = new ArrayList<>();
        titleList.add("基础");
//        titleList.add("梦幻");
//        titleList.add("动感");
//        titleList.add("复古");
//        titleList.add("基础");
//        titleList.add("梦幻");
//        titleList.add("动感");
//        titleList.add("复古");

        TitleAdapter adapter = new TitleAdapter(this.mActivity);
        adapter.updateData(titleList);
        adapter.setActionClickListener(this);
        mTitle_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mTitle_recycler.setLayoutManager(layoutManager);
        showItem1();

        Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
        List<VideoEffect> videoEffects = ((MediaTrack) track).getVideoEffects();
        if (videoEffects != null && videoEffects.size() > 0) {
            videoEffect = videoEffects.get(0);
        }
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        iv_submit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (listener!=null){
                    ActionItem actionItem=new ActionItem("完成",R.drawable.icon_wancheng);
                    actionItem.id="effect_selected";
                    listener.onActionClick(actionItem);
                }
            }
        });
        iv_cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                ToastUtil.showToast(getContext(),"取消特效");
            }
        });
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });
    }

    @Override
    public void onActionClick(ActionItem action) {
        switch (action.actionType) {
            case ActionItem.text:
                switch (action.actionName) {
                    case "基础":
                        showItem1();
                        break;
                    case "梦幻":
                        showItem1();
                        break;
                    case "动感":
                        showItem1();
                        break;
                    case "复古":
                        showItem1();
                        break;
                    default:
                        break;
                }
                break;
//            case "effect":
//                applyEffect(action);
//                break;
            default:
                applyEffect(action);
                break;
        }
    }

    private void applyEffect(ActionItem action) {
        String effectPath = action.actionType;
        if (videoEffect != null) {
            videoEffect.setResource(effectPath);
        } else {
            Track track = TrackActionManager.getInstance().getCurrentSelectTrack();
            VideoEffect videoEffect = ((MediaTrack) track).addVideoEffect(effectPath);
            if (videoEffect != null) {
                videoEffect.setEffectFollowType(SXEffect.SXEffectTimeFollowType.FollowWholeProcess);
                ActionManager.getInstance().getListener().updateCurrentFrame();
            } else {
                ToastUtil.showToast(getContext(), "添加失败");
            }
        }
        TrackActionManager.getInstance().hideDetailFragment();
    }

    private void showItem1() {
        itemArrayList = new ArrayList<>();

        itemArrayList.add(new ActionItem("特效1", Constants.assetsPath + "/" + folderName + "/v1.gif", Constants.resourcePath + "/" + folderName + "/v1"));
        itemArrayList.add(new ActionItem("特效2", Constants.assetsPath + "/" + folderName + "/v2.gif", Constants.resourcePath + "/" + folderName + "/v2"));
        itemArrayList.add(new ActionItem("特效3", Constants.assetsPath + "/" + folderName + "/v3.gif", Constants.resourcePath + "/" + folderName + "/v3"));
        itemArrayList.add(new ActionItem("特效4", Constants.assetsPath + "/" + folderName + "/v4.gif", Constants.resourcePath + "/" + folderName + "/v4"));
        itemArrayList.add(new ActionItem("特效5", Constants.assetsPath + "/" + folderName + "/v5.gif", Constants.resourcePath + "/" + folderName + "/v5"));
        itemArrayList.add(new ActionItem("心跳", Constants.assetsPath + "/" + folderName + "/心跳.gif", Constants.resourcePath + "/" + folderName + "/心跳"));
        itemArrayList.add(new ActionItem("抖动", Constants.assetsPath + "/" + folderName + "/抖动.gif", Constants.resourcePath + "/" + folderName + "/抖动"));
        itemArrayList.add(new ActionItem("灵魂出窍", Constants.assetsPath + "/" + folderName + "/灵魂出窍.gif", Constants.resourcePath + "/" + folderName + "/灵魂出窍"));
        itemArrayList.add(new ActionItem("split2", Constants.assetsPath + "/" + folderName + "/split2.gif", Constants.resourcePath + "/" + folderName + "/split2"));
        itemArrayList.add(new ActionItem("split4", Constants.assetsPath + "/" + folderName + "/split4.gif", Constants.resourcePath + "/" + folderName + "/split4"));
        itemArrayList.add(new ActionItem("split9", Constants.assetsPath + "/" + folderName + "/split9.gif", Constants.resourcePath + "/" + folderName + "/split9"));

        StickerAdapter adapter = new StickerAdapter(this.mActivity);
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);
        GridLayoutManager layoutManager = new GridLayoutManager(this.mActivity, 4);
        mItem1_recycler.setLayoutManager(layoutManager);
    }
}
