package com.shixing.edit.filter;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.MotionEvent;
import android.view.View;
import android.widget.SeekBar;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.R;
import com.shixing.edit.adapter.FilterAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.data.EditData;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.utils.ToastUtil;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.FilterEffect;
import com.shixing.sxedit.internal.MainTrackGroup;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;

import java.util.ArrayList;

public class FilterFragment extends BaseFragment implements OnActionClickListener, View.OnClickListener {

    private static final String KEY_IS_MAIN_GROUP_FILTER = "is global filter";
    private static final String KEY_FILTER_NAME = "filter name";
    private final String folderName = "滤镜";

    private RecyclerView mItem1_recycler;
    private ArrayList<ActionItem> itemArrayList;

    private SeekBar mSeekBar;
    private boolean isMainGroupFilter;
    private String filterName;
    private View seekbarGroup;
    private FilterEffect effect;

    public static FilterFragment getInstance(boolean isMainGroupFilter, String filterName) {
        FilterFragment fragment = new FilterFragment();
        Bundle bundle = new Bundle();
        bundle.putBoolean(KEY_IS_MAIN_GROUP_FILTER, isMainGroupFilter);
        bundle.putString(KEY_FILTER_NAME, filterName);
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    protected int initRootView() {
        return R.layout.fragment_filter;
    }

    @Override
    protected void initArguments(Bundle arguments) {
        isMainGroupFilter = arguments.getBoolean(KEY_IS_MAIN_GROUP_FILTER);
        filterName = arguments.getString(KEY_FILTER_NAME);
    }

    @Override
    protected void initView() {
        if (isMainGroupFilter)
            findViewById(R.id.group_apply_all).setVisibility(View.GONE);
        seekbarGroup = findViewById(R.id.group_seekbar);

        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        showItem1(R.drawable.icon_lvjing);

        mRootView.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                return true;
            }
        });
        findViewById(R.id.tv_applyall).setOnClickListener(this);
        findViewById(R.id.iv_submit).setOnClickListener(this);

        mSeekBar = (SeekBar) findViewById(R.id.seekBar);
        mSeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (fromUser) {
                    updateFilterAlpha(progress / 100f);
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        effect = (FilterEffect) TrackActionManager.getInstance().getCurrentSelectEffect();
        if (effect != null) {
            seekbarGroup.setVisibility(View.VISIBLE);
            float alpha = effect.alpha();
            mSeekBar.setProgress((int) (alpha * 100));
        }
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        findViewById(R.id.iv_submit).setOnClickListener(view ->
                TrackActionManager.getInstance().hideDetailFragment());
    }

    @Override
    public void onActionClick(ActionItem action) {
        double currentTime = ActionManager.getInstance().getListener().getCurrentTime();
        MainTrackGroup mainGroup = ActionManager.getInstance().getListener().getMainGroup();
        String filterPath = action.actionType;

        if (effect != null) {
            effect.setResource(filterPath);
        } else {
            effect = mainGroup.addFilter(filterPath);
            if (effect != null) {
                effect.setStartTime(currentTime);
                if (TrackActionManager.getInstance().canBeAddEffect(effect)){
                    TrackActionManager.getInstance().addEffect(effect);
                    TrackActionManager.getInstance().onEffectSelect(effect,false);
                }else {
                    mainGroup.removeFilter(effect.getEffectId());
                    effect=null;
                    ToastUtil.showToast(getContext(),"已经超出最大层级，不能在当前位置添加");
                }
            }
        }

        seekbarGroup.setVisibility(effect == null ? View.GONE : View.VISIBLE);
        ActionManager.getInstance().getListener().updateCurrentFrame();
    }


    private void updateFilterAlpha(float alpha) {
        effect.setAlpha(alpha);
        ActionManager.getInstance().getListener().updateCurrentFrame();
    }


    private void showItem1(int resId) {
        itemArrayList = new ArrayList<>();
        itemArrayList.add(new ActionItem("滤镜1", Constants.assetsPath + "/" + folderName + "/f1.png", Constants.resourcePath + "/" + folderName + "/f1"));
        itemArrayList.add(new ActionItem("滤镜2", Constants.assetsPath + "/" + folderName + "/f2.png", Constants.resourcePath + "/" + folderName + "/f2"));
        itemArrayList.add(new ActionItem("滤镜3", Constants.assetsPath + "/" + folderName + "/f3.png", Constants.resourcePath + "/" + folderName + "/f3"));
        itemArrayList.add(new ActionItem("滤镜4", Constants.assetsPath + "/" + folderName + "/f4.png", Constants.resourcePath + "/" + folderName + "/f4"));
        itemArrayList.add(new ActionItem("滤镜5", Constants.assetsPath + "/" + folderName + "/f5.png", Constants.resourcePath + "/" + folderName + "/f5"));
        itemArrayList.add(new ActionItem("滤镜6", Constants.assetsPath + "/" + folderName + "/f6.png", Constants.resourcePath + "/" + folderName + "/f6"));
        itemArrayList.add(new ActionItem("滤镜7", Constants.assetsPath + "/" + folderName + "/f7.png", Constants.resourcePath + "/" + folderName + "/f7"));
        itemArrayList.add(new ActionItem("滤镜8", Constants.assetsPath + "/" + folderName + "/f8.png", Constants.resourcePath + "/" + folderName + "/f8"));
        itemArrayList.add(new ActionItem("滤镜9", Constants.assetsPath + "/" + folderName + "/f9.png", Constants.resourcePath + "/" + folderName + "/f9"));
        itemArrayList.add(new ActionItem("滤镜10", Constants.assetsPath + "/" + folderName + "/f10.png", Constants.resourcePath + "/" + folderName + "/f10"));

        FilterAdapter adapter = new FilterAdapter(this.mActivity);
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);

        if (!TextUtils.isEmpty(filterName)) {
            for (int i = 0; i < itemArrayList.size(); i++) {
                ActionItem item = itemArrayList.get(i);
                if (item.actionName.equals(filterName)) {
//                    adapter.setSelectedPosition(i);
                    break;
                }
            }
            seekbarGroup.setVisibility(View.VISIBLE);
        } else {
//            if (isMainGroupFilter)
//                adapter.setSelectedPosition(0);
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.tv_applyall:
//                applyToAll();
                break;
            case R.id.iv_submit:
                break;
        }
    }
}
