package com.shixing.edit.filter;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.SeekBar;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.R;
import com.shixing.edit.adapter.FilterAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.FilterEffect;
import com.shixing.sxedit.internal.MediaTrack;

import java.util.ArrayList;
import java.util.List;

public class MediaFilterFragment extends BaseFragment implements OnActionClickListener, View.OnClickListener {
    private RecyclerView mItem1_recycler;
    private ArrayList<ActionItem> itemArrayList;

    private SeekBar mSeekBar;
    private View seekbarGroup;
    private FilterEffect effect;
    private MediaTrack track;
    private String folderName = "滤镜";

    @Override
    protected int initRootView() {
        return R.layout.fragment_media_filter;
    }

    @Override
    protected void initArguments(Bundle arguments) {
    }

    @Override
    protected void initView() {
        seekbarGroup = findViewById(R.id.group_seekbar);

        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        showItem1(R.drawable.icon_lvjing);

        findViewById(R.id.tv_applyall).setOnClickListener(this);
        findViewById(R.id.iv_submit).setOnClickListener(this);

        mSeekBar = (SeekBar) findViewById(R.id.seekBar);
        mSeekBar.setMax(100);

        track = (MediaTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        List<FilterEffect> filters = track.getFilters();
        if (filters != null && filters.size() > 0) {
            effect = filters.get(0);
        }

        seekbarGroup.setVisibility(effect == null ? View.GONE : View.VISIBLE);
        if (effect != null) {
            mSeekBar.setProgress((int) (effect.alpha() * 100));
        }
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        mSeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (fromUser && effect != null) {
                    effect.setAlpha(progress / 100f);
                    ActionManager.getInstance().getListener().updateCurrentFrame();
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });
    }

    @Override
    public void onActionClick(ActionItem action) {
        String filterPath = action.actionType;
        if (TextUtils.isEmpty(filterPath)) {
            removeFilter();
        } else {
            if (effect != null) {
                effect.setResource(filterPath);
            } else {
                effect = track.addFilter(filterPath);
                if (effect != null) {
                    effect.setEffectFollowType(SXEffect.SXEffectTimeFollowType.FollowWholeProcess);
                }
            }
        }

        if (effect != null) {
            mSeekBar.setProgress((int) (effect.alpha() * 100));
        }

        seekbarGroup.setVisibility("".equals(action.srcPath) ? View.GONE : View.VISIBLE);
        ActionManager.getInstance().getListener().updateCurrentFrame();
    }

    private void removeFilter() {
        if (effect != null) {
            track.removeFilter(effect.getEffectId());
            effect = null;
        }
    }


    private void showItem1(int resId) {
        itemArrayList = new ArrayList<>();
        itemArrayList.add(new ActionItem("原图", "", ""));
        itemArrayList.add(new ActionItem("滤镜1", Constants.assetsPath + "/" + folderName + "/f1.png", Constants.resourcePath + "/" + folderName + "/f1"));
        itemArrayList.add(new ActionItem("滤镜2", Constants.assetsPath + "/" + folderName + "/f2.png", Constants.resourcePath + "/" + folderName + "/f2"));
        itemArrayList.add(new ActionItem("滤镜3", Constants.assetsPath + "/" + folderName + "/f3.png", Constants.resourcePath + "/" + folderName + "/f3"));
        itemArrayList.add(new ActionItem("滤镜4", Constants.assetsPath + "/" + folderName + "/f4.png", Constants.resourcePath + "/" + folderName + "/f4"));
        itemArrayList.add(new ActionItem("滤镜5", Constants.assetsPath + "/" + folderName + "/f5.png", Constants.resourcePath + "/" + folderName + "/f5"));
        itemArrayList.add(new ActionItem("滤镜6", Constants.assetsPath + "/" + folderName + "/f6.png", Constants.resourcePath + "/" + folderName + "/f6"));
        itemArrayList.add(new ActionItem("滤镜7", Constants.assetsPath + "/" + folderName + "/f7.png", Constants.resourcePath + "/" + folderName + "/f7"));
        itemArrayList.add(new ActionItem("滤镜8", Constants.assetsPath + "/" + folderName + "/f8.png", Constants.resourcePath + "/" + folderName + "/f8"));
        itemArrayList.add(new ActionItem("滤镜9", Constants.assetsPath + "/" + folderName + "/f9.png", Constants.resourcePath + "/" + folderName + "/f9"));
        itemArrayList.add(new ActionItem("滤镜10", Constants.assetsPath + "/" + folderName + "/f10.png", Constants.resourcePath + "/" + folderName + "/f10"));

        FilterAdapter adapter = new FilterAdapter(this.mActivity);
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);

//        if (!TextUtils.isEmpty(filterName)) {
//            for (int i = 0; i < itemArrayList.size(); i++) {
//                ActionItem item = itemArrayList.get(i);
//                if (item.actionName.equals(filterName)) {
//                    adapter.setSelectedPosition(i);
//                    break;
//                }
//            }
//            seekbarGroup.setVisibility(View.VISIBLE);
//        } else {
//            if (isMainGroupFilter)
//                adapter.setSelectedPosition(0);
//        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.tv_applyall:
                applyToAll();
                break;
            case R.id.iv_submit:
                TrackActionManager.getInstance().hideDetailFragment();
                break;
        }
    }

    private void applyToAll() {
        ActionManager.getInstance().getListener().applyFilterToAllMediaTrack(effect);
        ActionManager.getInstance().getListener().updateCurrentFrame();
    }

}
