package com.shixing.edit.manager;

import android.graphics.Matrix;

import com.shixing.sxedit.audio.SXAudioItem;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.AudioItem;
import com.shixing.sxedit.internal.ColorAdjustEffect;
import com.shixing.sxedit.internal.EditManager;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.FilterEffect;
import com.shixing.sxedit.internal.GenericEffect;
import com.shixing.sxedit.internal.MainTrackGroup;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.StickerTrack;
import com.shixing.sxedit.internal.TextTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;
import com.shixing.sxedit.track.SXTrack;


import java.util.List;

public interface ActionListener {

    MediaTrack addMainTrack(String sourcePath);

    List<MediaTrack> addMainTrack(List<String> sourcePaths);

    Track addTrack(@SXTrack.SXTrackType int type, String source);

    MediaTrack addPIPTrack(String sourcePath);

    TextTrack addTextTrack(String content);

    StickerTrack addStickerTrack(String sourcePath);

    List<MediaTrack> getMainTracks();

    List<TrackGroup> getPIPGroups();

    List<TrackGroup> getTextStickerGroups();

    void removeTrack(Track track);

    float getSpeed(MediaTrack track);

    float changeSpeed(MediaTrack track, float speed);

    void horizontalFlip(Track track);

    void verticalFlip(Track track);

    void rotate(Track track);

    /**
     * 设置轨道不透明度
     */
    void setOpacity(Track track, float opacity);

    void cropTrack(MediaTrack track, int width, int height, Matrix matrix);

    void setVolume(MediaTrack track, float volume);

    float getVolume(MediaTrack track);

    void muteVideo(boolean mute);

    Track copyTrack(Track track);

    MediaTrack copyMainTrack(MediaTrack track);

    Track splitTrack(Track track);

    boolean setTrackOffset(Track track, double inPoint);

    boolean setMainTrackOffset(Track track, double inPoint);

    boolean cutTrackStart(Track track, double start);

    boolean cutMainTrackStart(MediaTrack track, double start);

    boolean cutTrackEnd(Track track, double end);

    boolean cutMainTrackEnd(MediaTrack track, double end);

    // effect
    // =====================================================

    Effect addEffect(Track track, SXEffect.SXEffectType type, String resourcePath, SXEffect.SXEffectTimeFollowType followType, double startTime, double duration);

    Effect addMainTrackGroupEffect(SXEffect.SXEffectType type, String resourcePath, SXEffect.SXEffectTimeFollowType followType, double startTime, double duration);

    double addTransition(String trackId, String resourcePath, double duration);

    boolean removeEffect(Track track, Effect effect);

    void removeMainTrackEffect(Effect effect);

    void removeTransition(String trackId);

    boolean hasTransition(MediaTrack track);

    void resetEditSize(int width, int height);


    void setBackgroundColor(int color);
    void setBackgroundColor(float red, float green, float blue);
    void setBackgroundResource(String path);
    void removeBackground();

    SXAudioItem addAudioItem(String path);

    void setAudioSpeed(SXAudioItem audioItem, float speed);

    void setAudioStart(SXAudioItem audioItem, double start);

    void setAudioEnd(SXAudioItem audioItem, double end);

    SXAudioItem copyAudio(SXAudioItem audioItem);

    SXAudioItem clipAudio(SXAudioItem audioItem);

    void removeAudio(SXAudioItem audioItem);

    void seekTo(double time);


    void play();

    boolean isPlaying();

    void pause();

    void updateCurrentFrame();
    void updateCurrentFrame(EditManager.UpdateListener listener);

    void addPlayStateListener(EditManager.PlayerStateListener listener);

    void removePlayStateListener(EditManager.PlayerStateListener listener);

    void applyFilterToAllMediaTrack(FilterEffect effect);
    void applyColorAdjustToAllMediaTrack(GenericEffect effect);

    double getCurrentTime();

    MainTrackGroup getMainGroup();
    EditManager getEditManager();

    AudioItem addAudioTrack(String musicPath);
    List<List<AudioItem>> getAudioGroups();
    void removeAudioTrack(String audioId);
    AudioItem splitAudio(AudioItem audioItem);
    AudioItem copyAudioTrack(AudioItem audioItem);

    void freeze(MediaTrack track);
}
