package com.shixing.edit.manager;

import android.graphics.Matrix;
import android.util.Size;

import com.shixing.edit.audio.AudioThumbManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.audio.SXAudioItem;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.AnimationEffect;
import com.shixing.sxedit.internal.AudioItem;
import com.shixing.sxedit.internal.ColorAdjustEffect;
import com.shixing.sxedit.internal.EditManager;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.FilterEffect;
import com.shixing.sxedit.internal.GenericEffect;
import com.shixing.sxedit.internal.MainTrackGroup;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.StickerTrack;
import com.shixing.sxedit.internal.TextAnimationEffect;
import com.shixing.sxedit.internal.TextTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackAnimationEffect;
import com.shixing.sxedit.internal.TrackGroup;
import com.shixing.sxedit.internal.VideoEffect;
import com.shixing.sxedit.track.SXTrack;

import java.util.ArrayList;
import java.util.List;

public class ActionListenerImpl implements ActionListener, EditManager.PlayerStateListener {

    public static double TEXT_TRACK_DURATION = 3;
    public static int MAX_PIP_GROUP_COUNT = 6;
    public static int MAX_TEXT_STICKER_GROUP_COUNT = 6;
    public static int MAX_AUDIO_TRACK_GROUP_COUNT = 6;

    private EditManager editManager;
    private List<EditManager.PlayerStateListener> listeners = new ArrayList<>();

    private MainTrackGroup mainTrackGroup;
    private List<TrackGroup> pipGroups = new ArrayList<>();
    private List<TrackGroup> textStickerGroups = new ArrayList<>();

    private List<List<AudioItem>> audioGroups = new ArrayList<>();

    private double currentTime = 0;

    ActionListenerImpl(EditManager editManager) {
        this.editManager = editManager;
        mainTrackGroup = editManager.mainGroup();
        editManager.setPlayStateListener(this);
    }

    @Override
    public MediaTrack addMainTrack(String sourcePath) {
        MediaTrack mediaTrack = editManager.createMediaTrack(sourcePath);
        if (mediaTrack != null && mediaTrack.isValid()) {
            mediaTrack.fitToEditContext(true, 0);
            mainTrackGroup.addTrack(mediaTrack, currentTime);
            return mediaTrack;
        }
        return null;
    }

    @Override
    public List<MediaTrack> addMainTrack(List<String> sourcePaths) {
        List<MediaTrack> tracks = new ArrayList<>();
        for (int i = sourcePaths.size() - 1; i >= 0; i--) {
            MediaTrack track = addMainTrack(sourcePaths.get(i));
            if (track != null) {
                tracks.add(0, track);
            }
        }
        return tracks;
    }

    @Override
    public List<TrackGroup> getPIPGroups() {
        return pipGroups;
    }

    @Override
    public List<TrackGroup> getTextStickerGroups() {
        return textStickerGroups;
    }

    @Override
    public Track addTrack(@SXTrack.SXTrackType int type, String source) {
        if (type == SXTrack.SX_TRACK_TYPE_MEDIA) {
            return addPIPTrack(source);
        } else if (type == SXTrack.SX_TRACK_TYPE_TEXT) {
            TextTrack track = editManager.createTextTrack(TEXT_TRACK_DURATION);
            if (track != null && track.isValid()) {
                track.setContent(source);
                if (addTextOrStickerTrack(track)) {
                    return track;
                }
                editManager.removeTrack(track.getTrackId(), false);
            }
        } else if (type == SXTrack.SX_TRACK_TYPE_STICKER) {
            StickerTrack track = editManager.createStickerTrack(source, 0);
            if (addTextOrStickerTrack(track)) {
                return track;
            }
            editManager.removeTrack(track.getTrackId(), false);
        }
        return null;
    }

    @Override
    public MediaTrack addPIPTrack(String sourcePath) {
        MediaTrack track = editManager.createMediaTrack(sourcePath);
        if (track != null && track.isValid()) {
            track.fitToEditContext(true, 50);
            if (addPIPTrack(track)) {
                return track;
            }
            editManager.removeTrack(track.getTrackId(), false);
        }
        return null;
    }

    @Override
    public TextTrack addTextTrack(String content) {
        TextTrack track = editManager.createTextTrack(TEXT_TRACK_DURATION);
        if (track != null && track.isValid()) {
            track.setContent(content);
            track.setFontSize(100);
            Size size = editManager.getSize();
            track.setPosition(new float[]{size.getWidth() / 2f, size.getHeight() / 2f});
            if (addTextOrStickerTrack(track)) {
                return track;
            }
            editManager.removeTrack(track.getTrackId(), false);
        }
        return null;
    }

    @Override
    public StickerTrack addStickerTrack(String sourcePath) {
        StickerTrack track = editManager.createStickerTrack(sourcePath, 0);
        if (track != null && track.isValid()) {
            Size size = editManager.getSize();
            track.setPosition(new float[]{size.getWidth() / 2f, size.getHeight() / 2f});
            if (addTextOrStickerTrack(track)) {
                return track;
            }
            editManager.removeTrack(track.getTrackId(), false);
        }
        return null;
    }

    @Override
    public List<MediaTrack> getMainTracks() {
        List<Track> tracks = mainTrackGroup.tracks();
        List<MediaTrack> mediaTracks = new ArrayList<>(tracks.size());
        for (Track track : tracks) {
            mediaTracks.add((MediaTrack) track);
        }
        return mediaTracks;
    }

    @Override
    public void removeTrack(Track track) {
        editManager.removeTrack(track.getTrackId(), true);
    }

    @Override
    public float getSpeed(MediaTrack track) {
        String groupId = track.getGroupId();
        TrackGroup group = editManager.group(groupId);
        return (float) group.mediaTrackSpeed(track.getTrackId());
    }

    @Override
    public float changeSpeed(MediaTrack track, float speed) {
        String groupId = track.getGroupId();
        TrackGroup group = editManager.group(groupId);
        return (float) group.setMediaTrackSpeed(track.getTrackId(), speed);
    }

    @Override
    public void horizontalFlip(Track track) {
        track.setHorizontalFlip(!track.horizontalFlip());
        updateCurrentFrame();
    }

    @Override
    public void verticalFlip(Track track) {
        track.setVerticalFlip(!track.verticalFlip());
        updateCurrentFrame();
    }

    @Override
    public void rotate(Track track) {
        track.setRotation(track.rotation() + 90);
        updateCurrentFrame();
    }

    @Override
    public void setOpacity(Track track, float opacity) {
        if (opacity > 1) opacity = 1;
        if (opacity < 0) opacity = 0;
        track.setOpacity(opacity);
        updateCurrentFrame();
    }

    @Override
    public void cropTrack(MediaTrack track, int width, int height, Matrix matrix) {
    }

    @Override
    public void setVolume(MediaTrack track, float volume) {
        track.setVolume(volume);
    }

    @Override
    public float getVolume(MediaTrack track) {
        return track.getVolume();
    }

    @Override
    public void muteVideo(boolean mute) {
        List<MediaTrack> mediaTracks = getAllMediaTracks();
        for (MediaTrack mediaTrack : mediaTracks) {
            if (!mediaTrack.isStaticImage()) {
                mediaTrack.setVolume(mute ? 0 : 1);
            }
        }
    }

    @Override
    public Track copyTrack(Track track) {
        Track newTrack = editManager.cloneTrack(track.getTrackId());
        if (newTrack != null) {
            if (newTrack.type() == SXTrack.SX_TRACK_TYPE_TEXT || newTrack.type() == SXTrack.SX_TRACK_TYPE_STICKER) {
                if (addTextOrStickerTrack(newTrack, track.inPoint())) {
                    return newTrack;
                }
            } else if (newTrack.type() == SXTrack.SX_TRACK_TYPE_MEDIA) {
                if (addPIPTrack((MediaTrack) newTrack)) {
                    return newTrack;
                }
            }
            editManager.removeTrack(newTrack.getTrackId(), false);
        }

        return null;
    }

    @Override
    public MediaTrack copyMainTrack(MediaTrack track) {
        Track newTrack = editManager.cloneTrack(track.getTrackId());
        if (newTrack != null) {
            if (mainTrackGroup.addTrack(newTrack, track.startTime() + track.duration())) {
                return (MediaTrack) newTrack;
            }
            editManager.removeTrack(newTrack.getTrackId(), false);
        }
        return null;
    }

    @Override
    public Track splitTrack(Track track) {
        String groupId = track.getGroupId();
        TrackGroup group = editManager.group(groupId);
        return group.splitTrack(track.getTrackId(), currentTime - track.inPoint());
    }

    @Override
    public boolean setTrackOffset(Track track, double inPoint) {
        String groupId = track.getGroupId();
        TrackGroup group = editManager.group(groupId);
        return group.offsetTrack(track.getTrackId(), inPoint);
    }

    @Override
    public boolean setMainTrackOffset(Track track, double inPoint) {
        return mainTrackGroup.offsetTrack(track.getTrackId(), inPoint);
    }

    @Override
    public boolean cutTrackStart(Track track, double start) {
        String groupId = track.getGroupId();
        TrackGroup group = editManager.group(groupId);
        if (track.type() == SXTrack.SX_TRACK_TYPE_MEDIA) {
            return group.setTrackInPoint(track.getTrackId(), start);
        } else {
            double oldDuration = track.duration();
            double newDuration = oldDuration + track.inPoint() - start;
            if (group.offsetTrack(track.getTrackId(), start)) {
                return group.setTrackDuration(track.getTrackId(), newDuration);
            }
            return false;
        }

    }

    @Override
    public boolean cutMainTrackStart(MediaTrack track, double start) {
        return mainTrackGroup.setTrackInPoint(track.getTrackId(), start);
    }

    @Override
    public boolean cutTrackEnd(Track track, double end) {
        String groupId = track.getGroupId();
        TrackGroup group = editManager.group(groupId);
        return group.setTrackDuration(track.getTrackId(), end - track.inPoint());
    }

    @Override
    public boolean cutMainTrackEnd(MediaTrack track, double end) {
        return mainTrackGroup.setTrackDuration(track.getTrackId(), end - track.inPoint());
    }

    @Override
    public Effect addEffect(Track track, SXEffect.SXEffectType type, String resourcePath, SXEffect.SXEffectTimeFollowType followType, double startTime, double duration) {
        switch (type) {
            case TextAnimation:
                TextAnimationEffect textAnimationEffect = ((TextTrack) track).addTextAnimation(resourcePath);
                textAnimationEffect.setStartTime(startTime);
                textAnimationEffect.setDuration(duration);
                textAnimationEffect.setEffectFollowType(followType);
                return textAnimationEffect;
            case Filter:
                FilterEffect filterEffect = ((MediaTrack) track).addFilter(resourcePath);
                filterEffect.setStartTime(startTime);
                filterEffect.setDuration(duration);
                filterEffect.setEffectFollowType(followType);
                return filterEffect;
            case Animation:
                AnimationEffect animationEffect = ((MediaTrack) track).addAnimation(resourcePath);
                animationEffect.setStartTime(startTime);
                animationEffect.setDuration(duration);
                animationEffect.setEffectFollowType(followType);
                return animationEffect;
            case VideoEffect:
                VideoEffect videoEffect = ((MediaTrack) track).addVideoEffect(resourcePath);
                videoEffect.setStartTime(startTime);
                videoEffect.setDuration(duration);
                videoEffect.setEffectFollowType(followType);
                return videoEffect;
            case TrackAnimation:
                TrackAnimationEffect trackAnimationEffect = ((StickerTrack) track).addTrackAnimation(resourcePath);
                trackAnimationEffect.setStartTime(startTime);
                trackAnimationEffect.setDuration(duration);
                trackAnimationEffect.setEffectFollowType(followType);
                return trackAnimationEffect;
            default:
                break;
        }
        return null;
    }

    @Override
    public Effect addMainTrackGroupEffect(SXEffect.SXEffectType type, String resourcePath, SXEffect.SXEffectTimeFollowType followType, double startTime, double duration) {
        switch (type) {
            case Filter:
                FilterEffect filterEffect = mainTrackGroup.addFilter(resourcePath);
                filterEffect.setStartTime(startTime);
                filterEffect.setDuration(duration);
                filterEffect.setEffectFollowType(followType);
                return filterEffect;
            case VideoEffect:
                VideoEffect videoEffect = mainTrackGroup.addVideoEffect(resourcePath);
                videoEffect.setStartTime(startTime);
                videoEffect.setDuration(duration);
                videoEffect.setEffectFollowType(followType);
                return videoEffect;
        }
        return null;
    }

    @Override
    public double addTransition(String trackId, String resourcePath, double duration) {
        return mainTrackGroup.addTransition(resourcePath, trackId, duration);
    }

    @Override
    public boolean removeEffect(Track track, Effect effect) {
        return false;
    }

    @Override
    public void removeMainTrackEffect(Effect effect) {
        switch (effect.type()) {
            case Filter:
                mainTrackGroup.removeFilter(effect.getEffectId());
                break;
            case VideoEffect:
                mainTrackGroup.removeVideoEffect(effect.getEffectId());
                break;
            case GenericEffect:
                mainTrackGroup.removeGenericEffect((GenericEffect) effect);
        }
    }

    @Override
    public void removeTransition(String trackId) {
        mainTrackGroup.removeTransition(trackId);
    }

    @Override
    public boolean hasTransition(MediaTrack track) {
        return false;
    }

    @Override
    public void resetEditSize(int width, int height) {

    }

    @Override
    public void setBackgroundColor(int color) {
        editManager.setBackgroundColor(color);
    }

    @Override
    public void setBackgroundColor(float red, float green, float blue) {
        editManager.setBackgroundColor(red, green, blue);
    }

    @Override
    public void setBackgroundResource(String path) {
        editManager.setBackground(path);
    }

    @Override
    public void removeBackground() {
        editManager.removeBackground();
    }

    @Override
    public SXAudioItem addAudioItem(String path) {
        return null;
    }

    @Override
    public void setAudioSpeed(SXAudioItem audioItem, float speed) {

    }

    @Override
    public void setAudioStart(SXAudioItem audioItem, double start) {

    }

    @Override
    public void setAudioEnd(SXAudioItem audioItem, double end) {

    }

    @Override
    public SXAudioItem copyAudio(SXAudioItem audioItem) {
        return null;
    }

    @Override
    public SXAudioItem clipAudio(SXAudioItem audioItem) {
        return null;
    }

    @Override
    public void removeAudio(SXAudioItem audioItem) {

    }

    @Override
    public void seekTo(double time) {
        editManager.seek(time);
    }

    @Override
    public void play() {
        editManager.start();
    }

    @Override
    public boolean isPlaying() {
        return editManager.isPlaying();
    }

    @Override
    public void pause() {
        editManager.pause();
    }

    @Override
    public void updateCurrentFrame() {
        editManager.updateCurrentFrame();
    }

    @Override
    public void updateCurrentFrame(EditManager.UpdateListener listener) {
        editManager.updateCurrentFrame(listener);
    }

    @Override
    public void addPlayStateListener(EditManager.PlayerStateListener listener) {
        listeners.add(listener);
    }

    @Override
    public void removePlayStateListener(EditManager.PlayerStateListener listener) {
        listeners.remove(listener);
    }

    @Override
    public void applyFilterToAllMediaTrack(FilterEffect effect) {
        List<MediaTrack> mediaTracks = getAllMediaTracks();

        for (MediaTrack track : mediaTracks) {
            List<FilterEffect> filters = track.getFilters();
            if (filters != null && filters.size() > 0) {
                if (!filters.get(0).equals(effect)) {
                    track.removeFilter(filters.get(0).getEffectId());
                    track.addFilterByCopy(effect);
                }
            } else  {
                track.addFilterByCopy(effect);
            }
        }
    }

    @Override
    public void applyColorAdjustToAllMediaTrack(GenericEffect effect) {
        List<MediaTrack> mediaTracks = getAllMediaTracks();
        for (MediaTrack mediaTrack : mediaTracks) {
            List<GenericEffect> effects = mediaTrack.getColorAdjusts();
            if (effects != null && effects.size() > 0) {
                if (!effects.get(0).equals(effect)) {
                    mediaTrack.removeGenericEffect(effects.get(0));
                    mediaTrack.addGenericEffectByCopy(effect);
                }
            } else {
                mediaTrack.addGenericEffectByCopy(effect);
            }
        }
    }

    @Override
    public double getCurrentTime() {
        return currentTime;
    }

    @Override
    public MainTrackGroup getMainGroup() {
        return editManager.mainGroup();
    }

    @Override
    public EditManager getEditManager() {
        return editManager;
    }

    @Override
    public AudioItem addAudioTrack(String audioPath) {
        AudioItem audioItem = editManager.createAudioItem(audioPath, currentTime);
        if (audioItem != null) {
            List<AudioItem> group = getAudioGroupToAdd(audioItem);
            if (group != null) {
                addAudioToGroup(audioItem, group);
                AudioThumbManager.getInstance().addAudio(audioPath, audioItem.getAudioId());
            } else {
                editManager.deleteAudioItem(audioItem.getAudioId());
                audioItem = null;
            }
        }
        return audioItem;
    }

    @Override
    public List<List<AudioItem>> getAudioGroups() {
        return audioGroups;
    }

    @Override
    public void removeAudioTrack(String audioId) {
        editManager.deleteAudioItem(audioId);
        for (List<AudioItem> audioGroup : audioGroups) {
            for (AudioItem audioItem : audioGroup) {
                if (audioItem.getAudioId().equals(audioId)) {
                    audioGroup.remove(audioItem);
                    AudioThumbManager.getInstance().removeAudio(audioId);
                    return;
                }
            }
        }
    }

    @Override
    public AudioItem splitAudio(AudioItem audioItem) {
        if (audioItem != null) {
            AudioItem newAudio = editManager.splitAudioItem(audioItem.getAudioId(), currentTime - audioItem.getInPoint());
            if (newAudio != null) {
                audioItem.updateInfo();
                for (List<AudioItem> audioGroup : audioGroups) {
                    for (AudioItem item : audioGroup) {
                        if (item.getAudioId().equals(audioItem.getAudioId())) {
                            int index = audioGroup.indexOf(item);
                            audioGroup.add(index + 1, newAudio);
                        }
                    }
                }
                AudioThumbManager.getInstance().addAudio(newAudio.getFilePath(), newAudio.getAudioId());
            }
            return newAudio;
        }
        return null;
    }

    @Override
    public AudioItem copyAudioTrack(AudioItem audioItem) {
        if (audioItem != null) {
            AudioItem newAudio = editManager.cloneAudioItem(audioItem.getAudioId());
            List<AudioItem> group = getAudioGroupToAdd(newAudio);
            if (group != null) {
                addAudioToGroup(newAudio, group);
                AudioThumbManager.getInstance().addAudio(newAudio.getFilePath(), newAudio.getAudioId());
                return newAudio;
            } else {
                editManager.deleteAudioItem(newAudio.getAudioId());
                return null;
            }
        }
        return null;
    }

    @Override
    public void freeze(MediaTrack track) {
        if (!track.inMainTrackGroup()) {
            String groupId = track.getGroupId();
            TrackGroup group = editManager.group(groupId);
            if (group != null) {
                Track nextTrack = group.findNextTrack(track.getTrackId());
                if (nextTrack != null) {

                }
            } else {
                track.freeze(currentTime);
            }
        }
    }

    private void addAudioToGroup(AudioItem audioItem, List<AudioItem> group) {
        int index = 0;
        for (int i = 0; i < group.size(); i++) {
            AudioItem item = group.get(i);
            if (item.getInPoint() >= audioItem.getInPoint() + audioItem.getDuration()) {
                index = i;
                break;
            }
        }
        group.add(index, audioItem);
    }

    private List<AudioItem> getAudioGroupToAdd(AudioItem audioItem) {
        if (audioGroups.size() == 0) {
            ArrayList<AudioItem> group = new ArrayList<>();
            audioGroups.add(group);
            return group;
        }

        List<AudioItem> group = null;
        for (List<AudioItem> audioGroup : audioGroups) {
            boolean canAddToThisGroup = true;
            for (AudioItem item : audioGroup) {
                if (!(item.getInPoint() >= audioItem.getInPoint() + audioItem.getDuration()) && !(item.getInPoint() + item.getDuration() <= audioItem.getInPoint())) {
                    canAddToThisGroup = false;
                    break;
                }
            }
            if (canAddToThisGroup) {
                group = audioGroup;
                break;
            }
        }

        if (group == null) {
            if (audioGroups.size() < MAX_AUDIO_TRACK_GROUP_COUNT) {
                group = new ArrayList<>();
                audioGroups.add(group);
            }
        }
        return group;
    }

    private List<MediaTrack> getAllMediaTracks() {
        List<MediaTrack> mediaTracks = getMainTracks();
        List<TrackGroup> pipGroups = getPIPGroups();
        for (TrackGroup pipGroup : pipGroups) {
            List<Track> tracks = pipGroup.tracks();
            for (Track track : tracks) {
                mediaTracks.add((MediaTrack) track);
            }
        }
        return mediaTracks;
    }

    @Override
    public void onPlayFrame(int frameIndex, double time) {
        currentTime = time;
        for (EditManager.PlayerStateListener listener : listeners) {
            listener.onPlayFrame(frameIndex, time);
        }
    }

    @Override
    public void onPlayFinished() {
        for (EditManager.PlayerStateListener listener : listeners) {
            listener.onPlayFinished();
        }
    }

    private int getNextPIPGroupIndex() {
        return pipGroups.size();
    }

    private int getNextTextStickerGroupIndex() {
        return pipGroups.size() + textStickerGroups.size();
    }

    private boolean addPIPTrack(MediaTrack track) {
        if (track != null && track.isValid()) {
            for (TrackGroup pipGroup : pipGroups) {
                if (pipGroup.addTrack(track, currentTime)) {
                    return true;
                }
            }
            if (pipGroups.size() < MAX_PIP_GROUP_COUNT) {
                TrackGroup group = editManager.insertNewGroupAt(getNextPIPGroupIndex());
                if (group != null) {
                    pipGroups.add(group);
                    if (group.addTrack(track, currentTime)) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    private boolean addTextOrStickerTrack(Track track, double time) {
        if (track != null && track.isValid()) {
            for (TrackGroup group : textStickerGroups) {
                if (group.addTrack(track, time)) {
                    return true;
                }
            }
            if (textStickerGroups.size() < MAX_TEXT_STICKER_GROUP_COUNT) {
                TrackGroup group = editManager.insertNewGroupAt(getNextTextStickerGroupIndex());
                if (group != null) {
                    textStickerGroups.add(group);
                    if (group.addTrack(track, time)) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    private boolean addTextOrStickerTrack(Track track) {
        return addTextOrStickerTrack(track, currentTime);
    }
}
