package com.shixing.edit.multitrack;

import android.util.Log;

import com.shixing.edit.data.TrackBean;
import com.shixing.edit.data.TrackSourceObject;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.utils.Logger;
import com.shixing.edit.widget.TrackGroupView;
import com.shixing.edit.widget.VideoTrackView;
import com.shixing.sxedit.internal.AudioItem;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;

import java.util.ArrayList;
import java.util.List;

public class TrackActionManager implements TrackViewListener,TrackActionListener{


    private static TrackActionManager instance;
    private List<TrackViewListener> trackViewList=new ArrayList<>();
    private List<TrackActionListener> trackActionListenerList=new ArrayList<>();
    private Track mCurrentSelectTrack;
    private Effect mCurrentSelectEffect;
    private AudioItem mCurrentSelectAudio;
    public static TrackActionManager getInstance(){
        if (instance==null)
            instance=new TrackActionManager();
        return instance;
    }

    public void putTrackView(TrackViewListener trackView){
        if (!trackViewList.contains(trackView)){
            trackViewList.add(trackView);
        }
    }
    public void removeTrackView(TrackViewListener trackView){
        trackViewList.remove(trackView);
    }
    public void putTrackActionListener(TrackActionListener trackActionListener){
        if (!trackActionListenerList.contains(trackActionListener)){
            trackActionListenerList.add(trackActionListener);
        }
    }

    public void removeTrackActionListener(TrackActionListener trackActionListener){
        trackActionListenerList.remove(trackActionListener);
    }
    public Track getCurrentSelectTrack(){
        return mCurrentSelectTrack;
    }
    public Effect getCurrentSelectEffect(){
        return mCurrentSelectEffect;
    }
    public AudioItem getCurrentSelectAudio(){
        return mCurrentSelectAudio;
    }

    /**获取当前指针的track**/
    public void getCursorSelectMainTrack(){
        for (TrackViewListener trackView:trackViewList){
            if (trackView instanceof VideoTrackView){
                Track track=((VideoTrackView)trackView).getCursorSelectMainTrack(TrackUtil.getInstance().getMainTrackScrollX()+TrackConfig.SCREEN_WIDTH_HALF);
                TrackActionManager.getInstance().onTrackSelect(track,false);
            }
        }
    }
    @Override
    public void updateMaxDuration(double duration, boolean isActionUp) {
        for (TrackViewListener trackView:trackViewList){
            trackView.updateMaxDuration(duration,isActionUp);
        }
    }

    @Override
    public void updateScaleSize(float scale) {
        for (TrackViewListener trackView:trackViewList){
            trackView.updateScaleSize(scale);
        }
    }

    @Override
    public boolean isClicked() {
        return false;
    }

    @Override
    public void cancelSelectTrackByClick() {
//        mCurrentSelectTrack=null;
        for (TrackViewListener trackView:trackViewList){
            trackView.cancelSelectTrackByClick();
        }
    }

    @Override
    public void cancelSelectEffectByClick() {
//        mCurrentSelectEffect=null;
        for (TrackViewListener trackView:trackViewList){
            trackView.cancelSelectEffectByClick();
        }
    }

    @Override
    public void cancelSelectAudioByClick() {
        for (TrackViewListener trackView:trackViewList){
            trackView.cancelSelectAudioByClick();
        }
    }

    @Override
    public void updateTrack(Track track) {
        for (TrackViewListener trackView:trackViewList){
            trackView.updateTrack(track);
        }
    }

    @Override
    public void updateMainTrack(MediaTrack mainTrack) {
        for (TrackViewListener trackView:trackViewList){
            trackView.updateMainTrack(mainTrack);
        }
    }


    @Override
    public void updateEffect(Effect effect) {
        for (TrackViewListener trackView:trackViewList){
            trackView.updateEffect(effect);
        }
    }

    @Override
    public void updateTrackDuration(Track track) {
        for (TrackViewListener trackView:trackViewList){
            trackView.updateTrackDuration(track);
        }
    }

    @Override
    public void updateMainTrackDuration(MediaTrack mainTrack) {
        for (TrackViewListener trackView:trackViewList){
            trackView.updateMainTrackDuration(mainTrack);
        }
    }


    @Override
    public void updateEffectDuration(Effect effect) {
        for (TrackViewListener trackView:trackViewList){
            trackView.updateEffectDuration(effect);
        }
    }

    @Override
    public void removeTrack(Track track) {
        for (TrackViewListener trackView:trackViewList){
            trackView.removeTrack(track);
        }
    }

    @Override
    public void removeMainTrack(MediaTrack mediaTrack) {
        for (TrackViewListener trackView:trackViewList){
            trackView.removeMainTrack(mediaTrack);
        }
    }


    @Override
    public void removeEffect(Effect effect) {
        for (TrackViewListener trackView:trackViewList){
            trackView.removeEffect(effect);
        }
    }

    @Override
    public void addTrack(Track track) {
        for (TrackViewListener trackView:trackViewList){
            trackView.addTrack(track);
        }
    }

    @Override
    public void addMainTrack(List<MediaTrack> mainTracks) {
        for (TrackViewListener trackView:trackViewList){
            trackView.addMainTrack(mainTracks);
        }
    }

    @Override
    public void addEffect(Effect effect) {
        for (TrackViewListener trackView:trackViewList){
            trackView.addEffect(effect);
        }
    }

    @Override
    public void addTrackGroup(List<TrackGroup> trackGroups, int trackType) {
        for (TrackViewListener trackView:trackViewList){
            trackView.addTrackGroup(trackGroups,trackType);
        }
    }

    @Override
    public void addEffectList(List<Effect> effectList, int trackType) {
        for (TrackViewListener trackView:trackViewList){
            trackView.addEffectList(effectList,trackType);
        }
    }


    @Override
    public void addTrackBeanSource(TrackSourceObject sourceObject) {
        for (TrackViewListener trackView:trackViewList){
            trackView.addTrackBeanSource(sourceObject);
        }
    }

    @Override
    public void onProgressChanged(double time) {
        for (TrackViewListener trackViewListener : trackViewList) {
            trackViewListener.onProgressChanged(time);
        }
    }

    @Override
    public boolean canBeAddEffect(Effect effect) {
        for (TrackViewListener trackViewListener : trackViewList) {
            if (trackViewListener instanceof TrackGroupView){//特效才能用这个调用
                return trackViewListener.canBeAddEffect(effect);
            }
        }
        return false;
    }


    @Override
    public void onAudioSelect(AudioItem audioItem, boolean isSWitch) {
        mCurrentSelectAudio=audioItem;
        if (mCurrentSelectAudio!=null)
            cancelSelectEffectByClick();
        if (mCurrentSelectTrack!=null)
            cancelSelectTrackByClick();
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onAudioSelect(audioItem,isSWitch);
        }
    }

    @Override
    public void onAudioUnSelect(AudioItem audioItem) {
        mCurrentSelectAudio=null;
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onAudioUnSelect(audioItem);
        }
    }

    @Override
    public void onMovingUp() {
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onMovingUp();
        }
    }

    @Override
    public void onMovingDown() {
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onMovingDown();
        }
    }

    @Override
    public void onTouchScroll() {
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onTouchScroll();
        }
    }


    @Override
    public void onTrackSelect(Track track,boolean isSwitch) {
        mCurrentSelectTrack=track;
        if (mCurrentSelectEffect!=null)
        cancelSelectEffectByClick();
        if (mCurrentSelectAudio!=null)
        cancelSelectAudioByClick();
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onTrackSelect(track,isSwitch);
        }
    }

    @Override
    public void onEffectSelect(Effect effect,boolean isSwitch) {
        mCurrentSelectEffect=effect;
        if (mCurrentSelectTrack!=null)
        cancelSelectTrackByClick();
        if (mCurrentSelectAudio!=null)
        cancelSelectAudioByClick();
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onEffectSelect(effect,isSwitch);
        }

    }


    @Override
    public void onEffectUnSelect(Effect effect) {
        mCurrentSelectEffect=null;
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onEffectUnSelect(effect);
        }
    }


    public void onTrackUnSelect(Track track) {
        mCurrentSelectTrack=null;
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onTrackUnSelect(track);
        }
    }

    @Override
    public void onClickTransition(Track track) {
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.onClickTransition(track);
        }
    }

    @Override
    public void setAudioGroup(List<List<AudioItem>> groups) {
        for (TrackViewListener trackViewListener : trackViewList) {
            trackViewListener.setAudioGroup(groups);
        }
    }

    public void cutTrackOrEffect(TrackBean trackBean, int touchPoint) {
        double start = trackBean.mStartX ;
        double end = trackBean.mEndX ;
        Log.d("TAG", "cutTrack: start=" + start + ", end=" + end + ", duration=" + trackBean.duration+",touchPoint="+trackBean.touchPoint);

        if (trackBean.track!=null){
            if (trackBean.track instanceof MediaTrack){
                MediaTrack mediaTrack= (MediaTrack) trackBean.track;
                if (mediaTrack.inMainTrackGroup()) {
                    if (touchPoint == 1) {
                        double startDurationIndex = TrackUtil.getInstance().getDurationByLength(trackBean.mStartX - TrackConfig.SCREEN_WIDTH_HALF);
                        ActionManager.getInstance().getListener().cutMainTrackStart(mediaTrack, startDurationIndex);
                    } else if (touchPoint == 2) {
                        double endDurationIndex = TrackUtil.getInstance().getDurationByLength(trackBean.mEndX - TrackConfig.SCREEN_WIDTH_HALF);
                        ActionManager.getInstance().getListener().cutMainTrackEnd(mediaTrack, endDurationIndex);
                    }else {
                        Logger.d("onTrackSeek touchPoint=0");
                    }
                }else {
                    cutTrackOrEffect(trackBean, mediaTrack,touchPoint);
                }
            }else {
                cutTrackOrEffect(trackBean, trackBean.track,touchPoint);
            }
        }else if (trackBean.effect!=null){
            cutTrackOrEffect(trackBean, trackBean.effect,touchPoint);
        }else if (trackBean.audioItem!=null){
            cutAudio(trackBean,trackBean.audioItem,touchPoint);
        }
    }

    private void cutTrackOrEffect(TrackBean trackBean, Track track,int touchPoint) {
        if (touchPoint == 1) {
            double startDurationIndex = TrackUtil.getInstance().getDurationByLength(trackBean.mStartX - TrackConfig.SCREEN_WIDTH_HALF);
            ActionManager.getInstance().getListener().cutTrackStart(track, startDurationIndex);
        } else if (touchPoint == 2) {
            double endDurationIndex = TrackUtil.getInstance().getDurationByLength(trackBean.mEndX - TrackConfig.SCREEN_WIDTH_HALF);
            ActionManager.getInstance().getListener().cutTrackEnd(track, endDurationIndex);
        }
    }
    private void cutTrackOrEffect(TrackBean trackBean, Effect effect,int touchPoint) {
        if (touchPoint == 1) {
            double startDurationIndex = TrackUtil.getInstance().getDurationByLength(trackBean.mStartX - TrackConfig.SCREEN_WIDTH_HALF);
            effect.setStartTime(startDurationIndex);
            double duration = TrackUtil.getInstance().getDurationByLength(trackBean.mEndX - trackBean.mStartX);
            effect.setDuration(duration);
        } else if (touchPoint == 2) {
            double duration = TrackUtil.getInstance().getDurationByLength(trackBean.mEndX - trackBean.mStartX);
            effect.setDuration(duration);
        }
    }
    private void cutAudio(TrackBean trackBean, AudioItem audioItem,int touchPoint) {
        if (touchPoint == 1) {
            double startDurationIndex = TrackUtil.getInstance().getDurationByLength(trackBean.mStartX - TrackConfig.SCREEN_WIDTH_HALF);
            audioItem.setInPoint((float) startDurationIndex);
        } else if (touchPoint == 2) {
            double duration = TrackUtil.getInstance().getDurationByLength(trackBean.mEndX - trackBean.mStartX);
            audioItem.setDuration((float) duration);
        }
    }
    public void hideDetailFragment(){
        for (TrackActionListener trackActionListener:trackActionListenerList){
            trackActionListener.hideDetailFragment();
        }
    }

    @Override
    public void updateTrackAnimation() {
        for (TrackViewListener listener : trackViewList) {
            listener.updateTrackAnimation();
        }
    }

    @Override
    public void showTrackAnimationMask(boolean show) {
        for (TrackViewListener listener : trackViewList) {
            listener.showTrackAnimationMask(show);
        }
    }

    @Override
    public void onAudioThumbChanged(String audioId, int[] data) {
        for (TrackViewListener listener : trackViewList) {
            if (listener instanceof TrackGroupView) {
                listener.onAudioThumbChanged(audioId, data);
            }
        }
    }

    @Override
    public void onTrackTextChanged(String trackId, String text) {
        for (TrackViewListener listener : trackViewList) {
            listener.onTrackTextChanged(trackId, text);
        }
    }
}
