package com.shixing.edit.multitrack;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.drawable.Drawable;
import android.media.MediaMetadataRetriever;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;

import androidx.annotation.NonNull;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.bitmap_recycle.BitmapPool;
import com.bumptech.glide.load.resource.bitmap.BitmapTransformation;
import com.bumptech.glide.request.RequestOptions;
import com.bumptech.glide.request.target.Target;
import com.shixing.edit.Myapplication;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.utils.AppExecutors;
import com.shixing.edit.utils.Logger;
import com.shixing.edit.utils.ScreenUtil;
import com.shixing.edit.utils.Utils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import static com.bumptech.glide.load.resource.bitmap.VideoDecoder.FRAME_OPTION;

public class TrackUtil {
    private static TrackUtil instance;
    private int mainTrackScrollX;//当前滚动位置，应用于插入素材点
    private TrackUtil(){}
    public static TrackUtil getInstance(){
        if (instance==null)
            instance=new TrackUtil();
        return instance;
    }

    public int getDurationWidth(double duration){
        Paint paint= new Paint();
        paint.setTextSize(ScreenUtil.INSTANCE.dp2px(9f));
        double textHalfWidth=paint.measureText("00:00");
        double unitSize=TrackConfig.THUMB_WIDTH+textHalfWidth;
        return (int)(duration*1f * unitSize/2f);//默认两秒一个间隔
    }
    //根据距离换算时长
    public double getDurationByLength(double length){
        Paint paint= new Paint();
        paint.setTextSize(ScreenUtil.INSTANCE.dp2px(9f));
        double textHalfWidth=paint.measureText("00:00");
        double unitSize=TrackConfig.THUMB_WIDTH+textHalfWidth;
        return (length/unitSize*2);
    }

    public  double getLocalVideoDuration(String videoPath) {
        double duration;
        try {
            MediaMetadataRetriever mmr = new MediaMetadataRetriever();
            mmr.setDataSource(videoPath);
            duration = Integer.parseInt(mmr.extractMetadata
                    (MediaMetadataRetriever.METADATA_KEY_DURATION));
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
        return duration;
    }
    public void getVideoBitmap (String trackId,String uri,int viewHeight,int length, double interval, OnFrameLoadListener frameLoadListener){
        for (int i = 0; i <length ; i++) {
            final int t=i;
            AppExecutors.getInstance().networkIO().execute(new Runnable() {
                @Override
                public void run() {
                    if (uri.substring(uri.lastIndexOf('.')).equalsIgnoreCase(".gif")) {
                        Bitmap tmp = BitmapFactory.decodeFile(uri);
                        Bitmap bitmap = Utils.centerSquareScaleBitmap(tmp, viewHeight);
                        frameLoadListener.onFrameLoadFinish(trackId,t,bitmap);
                    } else {
                        MediaMetadataRetriever retriever = new MediaMetadataRetriever();
                        retriever.setDataSource(uri);
                        Bitmap tmp = retriever.getFrameAtTime((long) (t * interval * 1000 * 1000));
                        if (tmp != null) {
                            Bitmap mBitmap = Utils.centerSquareScaleBitmap(tmp, viewHeight);
                            tmp.recycle();
                            frameLoadListener.onFrameLoadFinish(trackId, t, mBitmap);
                        }
                        retriever.release();
                    }
                }
            });
//            loadVideoScreenshot(Myapplication.getInstance(),uri,t*1000*1000,new BitmapTransformation() {
//                @Override
//                protected Bitmap transform(@NonNull BitmapPool pool, @NonNull Bitmap toTransform, int outWidth, int outHeight) {
//                    Bitmap mBitmap=Utils.centerSquareScaleBitmap(toTransform, viewHeight);
//                    frameLoadListener.onFrameLoadFinish(trackId,t,mBitmap);
//                    return toTransform;
//                }
//                @Override
//                public void updateDiskCacheKey(MessageDigest messageDigest) {
//                    try {
//                        messageDigest.update((Myapplication.getInstance().getPackageName() + "RotateTransform").getBytes("utf-8"));
//                    } catch (Exception e) {
//                        e.printStackTrace();
//                    }
//                }
//            });
        }
    }

    public static void loadVideoScreenshot(final Context context, String uri,  long frameTimeMicros,BitmapTransformation transformation) {
        RequestOptions requestOptions = RequestOptions.frameOf(frameTimeMicros);
        requestOptions.set(FRAME_OPTION, MediaMetadataRetriever.OPTION_CLOSEST);
        requestOptions.transform(transformation);
        Glide.with(context).load(uri).apply(requestOptions).submit();
    }
    public interface OnFrameLoadListener{
        public void onFrameLoadFinish(String trackId,int index,Bitmap bitmap);
    }
    /**
     * 横向拼接
     * <功能详细描述>

     * @return 
     */
    public Bitmap add2Bitmap(Bitmap[] bitmaps,int itemHeight,int itemWidth) {
        Bitmap result = Bitmap.createBitmap(itemWidth*bitmaps.length, itemHeight, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(result);
        for (int i = 0; i < bitmaps.length; i++) {
            canvas.drawBitmap(bitmaps[i], itemWidth*i, 0, null);
        }
        return result;
    }

    public int getMainTrackScrollX(){
        return mainTrackScrollX;
    }
    public void setMainTrackScrollX(int scrollX){
        this.mainTrackScrollX=scrollX;
    }

    HashMap<Integer,Double> maxDurationMap=new HashMap<>();
    /**
     * 更新最大时长
     * @param isActionUp 是否是touch up 事件，为true,需要把布局最终确认下来，如果是缩小，需要把scrollview总宽度减少
     * **/
    public void updateMaxTrackDuration(int from,double maxDuration,boolean isActionUp){
//        maxDurationMap.put(from,maxDuration);
//        Collection<Double> c =maxDurationMap.values();
//        double maxDu=Collections.max(c);
        double maxDu = ActionManager.getInstance().getListener().getEditManager().editDuration();
        TrackActionManager.getInstance().updateMaxDuration(maxDu,isActionUp);
    }

    public static Bitmap saveBitmapToPng(Bitmap bitmap, String path) {
        if (!path.endsWith(".png") && !path.endsWith(".PNG")) {
            throw new IllegalArgumentException();
        }

        File file = new File(path);
        if (file.exists()) {
            file.delete();
        }

        FileOutputStream out = null;
        try {
            out = new FileOutputStream(file);
            bitmap.compress(Bitmap.CompressFormat.PNG, 100, out);
            out.flush();
            out.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        return bitmap;
    }
}
