package com.shixing.edit.sticker;

import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.R;
import com.shixing.edit.adapter.AnimationStickerAdapter;
import com.shixing.edit.adapter.TitleAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.StickerTrack;
import com.shixing.sxedit.internal.TrackAnimationEffect;

import java.util.ArrayList;
import java.util.List;

public class AnimaStickerFragment extends BaseFragment<OnActionClickListener> implements OnActionClickListener {
    private static final String TAG = "AnimaStickerFragment";

    private RecyclerView mTitle_recycler;
    private RecyclerView mItem1_recycler;
    private ArrayList<String> titleList;
    private ArrayList<ActionItem> itemArrayList;
    private StickerTrack track;
    private TrackAnimationEffect trackAnimation;

    private SeekBar seekbar;
    private TextView tvDuration;
    private View seekbarGroup;

    private double trackDuration;
    private double maxDuration;
    private SXEffect.SXEffectTimeFollowType followType = SXEffect.SXEffectTimeFollowType.FollowStart;
    private AnimationStickerAdapter adapter;

    private final String folderName = "轨道动画";
    private final String[] effectType = {"入场动画", "出场动画", "循环动画"};
    private List<ActionItem> list1;
    private List<ActionItem> list2;
    private List<ActionItem> list3;

    @Override
    protected int initRootView() {
        return R.layout.fragment_sticker_anim;
    }

    @Override
    protected void initArguments(Bundle arguments) {

    }

    @Override
    protected void initView() {
        mTitle_recycler = mRootView.findViewById(R.id.title_recycler);
        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
        seekbar = findViewById(R.id.seekBar);
        tvDuration = findViewById(R.id.tv_seekbar_num);
        seekbarGroup = findViewById(R.id.group_seekbar);

        titleList = new ArrayList<>();
        titleList.add("入场动画");
        titleList.add("出场动画");
        titleList.add("循环动画");

        TitleAdapter typeAdapter = new TitleAdapter(this.mActivity);
        typeAdapter.updateData(titleList);
        typeAdapter.setActionClickListener(this);
        mTitle_recycler.setAdapter(typeAdapter);
        LinearLayoutManager typeLayoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mTitle_recycler.setLayoutManager(typeLayoutManager);

        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);
        adapter = new AnimationStickerAdapter(this.mActivity);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);

        initList();
        adapter.updateData(list1);


        track = (StickerTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        List<TrackAnimationEffect> trackAnimations = track.getTrackAnimations();

        seekbar.setMax(100);
        seekbar.setProgress(50);
        trackDuration = track.duration();
        maxDuration = trackDuration / 2;
        tvDuration.setText(String.format("%.1fs", maxDuration / 2));
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mRootView.findViewById(R.id.iv_submit).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (listener!=null){
                    ActionItem actionItem=new ActionItem("完成",R.drawable.icon_wancheng);
                    actionItem.id="sticker_selected";
                    listener.onActionClick(actionItem);
                }
            }
        });
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        seekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (fromUser) {
                    double duration = progress / 100f * maxDuration;
                    tvDuration.setText(String.format("%.1fs", duration));
                    if (trackAnimation.followType() != SXEffect.SXEffectTimeFollowType.FollowWholeProcess) {
                        trackAnimation.setDuration(duration);
                    }

                    trackAnimation.setDurationOfOneCycle(duration);
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });
    }

    @Override
    public void onActionClick(ActionItem action) {
        switch (action.actionType) {
            case ActionItem.text:
                switch (action.actionName) {
                    case "入场动画":
                        maxDuration = trackDuration / 2;
                        adapter.updateData(list1);
                        followType = SXEffect.SXEffectTimeFollowType.FollowStart;
                        break;
                    case "出场动画":
                        maxDuration = trackDuration / 2;
                        adapter.updateData(list2);
                        followType = SXEffect.SXEffectTimeFollowType.FollowEnd;
                        break;
                    case "循环动画":
                        maxDuration = trackDuration;
                        adapter.updateData(list3);
                        followType = SXEffect.SXEffectTimeFollowType.FollowWholeProcess;
                        break;
                    default:
                        break;
                }
                seekbarGroup.setVisibility(View.GONE);
                adapter.setSelected0();
                break;
//            case "anim":
//                applyAnim(action);
//                break;
            default:
                applyAnim(action);
                break;
        }
    }

    private void applyAnim(ActionItem action) {
        if (TextUtils.isEmpty(action.srcPath)) {
            seekbarGroup.setVisibility(View.GONE);
            List<TrackAnimationEffect> animations = track.getTrackAnimations();
            if (animations != null) {
                for (TrackAnimationEffect animation : animations) {
                    if (animation.followType() == followType) {
                        track.removeTrackAnimation(animation.getEffectId());
                    }
                }
            }
        } else {
            String animPath = action.actionType;
            TrackAnimationEffect anim = null;
            List<TrackAnimationEffect> animations = track.getTrackAnimations();
            if (animations != null) {
                for (TrackAnimationEffect animation : animations) {
                    if ((followType == SXEffect.SXEffectTimeFollowType.FollowStart || followType == SXEffect.SXEffectTimeFollowType.FollowEnd) && animation.followType() == SXEffect.SXEffectTimeFollowType.FollowWholeProcess) {
                        track.removeTrackAnimation(animation.getEffectId());
                    } else if (followType == SXEffect.SXEffectTimeFollowType.FollowWholeProcess && (animation.followType() == SXEffect.SXEffectTimeFollowType.FollowStart || animation.followType() == SXEffect.SXEffectTimeFollowType.FollowEnd)) {
                        track.removeTrackAnimation(animation.getEffectId());
                    } else if (animation.followType() == followType) {
                        animation.setResource(animPath);
                        animation.setDurationOfOneCycle(animation.duration());
                        anim = animation;
                    }
                }
            }

            if (anim == null) {
                anim = track.addTrackAnimation(animPath);
                if (anim != null) {
                    anim.setEffectFollowType(followType);
                    if (anim.resourceDuration() > maxDuration) {
                        anim.setDuration(maxDuration);
                        anim.setDurationOfOneCycle(maxDuration);
                    }
                }
            }

            if (anim != null) {
                trackAnimation = anim;
                seekbarGroup.setVisibility(View.VISIBLE);
                seekbar.setProgress((int) (anim.duration() / maxDuration * 100));
                tvDuration.setText(String.format("%.1fs", anim.duration()));
            }
        }
    }


    private void initList() {
        list1 = initList(effectType[0], "se");
        list2 = initList(effectType[1], "sx");
        list3 = initList(effectType[2], "sl");
    }

    private List<ActionItem> initList(String subFolderName, final String name) {
        List<ActionItem> animList = new ArrayList<>();
        String inAinmSubFolder = folderName + "/" + subFolderName;
        animList.add(new ActionItem("无", "", ""));
        animList.add(new ActionItem("动画1", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "1.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "1"));
        animList.add(new ActionItem("动画2", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "2.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "2"));
        animList.add(new ActionItem("动画3", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "3.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "3"));
        animList.add(new ActionItem("动画4", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "4.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "4"));
        animList.add(new ActionItem("动画5", Constants.assetsPath + "/" + inAinmSubFolder + "/" + name + "5.gif", Constants.resourcePath + "/" + inAinmSubFolder + "/" + name + "5"));
        return animList;
    }
}
