package com.shixing.edit.text;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.R;
import com.shixing.edit.adapter.AnimationStickerAdapter;
import com.shixing.edit.adapter.TitleAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.TextAnimationEffect;
import com.shixing.sxedit.internal.TextTrack;
import com.shixing.sxedit.internal.TrackAnimationEffect;

import java.util.ArrayList;
import java.util.List;

public class AnimaTextFragment extends BaseFragment<OnActionClickListener> implements OnActionClickListener {

    private RecyclerView mTitle_recycler;
    private RecyclerView mItem1_recycler;
    private ArrayList<String> titleList;
    private ArrayList<ActionItem> itemArrayList;
    private RecyclerView rv_bottom;
    private TextTrack track;
    private SeekBar seekbar;
    private View seekbarGroup;

    private double trackDuration;
    private double maxDuration;
    private TextView tvDuration;
    private AnimationStickerAdapter adapter;

    private List<ActionItem> list1;
    private List<ActionItem> list2;
    private List<ActionItem> list3;

    private TextAnimationEffect effect;
    private SXEffect.SXEffectTimeFollowType followType = SXEffect.SXEffectTimeFollowType.FollowStart;

    @Override
    protected int initRootView() {
        return R.layout.fragment_text_anim;
    }

    @Override
    protected void initArguments(Bundle arguments) {
    }

    @Override
    protected void initView() {
        mTitle_recycler = mRootView.findViewById(R.id.title_recycler);
        mItem1_recycler = mRootView.findViewById(R.id.item1_recycler);
//        rv_bottom=mRootView.findViewById(R.id.rv_bottom);

        titleList = new ArrayList<>();
        titleList.add("入场动画");
        titleList.add("出场动画");
        titleList.add("循环动画");


        TitleAdapter titleAdapter = new TitleAdapter(this.mActivity);
        titleAdapter.updateData(titleList);
        titleAdapter.setActionClickListener(this);
        mTitle_recycler.setAdapter(titleAdapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mTitle_recycler.setLayoutManager(layoutManager);

        initAnimsListView();
        initList();
        adapter.updateData(list1);

        track = (TextTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        trackDuration = track.duration();

        seekbar = findViewById(R.id.seekBar);
        tvDuration = findViewById(R.id.tv_seekbar_num);
        seekbarGroup = findViewById(R.id.group_seekbar);

        seekbar.setMax(100);
        seekbar.setProgress(50);
        maxDuration = trackDuration / 2;
        tvDuration.setText(String.format("%.1fs", maxDuration / 2));
    }

    private void initAnimsListView() {
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mItem1_recycler.setLayoutManager(layoutManager);
        adapter = new AnimationStickerAdapter(this.mActivity);
        adapter.setActionClickListener(this);
        mItem1_recycler.setAdapter(adapter);
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        seekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (fromUser) {
                    double duration = progress / 100f * maxDuration;
                    tvDuration.setText(String.format("%.1fs", duration));
                    if (effect.followType() != SXEffect.SXEffectTimeFollowType.FollowWholeProcess) {
                        effect.setDuration(duration);
                    }

                    effect.setDurationOfOneCycle(duration);
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });
    }

    private int animType = 0;

    @Override
    public void onActionClick(ActionItem action) {
        switch (action.actionType) {
            case ActionItem.text:
                switch (action.actionName) {
                    case "入场动画":
                        animType = 0;
                        maxDuration = trackDuration / 2;
                        adapter.updateData(list1);
                        followType = SXEffect.SXEffectTimeFollowType.FollowStart;
                        break;
                    case "出场动画":
                        animType = 1;
                        maxDuration = trackDuration / 2;
                        adapter.updateData(list2);
                        followType = SXEffect.SXEffectTimeFollowType.FollowEnd;
                        break;
                    case "循环动画":
                        animType = 2;
                        maxDuration = trackDuration;
                        adapter.updateData(list3);
                        followType = SXEffect.SXEffectTimeFollowType.FollowWholeProcess;
                        break;
                    default:
                        break;
                }
                seekbarGroup.setVisibility(View.GONE);
                adapter.setSelected0();
                break;
            default:
                applyAnim(action);
                break;
        }
    }

    private void applyAnim(ActionItem action) {
        if (TextUtils.isEmpty(action.srcPath)) {
            seekbarGroup.setVisibility(View.GONE);
            List<TextAnimationEffect> animations = track.getTextAnimations();
            if (animations != null) {
                for (TextAnimationEffect animation : animations) {
                    if (animation.followType() == followType) {
                        track.removeTextAnimation(animation.getEffectId());
                    }
                }
            }
        } else {
            String animPath = action.actionType;
            TextAnimationEffect anim = null;
            List<TextAnimationEffect> animations = track.getTextAnimations();
            if (animations != null) {
                for (TextAnimationEffect animation : animations) {
                    if ((followType == SXEffect.SXEffectTimeFollowType.FollowStart || followType == SXEffect.SXEffectTimeFollowType.FollowEnd) && animation.followType() == SXEffect.SXEffectTimeFollowType.FollowWholeProcess) {
                        track.removeTextAnimation(animation.getEffectId());
                    } else if (followType == SXEffect.SXEffectTimeFollowType.FollowWholeProcess && (animation.followType() == SXEffect.SXEffectTimeFollowType.FollowStart || animation.followType() == SXEffect.SXEffectTimeFollowType.FollowEnd)) {
                        track.removeTextAnimation(animation.getEffectId());
                    } else if (animation.followType() == followType) {
                        animation.setResource(animPath);
                        animation.setDurationOfOneCycle(animation.duration());
                        anim = animation;
                    }
                }
            }

            if (anim == null) {
                anim = track.addTextAnimation(animPath);
                if (anim != null) {
                    anim.setEffectFollowType(followType);
                    if (anim.resourceDuration() > maxDuration) {
                        anim.setDuration(maxDuration);
                        anim.setDurationOfOneCycle(maxDuration);
                    }
                }
            }

            if (anim != null) {
                effect = anim;
                seekbarGroup.setVisibility(View.VISIBLE);
                seekbar.setProgress((int) (anim.duration() / maxDuration * 100));
                tvDuration.setText(String.format("%.1fs", anim.duration()));
            }
        }
    }


    private void initList() {
        list1 = initList("入场动画", "te");
        list2 = initList("出场动画", "tx");
        list3 = initList("循环动画", "tl");
    }

    private List<ActionItem> initList(String subFolderName, String name) {
        List<ActionItem> anims = new ArrayList<>();
        String folder = "字符动画";
        String subfolder = folder + "/" + subFolderName;
        anims.add(new ActionItem("无", "", ""));
        anims.add(new ActionItem("动画1", Constants.assetsPath + "/" + subfolder + "/" + name + "1.gif", Constants.resourcePath + "/" + subfolder + "/" + name + "1"));
        anims.add(new ActionItem("动画2", Constants.assetsPath + "/" + subfolder + "/" + name + "2.gif", Constants.resourcePath + "/" + subfolder + "/" + name + "2"));
        anims.add(new ActionItem("动画3", Constants.assetsPath + "/" + subfolder + "/" + name + "3.gif", Constants.resourcePath + "/" + subfolder + "/" + name + "3"));
        anims.add(new ActionItem("动画4", Constants.assetsPath + "/" + subfolder + "/" + name + "4.gif", Constants.resourcePath + "/" + subfolder + "/" + name + "4"));
        anims.add(new ActionItem("动画5", Constants.assetsPath + "/" + subfolder + "/" + name + "5.gif", Constants.resourcePath + "/" + subfolder + "/" + name + "5"));
        return anims;
    }
}
