package com.shixing.edit.text;

import android.os.Bundle;
import android.text.Editable;
import android.text.Selection;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;

import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.EditActivity;
import com.shixing.edit.R;
import com.shixing.edit.adapter.BubbleAdapter;
import com.shixing.edit.adapter.TitleAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.listener.OnFragmentActionListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.internal.EditManager;
import com.shixing.sxedit.internal.TextTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.track.SXTrack;

import java.util.ArrayList;

import static android.content.Context.INPUT_METHOD_SERVICE;

public class TextFragment extends BaseFragment<OnActionClickListener> implements OnActionClickListener, OnFragmentActionListener {

    private final String folderName = "文字气泡";
    private RecyclerView mTitle_recycler;
    private RecyclerView rv_item;
    private ArrayList<String> titleList;
    private ArrayList<ActionItem> itemArrayList;
    private ImageView iv_submit;
    private FrameLayout fl_content;
    private EditText et_input;
    private TextTrack textTrack;

    @Override
    protected int initRootView() {
        return R.layout.fragment_text;
    }

    @Override
    protected void initArguments(Bundle arguments) {
    }

    @Override
    protected void initView() {
        mTitle_recycler = mRootView.findViewById(R.id.title_recycler);
        rv_item = mRootView.findViewById(R.id.rv_item);
        iv_submit=mRootView.findViewById(R.id.iv_submit);
        fl_content=mRootView.findViewById(R.id.fl_content);
        et_input=mRootView.findViewById(R.id.et_input);

        titleList = new ArrayList<>();
        titleList.add("键盘");
        titleList.add("样式");
        titleList.add("气泡");
        titleList.add("动画");


        TitleAdapter adapter = new TitleAdapter(this.mActivity);
        adapter.updateData(titleList);
        adapter.setActionClickListener(this);
        mTitle_recycler.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        mTitle_recycler.setLayoutManager(layoutManager);

        textTrack = (TextTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        et_input.setText(textTrack.content());
        Selection.setSelection(et_input.getText(), textTrack.content().length());

        int type = getArguments().getInt("type", 0);
        if (type == 0) {
            et_input.postDelayed(this::showKeyboard, 500);
        } else {
            adapter.setSelectedPosition(type);
        }
    }

    @Override
    protected void onLazyLoad() {
//        et_input.postDelayed(this::showKeyboard, 500);
    }

    @Override
    protected void initListener() {
        iv_submit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                hideSoftInput();
                et_input.postDelayed(
                        () -> TrackActionManager.getInstance().hideDetailFragment(),
                        0);
            }
        });
        mRootView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        et_input.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                String text = s.length() == 0 ? "输入文字" : s.toString();
                textTrack.setContent(text);
                TrackActionManager.getInstance().onTrackTextChanged(textTrack.getTrackId(), text);
                ActionManager.getInstance().getListener().updateCurrentFrame(new EditManager.UpdateListener() {
                    @Override
                    public void onUpdateFinish() {
                        ((EditActivity) mActivity).onTextChanged();
                    }
                });
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });
    }

    private void hideSoftInput() {
        InputMethodManager imm = (InputMethodManager) mActivity.getSystemService(INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(et_input.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
    }

    @Override
    public void onActionClick(ActionItem action) {
        switch (action.actionType) {
            case ActionItem.text:
                switch (action.actionName) {
                    case "键盘":
//                        showItem1("file:///android_asset/sticker/sticker_1.gif");
                        rv_item.setVisibility(View.GONE);
                        showKeyboard();
                        break;
                    case "样式":
                        hideSoftInput();
                        showTextStyleFragment();
                        break;
                    case "气泡":
                        hideSoftInput();
                        showItem1();
                        break;
                    case "动画":
                        hideSoftInput();
                        showTextAnimFragment();
                        break;
                    default:
                        break;
                }
                break;
            default:
                applyBubble(action);
                break;
        }
    }

    private void showKeyboard() {
        fl_content.setVisibility(View.GONE);
        et_input.requestFocus();
        InputMethodManager imm = (InputMethodManager) mActivity.getSystemService(INPUT_METHOD_SERVICE);
        imm.showSoftInput(et_input, InputMethodManager.SHOW_IMPLICIT);
    }

    private void applyBubble(ActionItem action) {
        if (TextUtils.isEmpty(action.srcPath)) {
            textTrack.removeBubble();
        } else {
            String bubblePath = action.actionType;
            textTrack.setBubble(bubblePath);
        }
        ActionManager.getInstance().getListener().updateCurrentFrame(() -> ((EditActivity) mActivity).onTextChanged());
    }



    private void showItem1() {
        rv_item.setVisibility(View.VISIBLE);
        fl_content.setVisibility(View.GONE);
        itemArrayList = new ArrayList<>();
        itemArrayList.add(new ActionItem("无", "", ""));
        for (int i = 1; i <= 18; i++) {
            itemArrayList.add(new ActionItem("气泡", Constants.assetsPath + "/" + folderName + "/qipao" + i + ".png", Constants.resourcePath + "/" + folderName + "/qipao" + i));
        }

        BubbleAdapter adapter = new BubbleAdapter(this.mActivity);
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        rv_item.setAdapter(adapter);
        GridLayoutManager layoutManager = new GridLayoutManager(this.mActivity, 4);
        rv_item.setLayoutManager(layoutManager);
    }

    private void showTextStyleFragment(){
        rv_item.setVisibility(View.GONE);
        fl_content.setVisibility(View.VISIBLE);
        TextStyleFragment fragment=new TextStyleFragment();
        this.getChildFragmentManager().beginTransaction()
                .add(R.id.fl_content, fragment)
                .commit();
        this.getChildFragmentManager().beginTransaction().show(fragment);
    }
    private void showTextAnimFragment(){
        rv_item.setVisibility(View.GONE);
        fl_content.setVisibility(View.VISIBLE);
        AnimaTextFragment fragment=new AnimaTextFragment();
        this.getChildFragmentManager().beginTransaction()
                .add(R.id.fl_content, fragment)
                .commit();
        this.getChildFragmentManager().beginTransaction().show(fragment);
    }


    @Override
    public void onTrackSelect(Track track, boolean isSwitch) {
        if (isSwitch){
            switch (track.type()) {
                case SXTrack.SX_TRACK_TYPE_TEXT:
                    textTrack= (TextTrack) track;
                    et_input.setText(textTrack.content());
                    break;
                case SXTrack.SX_TRACK_TYPE_STICKER:

                    break;
            }
        }else {
            TrackActionManager.getInstance().hideDetailFragment();
//            ((EditActivity) mActivity).hideSubmenu();
        }

    }

    @Override
    public void onTrackRemove(Track track) {

    }
}
