package com.shixing.edit.text;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.shixing.edit.Constants;
import com.shixing.edit.EditActivity;
import com.shixing.edit.R;
import com.shixing.edit.adapter.AlignmentAdapter;
import com.shixing.edit.adapter.AlignmentAdapter.AlignmentData;
import com.shixing.edit.adapter.BackgroundAdapter;
import com.shixing.edit.adapter.TextStyleAdapter;
import com.shixing.edit.adapter.TitleAdapter;
import com.shixing.edit.base.BaseFragment;
import com.shixing.edit.data.ActionItem;
import com.shixing.edit.listener.OnActionClickListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.sxedit.internal.TextTrack;
import com.shixing.sxedit.track.SXTextTrack;

import java.util.ArrayList;

public class TextStyleFragment extends BaseFragment<OnActionClickListener> implements OnActionClickListener, AlignmentAdapter.AlignmentClickListener {

    private RecyclerView rv_1,rv_2,rv_3,rv_4;
    private ArrayList<ActionItem> itemArrayList;
    private ArrayList<String> strList;
    private ConstraintLayout cl_bottom;
    private TextView mSeekBarName;
    private SeekBar mSeekBar;


    private static float max_stroke_width = 20;
    private TextTrack track;
    private View fl_color;
    private SeekBar seek_spacing;
    private RecyclerView list_alignment;
    private View alignmentGroup;

    @Override
    protected int initRootView() {
        return R.layout.fragment_text_style;
    }

    @Override
    protected void initArguments(Bundle arguments) {
    }

    @Override
    protected void initView() {

        rv_1 = mRootView.findViewById(R.id.rv_1);
        rv_2 = mRootView.findViewById(R.id.rv_2);
        rv_3 = mRootView.findViewById(R.id.rv_3);
        rv_4 = mRootView.findViewById(R.id.rv_4);

//        cl_bottom= mRootView.findViewById(R.id.cl_bottom);
        mSeekBarName = mRootView.findViewById(R.id.tv_bottom_right);
        mSeekBar = mRootView.findViewById(R.id.seekBar);

        track = (TextTrack) TrackActionManager.getInstance().getCurrentSelectTrack();
        float fontSize = track.fontSize();
        max_stroke_width = fontSize / 3f;

        fl_color = findViewById(R.id.fl_color);
        seek_spacing = findViewById(R.id.seekBar_spacing);
        list_alignment = findViewById(R.id.list_text_alignment);
        alignmentGroup = findViewById(R.id.group_alignment);

        int tracking = track.tracking();
        seek_spacing.setProgress(tracking);
        showItem1();
    }

    @Override
    protected void onLazyLoad() {

    }

    @Override
    protected void initListener() {
        mSeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (!fromUser) return;
                if (colorCategory == COLOR_TEXT) {
                    track.setOpacity(progress / 100f);
                } else {
                    track.setStrokeWidth(progress / 100f * max_stroke_width);
                }
                updateSurface();
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        seek_spacing.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                if (fromUser) {
                    track.setTracking(progress);
                    updateSurface();
                    ((EditActivity) mActivity).onTextChanged();
                }
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });
    }

    private void updateSurface() {
        ActionManager.getInstance().getListener().updateCurrentFrame();
    }

    @Override
    public void onActionClick(ActionItem action) {
        switch (action.actionType) {
            case ActionItem.text:
                switch (action.actionName) {
                    case "文本":
                        fl_color.setVisibility(View.VISIBLE);
                        seek_spacing.setVisibility(View.GONE);
                        alignmentGroup.setVisibility(View.GONE);
                        colorCategory = COLOR_TEXT;
                        mSeekBarName.setText("透明度");
                        float opacity = track.opacity();
                        mSeekBar.setProgress((int) (opacity * 100));
                        break;
                    case "描边":
                        fl_color.setVisibility(View.VISIBLE);
                        colorCategory = COLOR_STOKE;
                        mSeekBarName.setText("描边粗细度");
                        float width = track.strokeWidth();
                        mSeekBar.setProgress((int) (width * max_stroke_width));
                        fl_color.setVisibility(View.VISIBLE);
                        seek_spacing.setVisibility(View.GONE);
                        alignmentGroup.setVisibility(View.GONE);
                        break;
                    case "字间距":
                        fl_color.setVisibility(View.GONE);
                        seek_spacing.setVisibility(View.VISIBLE);
                        alignmentGroup.setVisibility(View.GONE);
                        break;
                    case "对齐":
                        fl_color.setVisibility(View.GONE);
                        seek_spacing.setVisibility(View.GONE);
                        alignmentGroup.setVisibility(View.VISIBLE);
                        break;
                    default:
                        break;
                }
                break;
            case "color list":
                applyColor(action.iconResId);
                break;
            case "text style":
                applyTextStyle(action);
                break;
            default:
                break;
        }
    }

    private void applyColor(int iconResId) {
        int color = getResources().getColor(iconResId);

        if (colorCategory == COLOR_TEXT) {
            track.setFillColor(color);
        } else {
            track.setStrokeColor(color);
        }
        updateSurface();
    }

    private void applyTextStyle(ActionItem action) {
        if (!TextUtils.isEmpty(action.srcPath)) {
            track.setTextStyle(action.srcPath);
        } else {
            track.resetTextStyle();
        }
        updateSurface();
    }

    private static final int COLOR_TEXT = 0;
    private static final int COLOR_STOKE = 1;
    private int colorCategory = COLOR_TEXT;


    private void showItem1() {

        itemArrayList = new ArrayList<>();
        itemArrayList.add(new ActionItem("系统", "", ActionItem.resId));
//        itemArrayList.add(new ActionItem("系统", srcPath, ActionItem.resId));
//        itemArrayList.add(new ActionItem("系统", srcPath, ActionItem.resId));
//        itemArrayList.add(new ActionItem("系统", srcPath, ActionItem.resId));
//        itemArrayList.add(new ActionItem("系统", srcPath, ActionItem.resId));
//        itemArrayList.add(new ActionItem("系统", srcPath, ActionItem.resId));
//        itemArrayList.add(new ActionItem("系统", srcPath, ActionItem.resId));
//        itemArrayList.add(new ActionItem("系统", srcPath, ActionItem.resId));
//        itemArrayList.add(new ActionItem("系统", srcPath, ActionItem.resId));

        TextStyleAdapter adapter = new TextStyleAdapter(this.mActivity,"text_style_name");
        adapter.updateData(itemArrayList);
        adapter.setActionClickListener(this);
        rv_1.setAdapter(adapter);
        LinearLayoutManager layoutManager = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        rv_1.setLayoutManager(layoutManager);


        String folder = "文字样式";
        ArrayList<ActionItem> textStyleList = new ArrayList<>();
        textStyleList.add(new ActionItem("", "", "text style"));
        for (int i = 1; i <= 5; i++) {
            textStyleList.add(new ActionItem(Constants.assetsPath + "/" + folder + "/ts" + i + ".png", Constants.resourcePath + "/" + folder + "/ts" + i, "text style"));
        }

        TextStyleAdapter adapter2 = new TextStyleAdapter(this.mActivity,"text_style");
        adapter2.updateData(textStyleList);
        adapter2.setActionClickListener(this);
        rv_2.setAdapter(adapter2);
        LinearLayoutManager layoutManager2 = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        rv_2.setLayoutManager(layoutManager2);

        strList=new ArrayList<>();
        strList.add("文本");
        strList.add("描边");
//        strList.add("标签");
        strList.add("字间距");
        strList.add("对齐");

        TitleAdapter adapter3 = new TitleAdapter(this.mActivity);
        adapter3.updateData(strList);
        adapter3.setActionClickListener(this);
        rv_3.setAdapter(adapter3);
        LinearLayoutManager layoutManager3 = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        rv_3.setLayoutManager(layoutManager3);

        ArrayList<ActionItem> colorList=new ArrayList<>();
        colorList.add(new ActionItem("color0", R.color.bg_color_alpha, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_1, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_2, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_3, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_4, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_5, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_6, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_7, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_8, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_9, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_10, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_11, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_12, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_13, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_14, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_15, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_16, "color list"));
        colorList.add(new ActionItem("color1", R.color.bg_color_17, "color list"));

        BackgroundAdapter adapter4 = new BackgroundAdapter(this.mActivity,"color");
        adapter4.updateData(colorList);
        adapter4.setActionClickListener(this);
        rv_4.setAdapter(adapter4);
        LinearLayoutManager layoutManager4 = new LinearLayoutManager(this.mActivity, RecyclerView.HORIZONTAL, false);
        rv_4.setLayoutManager(layoutManager4);

        ArrayList<AlignmentData> alignmentData = new ArrayList<>();
        alignmentData.add(new AlignmentData("左对齐", R.drawable.icon_zuoduiqi));
        alignmentData.add(new AlignmentData("左右居中", R.drawable.icon_zhongduiqi));
        alignmentData.add(new AlignmentData("右对齐", R.drawable.icon_youduiqi));
        alignmentData.add(new AlignmentData("上对齐", R.drawable.icon_shushangduiqi));
        alignmentData.add(new AlignmentData("上下居中", R.drawable.icon_shuzhongduiqi));
        alignmentData.add(new AlignmentData("下对齐", R.drawable.icon_shuxiaduiqi));

        list_alignment.setLayoutManager(new LinearLayoutManager(mActivity, RecyclerView.HORIZONTAL, false));
        AlignmentAdapter alignmentAdapter = new AlignmentAdapter();
        list_alignment.setAdapter(alignmentAdapter);
        alignmentAdapter.setData(alignmentData);
        alignmentAdapter.setListener(this);
    }


    @Override
    public void onAlignmentClick(AlignmentData data) {
        switch (data.name) {
            case "左对齐":
                track.setDirection(SXTextTrack.SXTextDirectionType.Horizontal);
                track.setAlignment(SXTextTrack.SXTextAlignmentType.Left);
                break;
            case "左右居中":
                track.setDirection(SXTextTrack.SXTextDirectionType.Horizontal);
                track.setAlignment(SXTextTrack.SXTextAlignmentType.Center);
                break;
            case "右对齐":
                track.setDirection(SXTextTrack.SXTextDirectionType.Horizontal);
                track.setAlignment(SXTextTrack.SXTextAlignmentType.Right);
                break;
            case "上对齐":
                track.setDirection(SXTextTrack.SXTextDirectionType.Vertical);
                track.setAlignment(SXTextTrack.SXTextAlignmentType.Left);
                break;
            case "上下居中":
                track.setDirection(SXTextTrack.SXTextDirectionType.Vertical);
                track.setAlignment(SXTextTrack.SXTextAlignmentType.Center);
                break;
            case "下对齐":
                track.setDirection(SXTextTrack.SXTextDirectionType.Vertical);
                track.setAlignment(SXTextTrack.SXTextAlignmentType.Right);
                break;
        }
        ActionManager.getInstance().getListener().updateCurrentFrame(() -> ((EditActivity) mActivity).onTextChanged());
    }
}
