package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Matrix;
import android.util.AttributeSet;
import android.util.Log;
import android.widget.FrameLayout;

public class AspectRatioFrameLayout extends FrameLayout {
    private static final String TAG = "AspectRatioFrameLayout";
    private float mAspectRatio;

    float mScale = 1;
    Matrix mMatrix = new Matrix();

    private int mSrcWidth;
    private int mSrcHeight;

    public AspectRatioFrameLayout(Context context) {
        this(context, null);
    }

    public AspectRatioFrameLayout(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public AspectRatioFrameLayout(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
    }

    public void setSize(int width, int height) {
        float ratio = width / (float) height;
        mSrcWidth = width;
        mSrcHeight = height;

        setAspectRatio(ratio);
    }

    public void setAspectRatio(float widthHeightRatio) {
        if (mAspectRatio != widthHeightRatio) {
            mAspectRatio = widthHeightRatio;
            requestLayout();
        }
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
        if (mAspectRatio <= 0) {
            return;
        }

        int width = getMeasuredWidth();
        int height = getMeasuredHeight();
        float viewAspectRatio = (float) width / height;
        float aspectDeformation = mAspectRatio / viewAspectRatio - 1;

        int retWidth = width;
        int retHeight = height;

        mMatrix.reset();
        if (aspectDeformation > 0) {
            retHeight = (int) (width / mAspectRatio);
            mScale = (float) retHeight / mSrcHeight;
            mMatrix.postTranslate(0, (height - retHeight) / 2f);
        } else {
            retWidth = (int) (height * mAspectRatio);
            mScale = (float) retWidth / mSrcWidth;
            mMatrix.postTranslate((width - retWidth) / 2f, 0);
        }
        mMatrix.preScale(mScale, mScale);

        Log.d(TAG, "onMeasure: ");

        super.onMeasure(MeasureSpec.makeMeasureSpec(retWidth, MeasureSpec.EXACTLY),
                MeasureSpec.makeMeasureSpec(retHeight, MeasureSpec.EXACTLY));
    }
}
