package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.text.TextPaint;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.view.MotionEvent;
import android.view.View;

import com.shixing.edit.R;

public class ChangeSpeedView extends View {
    private static final String TAG = "ChangeSpeedView";

    float mSpeeds[] = {0.1f, 1f, 2f, 5f, 10f, 100f};
    String mSpeedText[] = {"0.1x", "1x", "2x", "5x", "10x", "100x"};
    int mSpaces[] = {9, 10, 10, 10, 10};
    int totalSpace = 49;
    int mColor = Color.parseColor("#DDDDDD");
    float mTextSizeSp = 11;
    float mSpeed = 1f;

    Paint mTextPaint;
    private int mHeight;
    private int mWidth;
    private float mStartPadding;
    private int mTextHeight;
    private int mEndPadding;
    private float mTextMarginBottom;
    private float mTextMarginTop;
    private DisplayMetrics mDisplayMetrics;
    private float mMainMarkHeight;
    private float mMarkHeight;
    private float mMarkWidth;
    private float mSpaceWidth;
    private Paint mPaint;
    private float mMainMarkTop;
    private float mTextTop;
    private Rect mBounds;
    private Paint mHandlePaint;
    private float mHandleY;
    private float mHandleRadius;
    private Bitmap mThumb;
    private Bitmap mThumbSelect;
    public ChangeSpeedView(Context context) {
        this(context, null);
    }

    public ChangeSpeedView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public ChangeSpeedView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        mDisplayMetrics = getResources().getDisplayMetrics();

        mPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        mPaint.setColor(mColor);
        mPaint.setStrokeWidth(dp2px(1));
        mPaint.setStrokeCap(Paint.Cap.ROUND);

        mTextPaint = new TextPaint(Paint.ANTI_ALIAS_FLAG);
        float textSize =  dp2px(mTextSizeSp);
        mTextPaint.setTextSize(textSize);
        mTextPaint.setColor(mColor);

        mHandlePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
//        mHandlePaint.setStyle(Paint.Style.STROKE);
        mHandlePaint.setColor(Color.parseColor("#FF2656"));
        mHandlePaint.setStrokeWidth(dp2px(2));

        mBounds = new Rect();
        String text = mSpeedText[0];
        mTextPaint.getTextBounds(text, 0, text.length(), mBounds);
        mStartPadding = mBounds.left + mBounds.width() / 2.f;
        mTextHeight = mBounds.height();

        text = mSpeedText[mSpeedText.length - 1];
        mTextPaint.getTextBounds(text, 0, text.length(), mBounds);
        mEndPadding = mBounds.width() / 2;

        mTextMarginBottom = dp2px(27);
        mTextMarginTop = dp2px(21);

        mMainMarkHeight = dp2px(11);
        mMarkHeight = dp2px(5);
        mMarkWidth = dp2px(1);
        mThumb= BitmapFactory.decodeResource(getResources(), R.drawable.icon_tiaodudian);
        mThumbSelect= BitmapFactory.decodeResource(getResources(), R.drawable.icon_tiaodudian_selected);

    }

    float dp2px(float dpValue) {
        return dpValue * mDisplayMetrics.density;
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
        mWidth = getMeasuredWidth();
        mHeight = getMeasuredHeight();

        mSpaceWidth = (mWidth - mStartPadding - mEndPadding) / (float) totalSpace;
        mMainMarkTop = mHeight - mTextMarginTop - mTextMarginBottom - mTextHeight - mMainMarkHeight;
        mTextTop = mHeight - mTextMarginBottom - mTextHeight;
        mHandleY = mMainMarkTop + mMainMarkHeight / 2;
        mHandleRadius = dp2px(13 / 2f);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        int index = 0;
        for (int i = 0; i < mSpaces.length; i++) {
            float textX = mStartPadding + index * mSpaceWidth;
            String text = mSpeedText[i];
            mTextPaint.getTextBounds(text, 0, text.length(), mBounds);
            canvas.drawText(text, textX - mBounds.width() / 2f, mTextTop, mTextPaint);

            for (int j = 0; j < mSpaces[i]; j++) {
                boolean isMainMark = j == 0;
                float x = mStartPadding + index * mSpaceWidth;
                float y = isMainMark ? mMainMarkTop : mMainMarkTop + (mMainMarkHeight - mMarkHeight) / 2;
                canvas.drawLine(x, y, x, y + (isMainMark ? mMainMarkHeight : mMarkHeight), mPaint);

                index++;
            }
        }

        float x = mStartPadding + index * mSpaceWidth;
        canvas.drawLine(x, mMainMarkTop, x, mMainMarkTop + mMainMarkHeight, mPaint);

        String text = mSpeedText[mSpaces.length];
        mTextPaint.getTextBounds(text, 0, text.length(), mBounds);
        canvas.drawText(text, x - mBounds.width() / 2f, mTextTop, mTextPaint);

        float centerX = speed2x(mSpeed);
        canvas.drawBitmap(mThumb,centerX -mThumb.getWidth()/2, mHandleY-mThumb.getHeight()/2,  mHandlePaint);
//        canvas.drawCircle(centerX + mStartPadding, mHandleY, mHandleRadius, mHandlePaint);
        String speedText = String.format("%.1fx", mSpeed);
        mTextPaint.getTextBounds(speedText, 0, speedText.length(), mBounds);
        canvas.drawText(speedText, centerX - mBounds.width() / 2f + mStartPadding, mHandleY - mMainMarkHeight, mTextPaint);

    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {

        return super.onTouchEvent(event); //TODO
    }

    void setSpeed(float speed) {
        if (speed < 0.1f) speed = 0.1f;
        if (speed > 100f) speed = 100f;
        mSpeed = speed;
        invalidate();
    }

    float speed2x(float speed) {
        if (speed < 0.1f) speed = 0.1f;
        if (speed > 100f) speed = 100f;

        for (int i = mSpaces.length; --i >= 0;) {
            if (speed > mSpeeds[i]) {
                float v = speed - mSpeeds[i];
                float p = v / (mSpeeds[i + 1] - mSpeeds[i]) * mSpaces[i];
                for (int j = 0; j < i; j++) {
                    p += mSpaces[j];
                }

                return p * mSpaceWidth;
            }
        }

        return 0;
    }
}
