package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Point;
import android.graphics.RectF;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

import androidx.annotation.Nullable;

import com.shixing.edit.R;
import com.shixing.edit.multitrack.TrackConfig;
import com.shixing.edit.utils.Logger;
import com.shixing.edit.utils.MathUtil;

public class CropView extends View  implements CustomSeekBar.onAngleChangeListener {


    /**
     * 用于对图片进行移动和缩放变换的矩阵
     */
    private Matrix matrix = new Matrix();

    /**
     * 待展示的Bitmap对象
     */
    private Bitmap sourceBitmap;


    /**
     * ZoomImageView控件的宽度
     */
    private int viewWidth;

    /**
     * ZoomImageView控件的高度
     */
    private int viewHeight;


    /**
     * 记录当前图片的宽度，图片被缩放时，这个值会一起变动
     */
    private float currentBitmapWidth;

    /**
     * 记录当前图片的高度，图片被缩放时，这个值会一起变动
     */
    private float currentBitmapHeight;


    /**
     * 记录图片在矩阵上的横向偏移值
     */
    private float totalTranslateX;

    /**
     * 记录图片在矩阵上的纵向偏移值
     */
    private float totalTranslateY;

    /**
     * 记录图片在矩阵上的总缩放比例
     */
    private float totalRatio;

    private float angle;

    @Override
    public void onAngleChangeStart() {

    }

    @Override
    public void onAngleChangeEnd() {

    }

    @Override
    public void onAngleChange(float angle) {
        this.angle=angle;
        setRatio(ratio);

        invalidate();
    }


    public enum Ratio{
        RATIO_FREE(0),RATIO_916(9f/16),RATIO_34(3f/4),RATIO_11(1f),RATIO_43(4f/3),RATIO_169(16f/9);
        private float ratio;
        private Ratio(float ratio){
            this.ratio=ratio;
        }
        public float getRatio(){
            return this.ratio;
        }
    }
    Paint paint;
    RectF rectF;

    float rectHeight;
    float rectWidth;
    float tmpX;//方框与x边界的差
    float tmpY;
    Ratio ratio=Ratio.RATIO_FREE;
    Point pTopLeft,pTopRight,pBottomLeft,pBottomRight;
    float pxSize;
    float shortLineLength;
    float[] frameLines;
    float[] shortLines;
    Point pBitmapTopLeft,pBitmapTopRight,pBitmapBottomLeft,pBitmapBottomRight;
    public CropView(Context context) {
        this(context, null);
    }

    public CropView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CropView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);

        pTopLeft=new Point();
        pTopRight=new Point();
        pBottomLeft=new Point();
        pBottomRight=new Point();
        pBitmapTopLeft=new Point();
        pBitmapTopRight=new Point();
        pBitmapBottomLeft=new Point();
        pBitmapBottomRight=new Point();

        pxSize= TrackConfig.PX_SIZE;
        shortLineLength=20*pxSize;

        this.matrix=new Matrix();
        this.paint=new Paint();
        this.rectF=new RectF();
    }
    /**
     * 将待展示的图片设置进来。
     *
     * @param bitmap
     *            待展示的Bitmap对象
     */
    public void setImageBitmap(Bitmap bitmap) {
        sourceBitmap = bitmap;
        pBitmapTopRight.x=(int)(sourceBitmap.getWidth()+2*pxSize);
        pBitmapTopRight.y=(int)(0+2*pxSize);
        pBitmapBottomRight.x=(int)(sourceBitmap.getWidth()+2*pxSize);
        pBitmapBottomRight.y=(int)(sourceBitmap.getHeight()+2*pxSize);
        Logger.e("cf --- 0 p1.x="+pBitmapTopRight.x+" p1.y="+pBitmapTopRight.y+" p2.x="+pBitmapBottomRight.x+" p2.y="+pBitmapBottomRight.y);

        setRatio(ratio);
        invalidate();
    }


    public void setRatio(Ratio ratio){
        if (sourceBitmap==null||viewHeight==0){
            Logger.e(" cropview is not init,sourceBitmap is null");
            return;
        }
        matrix.reset();
        this.ratio=ratio;
        pBitmapTopRight.x=(int)(sourceBitmap.getWidth()+2*pxSize);
        pBitmapTopRight.y=(int)(0+2*pxSize);
        pBitmapBottomRight.x=(int)(sourceBitmap.getWidth()+2*pxSize);
        pBitmapBottomRight.y=(int)(sourceBitmap.getHeight()+2*pxSize);

        float bitmapScale=1;
        int bitmapWidth = sourceBitmap.getWidth();
        int bitmapHeight = sourceBitmap.getHeight();
        Logger.e("cf  ratio="+ratio);
        switch (ratio){
            case RATIO_FREE:
                float bitRatio=sourceBitmap.getWidth()/sourceBitmap.getHeight();
                Logger.e("cf  bitRatio="+bitRatio);
                if (sourceBitmap.getWidth()/sourceBitmap.getHeight()<viewWidth*1f/ viewHeight){
                    rectHeight=viewHeight-(int)pxSize*4;
                    rectWidth=viewHeight*bitRatio;
                    bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    Logger.e("cf 2 bitmapScale="+bitmapScale);
                }else {
                    rectWidth=viewWidth-(int)pxSize*4;
                    rectHeight=viewWidth/bitRatio;
                    bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    Logger.e("cf 2 bitmapScale="+bitmapScale);
                }

                break;
            case RATIO_916:

                if (sourceBitmap.getWidth()/sourceBitmap.getHeight()>9f/16f){
                    if (viewWidth/viewHeight>9f/16f){
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*9/16;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }else {
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*16/9;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }

                }else {
                    if (viewWidth/viewHeight>9f/16f){
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*16/9;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }else {
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*9/16;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }

                }
                break;
            case RATIO_34:
                if (sourceBitmap.getWidth()/sourceBitmap.getHeight()<3f/4f){
                    if (viewWidth/viewHeight>3f/4f){
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*4/3;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }else {
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*3/4;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }
                }else {
                    if (viewWidth/viewHeight>3f/4f){
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*3/4;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }else {
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*4/3;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }
                }
                break;
            case RATIO_11:
                if (sourceBitmap.getWidth()/sourceBitmap.getHeight()>1f/1f){
                    if (viewWidth/viewHeight>1f/1f){
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*1/1;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }else {
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*1/1;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }

                }else {
                    if (viewWidth/viewHeight>1f/1f){
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*1/1;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }else {
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*1/1;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }
                }
                break;
            case RATIO_43:
                if (sourceBitmap.getWidth()/sourceBitmap.getHeight()>4f/3f){
                    if (viewWidth/viewHeight>4f/3f){
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*3/4;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }else {
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*4/3;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }
                }else {
                    if (viewWidth/viewHeight>4f/3f){
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*3/4;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }else {
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*4/3;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }

                }
                break;
            case RATIO_169:
                if (sourceBitmap.getWidth()/sourceBitmap.getHeight()>16f/9f){
                    if (viewWidth/viewHeight>16f/9f){
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*9/16;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }else {
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*16/9;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }

                }else {
                    if (viewWidth/viewHeight>16f/9f){
                        rectHeight=viewHeight-(int)pxSize*4;
                        rectWidth=rectHeight*16/9;
                        bitmapScale=rectHeight*1f/sourceBitmap.getHeight();
                    }else {
                        rectWidth=viewWidth-(int)pxSize*4;
                        rectHeight=rectWidth*9/16;
                        bitmapScale=rectWidth*1f/sourceBitmap.getWidth();
                    }

                }
                break;

        }

        totalRatio=bitmapScale;
        totalTranslateY = (viewHeight - (bitmapHeight * bitmapScale)) / 2f;
        totalTranslateX = (viewWidth - (bitmapWidth * bitmapScale)) / 2f;

        matrix.postScale(totalRatio, totalRatio);
        matrix.postTranslate(totalTranslateX,totalTranslateY);
        matrix.postRotate(angle,viewWidth/2,viewHeight/2);
        Logger.e("cf --- 1 p1.x="+pBitmapTopRight.x+" p1.y="+pBitmapTopRight.y+" p2.x="+pBitmapBottomRight.x+" p2.y="+pBitmapBottomRight.y+" totalRatio="+totalRatio);
        float[] points1=new float[]{pBitmapTopRight.x,pBitmapTopRight.y,pBitmapBottomRight.x,pBitmapBottomRight.y};
        float[] points=new float[4];
        matrix.mapPoints(points,points1);

        pBitmapTopRight.x=(int)points[0];
        pBitmapTopRight.y=(int)points[1];
        pBitmapBottomRight.x=(int)points[2];
        pBitmapBottomRight.y=(int)points[3];
        Logger.e("cf --- 2 p1.x="+pBitmapTopRight.x+" p1.y="+pBitmapTopRight.y+" p2.x="+pBitmapBottomRight.x+" p2.y="+pBitmapBottomRight.y);


        currentBitmapWidth = bitmapWidth * totalRatio;
        currentBitmapHeight = bitmapHeight * totalRatio;
        tmpX=(viewWidth-rectWidth)/2;
        tmpY=(viewHeight-rectHeight)/2;
        pTopLeft.x=(int)(tmpX);
        pTopLeft.y=(int)(tmpY+pxSize*2);
        pTopRight.x=(int)(viewWidth-tmpX);
        pTopRight.y=(int)(tmpY+pxSize*2);
        pBottomRight.x=(int)(viewWidth-tmpX);
        pBottomRight.y=(int)(viewHeight-tmpY-pxSize*2);
        pBottomLeft.x=(int)(tmpX);
        pBottomLeft.y=(int)(viewHeight-tmpY-pxSize*2);
        Point pCenter=new Point();
        pCenter.x=getWidth()/2;
        pCenter.y=getHeight()/2;
        getChangeRadius(pBitmapTopRight,pBitmapBottomRight,pCenter,pBottomRight);

        if (angle<0){
            matrix.postScale(sy,sy,getWidth()/2,getHeight()/2);
        }else {
            matrix.postScale(sx,sx,getWidth()/2,getHeight()/2);
        }


        frameLines=new float[]{pTopLeft.x,pTopLeft.y,pTopRight.x,pTopRight.y,
                pTopLeft.x,pTopLeft.y+rectHeight/3,pTopRight.x,pTopRight.y+rectHeight/3,
                pTopLeft.x,pTopLeft.y+rectHeight*2/3,pTopRight.x,pTopRight.y+rectHeight*2/3,
                pBottomLeft.x,pBottomLeft.y,pBottomRight.x,pBottomRight.y,
                pBottomLeft.x,pBottomLeft.y,pTopLeft.x,pTopLeft.y,
                pBottomLeft.x+rectWidth/3,pBottomLeft.y,pTopLeft.x+rectWidth/3,pTopLeft.y,
                pBottomLeft.x+rectWidth*2/3,pBottomLeft.y,pTopLeft.x+rectWidth*2/3,pTopLeft.y,
                pBottomRight.x,pBottomLeft.y,pTopRight.x,pTopLeft.y,
        };//画网格边框
        shortLines=new float[]{
                pTopLeft.x-2*pxSize,pTopLeft.y,pTopLeft.x+shortLineLength,pTopLeft.y,
                pTopLeft.x+rectWidth/2-shortLineLength,pTopLeft.y,pTopLeft.x+rectWidth/2+shortLineLength,pTopLeft.y,
                pTopLeft.x+rectWidth-shortLineLength,pTopLeft.y,pTopLeft.x+rectWidth+2*pxSize,pTopLeft.y,

                pTopLeft.x-2*pxSize,pBottomLeft.y,pTopLeft.x+shortLineLength,pBottomLeft.y,
                pTopLeft.x+rectWidth/2-shortLineLength,pBottomLeft.y,pTopLeft.x+rectWidth/2+shortLineLength,pBottomLeft.y,
                pTopLeft.x+rectWidth-shortLineLength,pBottomLeft.y,pTopLeft.x+rectWidth+2*pxSize,pBottomLeft.y,

                pTopLeft.x,pTopLeft.y-2*pxSize,pTopLeft.x,pTopLeft.y+shortLineLength,
                pTopLeft.x,pTopLeft.y+rectHeight/2-shortLineLength,pTopLeft.x,pTopLeft.y+rectHeight/2+shortLineLength,
                pBottomLeft.x,pBottomLeft.y-shortLineLength,pBottomLeft.x,pBottomLeft.y+2*pxSize,

                pTopRight.x,pTopLeft.y-2*pxSize,pTopRight.x,pTopLeft.y+shortLineLength,
                pTopRight.x,pTopLeft.y+rectHeight/2-shortLineLength,pTopRight.x,pTopLeft.y+rectHeight/2+shortLineLength,
                pBottomRight.x,pBottomRight.y-shortLineLength,pBottomRight.x,pBottomRight.y+2*pxSize,
        };

        invalidate();
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        if (changed) {
            // 分别获取到ZoomImageView的宽度和高度
            viewWidth = getWidth();
            viewHeight = getHeight();
            setRatio(ratio);
        }
    }


    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        if (sourceBitmap!=null){


            canvas.drawBitmap(sourceBitmap,matrix,null);
            Paint p1=new Paint();
            p1.setColor(getResources().getColor(R.color.white));
            p1.setStrokeWidth(pxSize);
            canvas.drawLines(frameLines,p1);
            p1.setStrokeWidth(5*pxSize);
            canvas.drawLines(shortLines,p1);
            }

            Path framePath=new Path();
            framePath.moveTo(pTopLeft.x,pTopLeft.y);
            framePath.lineTo(pTopRight.x,pTopRight.y);
            framePath.lineTo(pBottomRight.x,pBottomRight.y);
            framePath.lineTo(pBottomLeft.x,pBottomLeft.y);
            framePath.close();
            invalidate();

        }

        private void getChangeRadius(Point pBitmapTopRight,Point pBitmapBottomRight,Point pCircleCenter,Point pBottomRight){
        if ((pBitmapTopRight.x-pBitmapBottomRight.x)==0){
            Logger.e("cf crosspoint 垂直状态");
            return;
        }

            float k=(pCircleCenter.y-pBitmapBottomRight.y)*1f/(pCircleCenter.x-pBitmapBottomRight.x);
            Logger.e("cf crosspoint  (pCircleCenter.y-pBitmapBottomRight.y)="+ (pCircleCenter.y-pBitmapBottomRight.y)+" (pCircleCenter.x-pBitmapBottomRight.x)="+(pCircleCenter.x-pBitmapBottomRight.x));
            Point othPoint=new Point();
            othPoint.x=pBottomRight.x;
            othPoint.y=(int)((pBottomRight.x-pCircleCenter.x)*1f/k+pCircleCenter.y);
            sx=(pBottomRight.x-pCircleCenter.x)*1f/(pBitmapBottomRight.x-pCircleCenter.x);
            sy=(othPoint.y-pCircleCenter.y)*1f/(pBitmapBottomRight.y-pCircleCenter.y);

            Logger.e("cf crosspoint  othPoint.x="+ othPoint.x+" othPoint.y="+othPoint.y+" k="+k+" sx="+sx+" sy="+sy);


        }
    float sx=1,sy=1;
    public Matrix getMatrix(){
        return matrix;
    }
    public int getCropWidth(){
        return (int)currentBitmapWidth;
    }
    public int getCropHeight(){
        return (int)currentBitmapHeight;
    }
}
