package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.DashPathEffect;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Rect;
import android.graphics.RectF;
import android.text.Layout;
import android.text.StaticLayout;
import android.text.TextPaint;
import android.util.AttributeSet;
import android.util.Log;
import android.util.TypedValue;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewConfiguration;

import com.shixing.edit.R;

import java.util.ArrayList;
import java.util.List;

public class CurveSpeedView extends View {
    private static final String TAG = "CurveSpeedView";

    public static final int RADIUS_DP = 10;
    private static final int POINT_SIZE = 5;
    private ArrayList<ControlPoint> points = new ArrayList<>();
    private float radius;
    private Paint rectPaint;
    private RectF rect = new RectF();
    private Paint dashPaint;
    private StaticLayout staticLayout;
    private StaticLayout staticLayout1;
    private StaticLayout staticLayout2;
    private TextPaint textPaint;
    private Rect bounds;
    private Paint whitePaint;
    private Paint blackPaint;
    private Path curve;
    private Paint curvePaint;
    private ViewConfiguration viewConfiguration;
    private Paint cursorPaint;
    private double totalDuration = 10;
    private double currentDuration = 0;

    public CurveSpeedView(Context context) {
        this(context, null);
    }

    public CurveSpeedView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CurveSpeedView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        setLayerType(LAYER_TYPE_SOFTWARE, null);
        radius = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, RADIUS_DP, getResources().getDisplayMetrics());
        rectPaint = new Paint();
        rectPaint.setStyle(Paint.Style.STROKE);
        rectPaint.setStrokeWidth(TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 1, getResources().getDisplayMetrics()));
        rectPaint.setColor(getResources().getColor(R.color.transparent_15p_white));

        dashPaint = new Paint();
        dashPaint.setColor(getResources().getColor(R.color.transparent_15p_white));
        dashPaint.setStyle(Paint.Style.STROKE);
        dashPaint.setStrokeWidth(TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 1, getResources().getDisplayMetrics()));
        dashPaint.setPathEffect(new DashPathEffect(new float[]{15, 5}, 0));

        whitePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        whitePaint.setColor(Color.WHITE);

        blackPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        blackPaint.setColor(Color.BLACK);

        bounds = new Rect();
        textPaint = new TextPaint();
        textPaint.setColor(getResources().getColor(R.color.transparent_15p_white));
        textPaint.setTextSize(TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 10, getResources().getDisplayMetrics()));
        textPaint.getTextBounds("10x", 0, 3, bounds);
        staticLayout = new StaticLayout("10x", textPaint, 100, Layout.Alignment.ALIGN_NORMAL, 1.0f, 0.0f, true);
        staticLayout1 = new StaticLayout("1x", textPaint, 100, Layout.Alignment.ALIGN_NORMAL, 1.0f, 0.0f, true);
        staticLayout2 = new StaticLayout("0.1x", textPaint, 100, Layout.Alignment.ALIGN_NORMAL, 1.0f, 0.0f, true);

        initPoints(whitePaint, blackPaint);

        curve = new Path();
        curvePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        curvePaint.setStyle(Paint.Style.STROKE);
        curvePaint.setColor(Color.YELLOW);
        curvePaint.setStrokeWidth(TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 1.5f, getResources().getDisplayMetrics()));

        cursorPaint = new Paint();
        cursorPaint.setStrokeWidth(TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 2f, getResources().getDisplayMetrics()));
        cursorPaint.setColor(Color.WHITE);

        viewConfiguration = ViewConfiguration.get(getContext());
    }

    private void initPoints(Paint whitePaint, Paint blackPaint) {
        for (int i = 0; i < POINT_SIZE; i++) {
            ControlPoint controlPoint = new ControlPoint();
            controlPoint.whitePaint = whitePaint;
            controlPoint.blackPaint = blackPaint;
            controlPoint.radius = radius;
            controlPoint.strokeWidth = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 2, getResources().getDisplayMetrics());

            points.add(controlPoint);
        }
    }

    private void initPointsPosition() {
        for (int i = 0; i < POINT_SIZE; i++) {
            points.get(i).setPosition(rect.left + rect.width() / (POINT_SIZE - 1) * i, rect.top + rect.height() / 2);
        }
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        int width = getWidth();
        int height = getHeight();

        rect.set(radius, radius, width - radius, height - radius);
        initPointsPosition();
    }

    @Override
    protected void onDraw(Canvas canvas) {
        canvas.drawRect(rect, rectPaint);
        canvas.drawLine(rect.left, rect.top + rect.height() / 4, rect.right, rect.top + rect.height() / 4, dashPaint);
        canvas.drawLine(rect.left + bounds.width(), rect.top + rect.height() / 2, rect.right, rect.top + rect.height() / 2, rectPaint);
        canvas.drawLine(rect.left, rect.top + rect.height() / 4 * 3, rect.right, rect.top + rect.height() / 4 * 3, dashPaint);

        drawText(canvas);

        curve.reset();
        curve.moveTo(points.get(0).x, points.get(0).y);
        ControlPoint last = points.get(0);
        for (int i = 1; i < points.size(); i++) {
            ControlPoint point = points.get(i);
            curve.cubicTo(last.x + (point.x - last.x) / 2, last.y, point.x - (point.x - last.x) / 2, point.y, point.x, point.y);
            last = point;
        }

        canvas.drawPath(curve, curvePaint);

        for (ControlPoint point : points) {
            point.draw(canvas);
        }

        float cursorX = (float)(currentDuration / totalDuration) * rect.width() + rect.left;
        canvas.drawLine(cursorX, rect.top, cursorX, rect.bottom, cursorPaint);
    }

    float lastX;
    float lastY;
//    float downX;
//    float downY;
    ControlPoint activePoint;
    boolean seeking = false;

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                for (ControlPoint point : points) {
                    if (point.touchPoint(event.getX(), event.getY())) {
                        point.active = true;
                        lastX = event.getX();
                        lastY = event.getY();
                        activePoint = point;
                        invalidate();
                    } else {
                        point.active = false;
                    }
                }
                if (rect.contains(event.getX(), event.getY())) {
                    currentDuration = (event.getX() - rect.left) / rect.width() * totalDuration;
                    seeking = true;
                    invalidate();
                }
                break;
            case MotionEvent.ACTION_MOVE:
                if (activePoint != null) {
                    float deltaX = event.getX() - lastX;
                    float deltaY = event.getY() - lastY;
//                    if (deltaX > viewConfiguration.getScaledTouchSlop() || deltaY > viewConfiguration.getScaledTouchSlop()) {
//                    }

                    activePoint.y += deltaY;
                    activePoint.y = clamp(activePoint.y, rect.top, rect.bottom);

                    int index = points.indexOf(activePoint);
                    if (index != 0 && index != points.size() - 1) {
                        activePoint.x += deltaX;
                        activePoint.x = clamp(activePoint.x, points.get(index - 1).x + radius, points.get(index + 1).x - radius);
                    }

                    currentDuration = (activePoint.x - rect.left) / rect.width() * totalDuration;
                    invalidate();
                } else if (seeking) {
                    currentDuration = (clamp(event.getX(), rect.left, rect.right) - rect.left) / rect.width() * totalDuration;
                    invalidate();
                }
                lastX = event.getX();
                lastY = event.getY();
                break;
            case MotionEvent.ACTION_UP:
                seeking = false;
                if (activePoint != null) {
                    activePoint = null;
                    if (actionListener != null) {
                        actionListener.onPathChanged(points);
                    }
                }
                break;
            default:
                break;
        }
        return true;
    }

    float clamp(float v, float min, float max) {
        if (v < min)
            return min;
        if (v > max)
            return max;
        return v;
    }

    private void drawText(Canvas canvas) {
        String s = "10x";
        textPaint.getTextBounds(s, 0, s.length(), bounds);

        canvas.save();
        canvas.translate(rect.left + 10, rect.top);
        staticLayout.draw(canvas);
        canvas.restore();

        canvas.save();
        canvas.translate(rect.left + 10, rect.top + rect.height() / 2 + bounds.top);
        staticLayout1.draw(canvas);
        canvas.restore();

        canvas.save();
        canvas.translate(rect.left + 10, rect.bottom + bounds.top * 2);
        staticLayout2.draw(canvas);
        canvas.restore();
    }

    public void reset() {
        points.clear();
        initPoints(whitePaint, blackPaint);
        initPointsPosition();
        invalidate();

        if (actionListener != null) {
            actionListener.onPathChanged(points);
        }
    }

    public RectF getFrame() {
        return rect;
    }

    private ActionListener actionListener;

    public void setActionListener(ActionListener actionListener) {
        this.actionListener = actionListener;
    }

    public interface ActionListener {
        void onPathChanged(List<ControlPoint> points);

        void onSeek(double time);

        void onSelect(boolean select, boolean cenBeRemove);
    }

    public static class ControlPoint {
        boolean active = false;
        float x;
        float y;
        float radius;
        float strokeWidth;
        Paint whitePaint;
        Paint blackPaint;

        void draw(Canvas canvas) {
            canvas.drawCircle(x, y, radius, whitePaint);
            if (!active) {
                canvas.drawCircle(x, y, radius - strokeWidth, blackPaint);
            }
        }

        void setPosition(float x, float y) {
            this.x = x;
            this.y = y;
        }

        boolean touchPoint(float touchX, float touchY) {
            return Math.abs(touchX - x) < radius + 15 && Math.abs(touchY - y) < radius + 15;
        }

        public float getX() {
            return x;
        }

        public float getY() {
            return y;
        }
    }
}
