package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

import androidx.annotation.Nullable;

import com.shixing.edit.R;
import com.shixing.edit.multitrack.TrackConfig;
import com.shixing.edit.utils.Logger;
import com.shixing.edit.utils.ScreenUtil;

public class CustomSeekBar extends View {

    Bitmap cursorBitmap;
    float pxSize;
    float sx,sy;
    float cursorX;
    float margin=30;
    double textHalfWidth;
    Paint pText;
    public CustomSeekBar(Context context) {
        this(context, null);
    }

    public CustomSeekBar(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CustomSeekBar(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        cursorBitmap= BitmapFactory.decodeResource(getContext().getResources(), R.drawable.icon_caijian);
        pxSize= TrackConfig.PX_SIZE;
        pText=new Paint();
        pText.setColor(getResources().getColor(R.color.text_selected_red));
        pText.setTextSize(15*pxSize);


    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        sx=(getWidth()-margin*2*pxSize)/cursorBitmap.getWidth();
        sy=(13*pxSize)/cursorBitmap.getHeight();
        cursorX=getWidth()/2;
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        switch (event.getAction()){
            case MotionEvent.ACTION_DOWN:
                if (Math.abs(cursorX-event.getX())<10*pxSize){
                    if (event.getX()<=margin*pxSize){
                        cursorX=margin*pxSize;
                    }else if (event.getX()>=getWidth()-margin*pxSize){
                        cursorX=getWidth()-margin*pxSize;
                    }else {
                        cursorX=event.getX();
                    }
                    if (listener!=null){
                        listener.onAngleChangeStart();
                    }
                    invalidate();
                    return true;
                }
                break;
            case MotionEvent.ACTION_MOVE:
                if (event.getX()<=margin*pxSize){
                    cursorX=margin*pxSize;
                }else if (event.getX()>=getWidth()-margin*pxSize){
                    cursorX=getWidth()-margin*pxSize;
                }else {
                    cursorX=event.getX();
                }
                if (listener!=null){
                    listener.onAngleChange(getAngleText());
                }
                invalidate();
                return true;
            case MotionEvent.ACTION_UP:
                if (event.getX()<=margin*pxSize){
                    cursorX=margin*pxSize;
                }else if (event.getX()>=getWidth()-margin*pxSize){
                    cursorX=getWidth()-margin*pxSize;
                }else {
                    cursorX=event.getX();
                }
                if (listener!=null){
                    listener.onAngleChangeEnd();
                }
                invalidate();

                return true;
        }

        return super.onTouchEvent(event);
    }


    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        if (cursorBitmap!=null&&sx!=0){
            Matrix matrix=new Matrix();
            matrix.postScale(sx,sy);
            matrix.postTranslate(margin*pxSize,getHeight()-cursorBitmap.getHeight()*sy-5*pxSize);
            canvas.drawBitmap(cursorBitmap,matrix,null);
            Paint paint=new Paint();
            paint.setAntiAlias(true);//抗锯齿
            paint.setStrokeWidth(pxSize);
            paint.setColor(getResources().getColor(R.color.text_selected_red));
            canvas.drawLine(cursorX,getHeight()-6*pxSize,cursorX,getHeight()-26*pxSize,paint);
            float textHalfWidth=pText.measureText((int)getAngleText()+"");
            canvas.drawText((int)getAngleText()+"",cursorX-(int)textHalfWidth/2,getHeight()-32*pxSize,pText);
            Paint p2=new Paint();
            p2.setStyle(Paint.Style.STROKE);
            p2.setAntiAlias(true);//抗锯齿
            p2.setColor(getResources().getColor(R.color.text_selected_red));
            canvas.drawCircle(cursorX+(int)textHalfWidth/2+5*pxSize,getHeight()-42*pxSize,1*pxSize,p2);

        }


    }

    private float getAngleText(){
        float n=((cursorX-getWidth()/2)*90/(getWidth()-margin*pxSize*2));
        return n;
    }
    public onAngleChangeListener listener;
    public interface onAngleChangeListener{
        public void onAngleChangeStart();
        public void onAngleChangeEnd();
        public void onAngleChange(float angle);
    }
    public void setAngleChangeListener(onAngleChangeListener listener){
        this.listener=listener;
    }
}
