package com.shixing.edit.widget;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.RectF;
import android.util.Log;

import com.shixing.edit.manager.ActionManager;
import com.shixing.sxedit.SXShape;
import com.shixing.sxedit.internal.Track;

public class MaskView {
    Track mTrack;
    Paint mPaint;
    Matrix mMatrix = new Matrix();
    Matrix mInverse = new Matrix();
    Matrix mTransform = new Matrix();
    Matrix mDrawMatrix = new Matrix();
    RectF mFrame = new RectF();
    RectF mNewFrame = new RectF();

    public MaskView() {
        mPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        mPaint.setColor(Color.YELLOW);
        mPaint.setStyle(Paint.Style.STROKE);
        mPaint.setStrokeWidth(5);
    }

    void setTrack(Track track) {
        mTrack = track;
        if (mTrack == null) return;

        float[] transform = mTrack.transform();
        mMatrix.setValues(transform);
        mMatrix.invert(mInverse);
        int[] trackSize = mTrack.getTrackSize();
        mDrawMatrix.set(mMatrix);
        mDrawMatrix.preTranslate(-trackSize[0] / 2f, -trackSize[1] / 2f);
    }

    void draw(Canvas canvas) {
        if (mTrack == null) return;


        SXShape shape = mTrack.getMaskShape();
        if (shape == null) {
            return;
        }

        Path path = shape.getPath();

        path.transform(mDrawMatrix);
        canvas.drawPath(path, mPaint);
    }

    public void translate(float x, float y) {
        SXShape shape = mTrack.getMaskShape();
        if (shape == null) {
            return;
        }

        ActionManager.getInstance().getListener().getEditManager().sendActionToRenderThread(new Runnable() {
            @Override
            public void run() {
                SXShape shape = mTrack.getMaskShape();
                mTransform.setTranslate(x, y);
                mTransform.preConcat(mMatrix);
                mTransform.postConcat(mInverse);
                shape.transform(mTransform);

                mTrack.updateShape();
            }
        });

//        mTransform.setTranslate(x, y);
//        mTransform.preConcat(mMatrix);
//        mTransform.postConcat(mInverse);
//        shape.transform(mTransform);
//
//        float[] value = {x, y};
//        mTransform.mapPoints(value);
//        shape.translate(value[0], value[1]);

//        shape.translate(x, y);
//        mTrack.updateShape();
    }

    public void scale(float scaleX, float scaleY) {
        SXShape shape = mTrack.getMaskShape();
        if (shape == null) {
            return;
        }

        ActionManager.getInstance().getListener().getEditManager().sendActionToRenderThread(new Runnable() {
            @Override
            public void run() {
                SXShape shape = mTrack.getMaskShape();
                shape.getBbox(mFrame);
                shape.scale(scaleX, scaleY);
                shape.getBbox(mNewFrame);
                float midX = (mFrame.left + mFrame.right) / 2;
                float midY = (mFrame.top + mFrame.bottom) / 2;
                float newMidX = (mNewFrame.left + mNewFrame.right) / 2;
                float newMidY = (mNewFrame.top + mNewFrame.bottom) / 2;
                shape.translate(midX - newMidX, midY - newMidY);

                mTrack.updateShape();
            }
        });


    }

    public void rotate(float degree) {
        SXShape shape = mTrack.getMaskShape();
        if (shape == null) {
            return;
        }

        ActionManager.getInstance().getListener().getEditManager().sendActionToRenderThread(new Runnable() {
            @Override
            public void run() {
                SXShape shape = mTrack.getMaskShape();
                shape.getBbox(mFrame);
                shape.rotate(degree);
                shape.getBbox(mNewFrame);
                float midX = (mFrame.left + mFrame.right) / 2;
                float midY = (mFrame.top + mFrame.bottom) / 2;
                float newMidX = (mNewFrame.left + mNewFrame.right) / 2;
                float newMidY = (mNewFrame.top + mNewFrame.bottom) / 2;
                shape.translate(midX - newMidX, midY - newMidY);

                mTrack.updateShape();
            }
        });

    }
}
