package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.BitmapShader;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.RectF;
import android.graphics.Shader;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.MotionEvent;
import android.view.ViewConfiguration;

import androidx.annotation.NonNull;

import com.shixing.edit.R;
import com.shixing.edit.data.EditData;
import com.shixing.edit.data.TrackBean;
import com.shixing.edit.data.TrackSourceObject;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.multitrack.TrackConfig;
import com.shixing.edit.multitrack.TrackUtil;
import com.shixing.edit.utils.ScreenUtil;
import com.shixing.edit.utils.Utils;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;
import com.shixing.sxedit.track.SXTrack;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

public class PipTrackInstance implements TrackGroupView.TrackView {

    private List<TrackBean> trackBeanList;
    private float pxSize ;
    private float mDefaultTextSize;
    public int mMaxDurationWidth;//最大长度
    public double mSelfMaxDuration;//自身最大时长
    Bitmap mDefaultBitmap, mLeftSeekBitmap, mRightSeekBitmap;
    private int mTrackHeight;
    public boolean isSelectMoving;//是否选中并拖动了
    public float eventX, eventY;//纠正后的点击位置
    private float clickX,clickY;
    private TrackBean mSelectedTrack = null;//当前选中的track
    private int mSelectedTrackIndex = 0;//被选中的位置
    private Paint pText=null;
    HashMap<String,Double> mDurationMap =new HashMap<>();
    Context context;
    private TrackGroupView parentView;
    Paint mPaint = new Paint();

    private static final int EVENT_LONG_TOUCH = 20;
    private Handler mHandler;
    private TrackBean trackPressed;
    private Paint mTranslucentPaint;
    private boolean mIsMovingTrack;
    private float lastX;

    public PipTrackInstance(TrackGroupView view){
        init(view);
    }

    @Override
    public void init(TrackGroupView view) {
        parentView=view;
        context=view.getContext();

        pxSize = (float) ScreenUtil.INSTANCE.dp2px(1f);
        mDefaultTextSize = (float) ScreenUtil.INSTANCE.dp2px(13f);

        mTrackHeight = TrackConfig.TRACK_HEIGHT;
        trackBeanList = new ArrayList<>();
        Bitmap b1=BitmapFactory.decodeResource(context.getResources(), R.drawable.icon_default_pic);
        float  sc = mTrackHeight * 1f / b1.getHeight();
        mDefaultBitmap = Utils.scaleBitmap(b1,sc);
        Bitmap b2=BitmapFactory.decodeResource(context.getResources(), R.drawable.icon_tiaojiekuang_zuo);
        float  sc2 = mTrackHeight * 1f / b2.getHeight();
        mLeftSeekBitmap = Utils.scaleBitmap(b2,sc2);
        Bitmap b3=BitmapFactory.decodeResource(context.getResources(), R.drawable.icon_tiaojiekuang_you);
        float  sc3 = mTrackHeight * 1f / b3.getHeight();
        mRightSeekBitmap = Utils.scaleBitmap(b3,sc3);

        pText=new Paint();
        pText.setColor(context.getResources().getColor(R.color.text_white));
        pText.setTextSize(mDefaultTextSize);

        mTranslucentPaint = new Paint();
        mTranslucentPaint.setAlpha(120);
        mHandler = new Handler(new Handler.Callback() {
            @Override
            public boolean handleMessage(@NonNull Message msg) {
                switch (msg.what) {
                    case EVENT_LONG_TOUCH:
                        Log.d("TAG", "handleMessage: mIsMovingTrack");
                        TrackActionManager.getInstance().onMovingDown();
                        mIsMovingTrack = true;
                        trackPressed.movingStartX = trackPressed.mStartX;
                        trackPressed.movingEndX = trackPressed.mEndX;
                        trackPressed.durationWidth = TrackUtil.getInstance().getDurationWidth(trackPressed.duration);
                        lastX = clickX;
                        if (mSelectedTrack != null) {
                            TrackActionManager.getInstance().onTrackUnSelect(mSelectedTrack.track);
                            mSelectedTrack = null;
                        }
                        parentView.setCanVerticalScroll(false);
                        parentView.invalidate();
                        break;
                }
                return false;
            }
        });
    }

    @Override
    public void onDraw(Canvas canvas) {
        if (canvas != null && this.mMaxDurationWidth > 0) {
            for (int i = 0; i < trackBeanList.size(); i++) {
                TrackBean trackBean = trackBeanList.get(i);
                BitmapShader bitmapShader = new BitmapShader(trackBean.shaderBitmap, Shader.TileMode.REPEAT, Shader.TileMode.REPEAT);
                Matrix matrix=new Matrix();
                int xtmp;
                RectF rectF;
                Paint paint;
                if (mIsMovingTrack && trackBean.id.equals(trackPressed.id)) {
                    xtmp = -TrackUtil.getInstance().getDurationWidth(trackBean.mStartFrameIndex)+trackBean.movingStartX;
                    rectF = new RectF(trackBean.movingStartX, trackBean.mStartY+pxSize * 2, trackBean.movingEndX, trackBean.mStartY+mTrackHeight - pxSize * 2);
                    paint = mTranslucentPaint;
                } else {
                    xtmp = -TrackUtil.getInstance().getDurationWidth(trackBean.mStartFrameIndex)+trackBean.mStartX;
                    rectF = new RectF(trackBean.mStartX, trackBean.mStartY+pxSize * 2, trackBean.mEndX, trackBean.mStartY+mTrackHeight - pxSize * 2);
                    paint = mPaint;
                }
                matrix.postTranslate(xtmp,0);
                bitmapShader.setLocalMatrix(matrix);
                paint.setShader(bitmapShader);
                canvas.drawRect(rectF, paint);
            }
            if (mSelectedTrack != null) {
                Paint p2 = new Paint();
                p2.setColor(context.getResources().getColor(R.color.bg_track_blake));
                RectF rectF;
                RectF rectF_top;
                RectF rectF_bottom;

                Matrix matrix = new Matrix();
                Matrix matrix_end = new Matrix();
                switch (mSelectedTrack.touchPoint) {
                    case 0:
                        matrix.postTranslate(mSelectedTrack.mStartX - mLeftSeekBitmap.getWidth(), mSelectedTrack.mStartY);
                        canvas.drawBitmap(mLeftSeekBitmap, matrix, p2);//画滑块
                        rectF_top = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY, mSelectedTrack.mEndX, mSelectedTrack.mStartY+ pxSize * 2);
                        rectF_bottom = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY+ mTrackHeight - pxSize * 2, mSelectedTrack.mEndX, mSelectedTrack.mStartY+ mTrackHeight);
                        p2.setColor(context.getResources().getColor(R.color.white));
                        canvas.drawRect(rectF_top, p2);
                        canvas.drawRect(rectF_bottom, p2);
                        matrix_end.postTranslate(mSelectedTrack.mEndX, mSelectedTrack.mStartY);
                        canvas.drawBitmap(mRightSeekBitmap, matrix_end, p2);
                        break;
                    case 1://按住左边滑动头
                        rectF = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY, eventX, mSelectedTrack.mStartY+ mTrackHeight);
                        canvas.drawRect(rectF, p2);//画滑动的黑色背景
                        matrix.postTranslate(eventX - mLeftSeekBitmap.getWidth() , mSelectedTrack.mStartY);
                        canvas.drawBitmap(mLeftSeekBitmap, matrix, p2);//画滑块
                        rectF_top = new RectF(eventX, mSelectedTrack.mStartY, mSelectedTrack.mEndX, mSelectedTrack.mStartY+ pxSize * 2);
                        rectF_bottom = new RectF(eventX, mSelectedTrack.mStartY+ mTrackHeight - pxSize * 2, mSelectedTrack.mEndX, mSelectedTrack.mStartY+ mTrackHeight);
                        p2.setColor(context.getResources().getColor(R.color.white));
                        canvas.drawRect(rectF_top, p2);
                        canvas.drawRect(rectF_bottom, p2);
                        matrix_end.postTranslate(mSelectedTrack.mEndX, mSelectedTrack.mStartY);
                        canvas.drawBitmap(mRightSeekBitmap, matrix_end, p2);
                        break;
                    case 2://右边
                        rectF = new RectF(eventX, mSelectedTrack.mStartY, mSelectedTrack.mEndX , mSelectedTrack.mStartY+ mTrackHeight);
                        canvas.drawRect(rectF, p2);//画滑动的黑色背景
                        matrix.postTranslate(mSelectedTrack.mStartX - mLeftSeekBitmap.getWidth() , mSelectedTrack.mStartY);
                        canvas.drawBitmap(mLeftSeekBitmap, matrix, p2);//画滑块
                        rectF_top = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY, eventX, mSelectedTrack.mStartY+ pxSize * 2);
                        rectF_bottom = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY+ mTrackHeight - pxSize * 2, eventX, mSelectedTrack.mStartY+ mTrackHeight);
                        p2.setColor(context.getResources().getColor(R.color.white));
                        canvas.drawRect(rectF_top, p2);
                        canvas.drawRect(rectF_bottom, p2);
                        matrix_end.postTranslate(eventX, mSelectedTrack.mStartY);
                        canvas.drawBitmap(mRightSeekBitmap, matrix_end, p2);
                        break;
                }

            }

        }
    }


    @Override
    public boolean onTouchEvent(MotionEvent event) {
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                clickX = event.getX();
                clickY=event.getY();
                if (mSelectedTrack != null) {
                    if (mSelectedTrack.mLeftRect.contains(event.getX(), event.getY())) {
                        mSelectedTrack.difX = mSelectedTrack.mLeftRect.right - event.getX();
                        TrackActionManager.getInstance().onMovingDown();
                        parentView.setCanVerticalScroll(false);
                        eventX = event.getX() + mSelectedTrack.difX;
                        eventY = event.getY();
                        mSelectedTrack.touchPoint = 1;
                        isSelectMoving = true;
                        return true;
                    } else if (mSelectedTrack.mRightRect.contains(event.getX(), event.getY())) {
                        mSelectedTrack.difX = mSelectedTrack.mRightRect.left - event.getX();
                        TrackActionManager.getInstance().onMovingDown();
                        parentView.setCanVerticalScroll(false);
                        eventX = event.getX() + mSelectedTrack.difX;
                        eventY = event.getY();
                        mSelectedTrack.touchPoint = 2;
                        isSelectMoving = true;
                        return true;
                    } else {
                        mSelectedTrack.touchPoint = 0;
                    }
                }
                trackPressed = getTrackAtPoint(clickX, clickY);
                if (trackPressed != null) {
                    Log.d("TAG", "onTouchEvent: trackPressed");
                    mHandler.sendEmptyMessageDelayed(EVENT_LONG_TOUCH, ViewConfiguration.getLongPressTimeout());
                }
                
                break;
            case MotionEvent.ACTION_MOVE:
                double dtmp = 0;
                if (isSelectMoving) {
                    if (mSelectedTrack.touchPoint == 1) {
                        dtmp = TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX - (event.getX() + mSelectedTrack.difX));
                    }
                    else if (mSelectedTrack.touchPoint == 2) {
                        dtmp = TrackUtil.getInstance().getDurationByLength((event.getX() + mSelectedTrack.difX) - mSelectedTrack.mStartX);
                    }
                    if (canBeMove(mSelectedTrackIndex, (int)(event.getX() + mSelectedTrack.difX), mSelectedTrack.touchPoint,dtmp)){
                        eventX = event.getX() + mSelectedTrack.difX;
                        eventY = event.getY();
                        if (mSelectedTrack.touchPoint == 1) {
                            trackBeanList.get(mSelectedTrackIndex).duration = TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX - eventX);
                        } else if (mSelectedTrack.touchPoint == 2) {
                            trackBeanList.get(mSelectedTrackIndex).duration = TrackUtil.getInstance().getDurationByLength(eventX - mSelectedTrack.mStartX);
                            updateSelfMaxDuration(mSelectedTrack.id,TrackUtil.getInstance().getDurationByLength(eventX-TrackConfig.SCREEN_WIDTH_HALF));
                        }
                    }
                    parentView.invalidate();
                } else {
                    if (!mIsMovingTrack) {
                        if (Math.abs(clickX - event.getX()) > 20 || Math.abs(clickY - event.getY()) > 20) {
                            mHandler.removeMessages(EVENT_LONG_TOUCH);
                            return true;
                        }
                    } else {
                        int delta = (int) (event.getX() - lastX);
                        lastX = event.getX();
                        trackPressed.movingStartX = Math.max(trackPressed.movingStartX + delta, TrackConfig.SCREEN_WIDTH_HALF);
                        trackPressed.movingEndX = trackPressed.movingStartX + trackPressed.durationWidth;
                        parentView.invalidate();
                    }
                }
                break;
            case MotionEvent.ACTION_UP:
                mHandler.removeMessages(EVENT_LONG_TOUCH);
                TrackActionManager.getInstance().onMovingUp();
                parentView.setCanVerticalScroll(true);
                double durationTmp = 0;
                if (isSelectMoving) {
                    int touchPoint=mSelectedTrack.touchPoint;
                    switch (mSelectedTrack.touchPoint) {
                        case 1:
                            durationTmp = TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX - eventX);
                            trackBeanList.get(mSelectedTrackIndex).duration = durationTmp;
                            trackBeanList.get(mSelectedTrackIndex).mStartFrameIndex = trackBeanList.get(mSelectedTrackIndex).mEndFrameIndex - durationTmp;
                            trackBeanList.get(mSelectedTrackIndex).mStartX=(int)eventX;
                            trackBeanList.get(mSelectedTrackIndex).mLeftRect=new RectF();
                            trackBeanList.get(mSelectedTrackIndex).mLeftRect = new RectF(trackBeanList.get(mSelectedTrackIndex).mStartX - mLeftSeekBitmap.getWidth() , trackBeanList.get(mSelectedTrackIndex).mStartY, trackBeanList.get(mSelectedTrackIndex).mStartX, trackBeanList.get(mSelectedTrackIndex).mStartY+ mTrackHeight);
                            trackBeanList.get(mSelectedTrackIndex).mRightRect = new RectF(trackBeanList.get(mSelectedTrackIndex).mEndX, trackBeanList.get(mSelectedTrackIndex).mStartY, trackBeanList.get(mSelectedTrackIndex).mEndX + mLeftSeekBitmap.getWidth() , trackBeanList.get(mSelectedTrackIndex).mStartY+ mTrackHeight);
                            break;
                        case 2:
                            durationTmp = TrackUtil.getInstance().getDurationByLength(eventX - mSelectedTrack.mStartX);
                            trackBeanList.get(mSelectedTrackIndex).duration = durationTmp;
                            trackBeanList.get(mSelectedTrackIndex).mEndFrameIndex = trackBeanList.get(mSelectedTrackIndex).mStartFrameIndex + durationTmp;
                            trackBeanList.get(mSelectedTrackIndex).mEndX=(int)eventX;
                            trackBeanList.get(mSelectedTrackIndex).mLeftRect = new RectF(trackBeanList.get(mSelectedTrackIndex).mStartX - mLeftSeekBitmap.getWidth() , trackBeanList.get(mSelectedTrackIndex).mStartY, trackBeanList.get(mSelectedTrackIndex).mStartX, trackBeanList.get(mSelectedTrackIndex).mStartY+ mTrackHeight);
                            trackBeanList.get(mSelectedTrackIndex).mRightRect = new RectF(trackBeanList.get(mSelectedTrackIndex).mEndX, trackBeanList.get(mSelectedTrackIndex).mStartY, trackBeanList.get(mSelectedTrackIndex).mEndX + mLeftSeekBitmap.getWidth() ,trackBeanList.get(mSelectedTrackIndex).mStartY+ mTrackHeight);
                            break;
                    }
                    mSelectedTrack = trackBeanList.get(mSelectedTrackIndex);
                    TrackActionManager.getInstance().cutTrackOrEffect(mSelectedTrack,touchPoint);
                    updateSelfMaxDuration(mSelectedTrack.id,TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX-TrackConfig.SCREEN_WIDTH_HALF),true);
                } else if (mIsMovingTrack) {
                    mIsMovingTrack = false;
                    double newInPoint = TrackUtil.getInstance().getDurationByLength(trackPressed.movingStartX - TrackConfig.SCREEN_WIDTH_HALF);
                    boolean success = ActionManager.getInstance().getListener().setTrackOffset(trackPressed.track, newInPoint);
                    if (success) {
//                        if (trackPressed.movingEndX + TrackConfig.SCREEN_WIDTH_HALF > parentView.getWidth()) {
                            double duration = ActionManager.getInstance().getListener().getEditManager().editDuration();
                            TrackActionManager.getInstance().updateMaxDuration(duration,true);
//                        }
                        trackPressed.mStartX = trackPressed.movingStartX;
                        trackPressed.mEndX = trackPressed.movingEndX;
                        trackPressed.mLeftRect.set(trackPressed.mStartX - mLeftSeekBitmap.getWidth(), trackPressed.mStartY, trackPressed.mStartX, trackPressed.mStartY + mTrackHeight);
                        trackPressed.mRightRect.set(trackPressed.mEndX, trackPressed.mStartY, trackPressed.mEndX + mLeftSeekBitmap.getWidth(), trackPressed.mStartY + mTrackHeight);
                        ActionManager.getInstance().getListener().updateCurrentFrame();
                        parentView.requestLayout();
                    }
                    parentView.invalidate();
                    return true;
                }
                isSelectMoving = false;
                trackPressed = null;
                break;
            case MotionEvent.ACTION_CANCEL:
                mHandler.removeMessages(EVENT_LONG_TOUCH);
                break;
        }

        return parentView.getOnTouchEvent(event);
    }

    private TrackBean getTrackAtPoint(float x, float y) {
        for (int i = 0; i < trackBeanList.size(); i++) {
            TrackBean v = trackBeanList.get(i);
            if (v.mStartX < clickX && v.mEndX > clickX
                    &&v.mStartY<clickY&&v.mStartY+mTrackHeight>clickY) {
                return v;
            }
        }
        return null;
    }


    private void updateSelfMaxDuration(String from,double duration){
        updateSelfMaxDuration(from,duration,false);
    }
    private void updateSelfMaxDuration(String from,double duration,boolean isActionUp){
        if (duration < 0L){
            return;
        }
        //计算textTrackView中的最大时长
        mDurationMap.put(from,duration);
        Collection<Double> c = mDurationMap.values();
        mSelfMaxDuration= Collections.max(c);
        int w=TrackUtil.getInstance().getDurationWidth(mSelfMaxDuration);
        if (mMaxDurationWidth<w){
            mMaxDurationWidth=w;
        }
        TrackUtil.getInstance().updateMaxTrackDuration(3,mSelfMaxDuration,isActionUp);
        parentView.requestLayout();
        parentView.invalidate();
    }

    @Override
    public void addTrackBeanSource(TrackSourceObject groups) {
        trackBeanList.clear();
        List<EditData.PIPTrackGroup> pipGroups = groups.pipTrackGroups;
        for (int i = 0; i < pipGroups.size(); i++) {
            EditData.PIPTrackGroup group = pipGroups.get(i);
            for (MediaTrack track : group.tracks) {
                addMediaTracks(track, i);
            }
        }
    }

    @Override
    public void onProgressChanged(double time) {

    }

    @Override
    public boolean canBeAddEffect(Effect effect) {
        return false;
    }


    private HashMap<String,Bitmap> shaderBitmaps=new HashMap<>();
    public void addMediaTracks(MediaTrack track, int level) {
        TrackBean trackBean = new TrackBean();
        trackBean.id = track.getTrackId();
        trackBean.srcPath = track.getFilePath();
        if (track.resourceDuration() <= 0){
            trackBean.mediaType=TrackBean.typePic;
            trackBean.origDuration =TrackConfig.PIC_TRACK_DURATION_DEFAULT;
            Bitmap bitmap = BitmapFactory.decodeFile(trackBean.srcPath);
            if (bitmap == null) bitmap = mDefaultBitmap;
            float scale = (float) mTrackHeight / bitmap.getHeight();
            trackBean.shaderBitmap = Utils.scaleBitmap(bitmap, scale);
        }else {
            trackBean.mediaType=TrackBean.typeVideo;
            trackBean.mSpeed= ActionManager.getInstance().getListener().getSpeed(track);
            trackBean.origDuration=track.resourceDuration()/trackBean.mSpeed;
            if (!shaderBitmaps.containsKey(track.getTrackId())){
                double unit=TrackUtil.getInstance().getDurationByLength(mTrackHeight);
                double duration=TrackUtil.getInstance().getLocalVideoDuration(trackBean.srcPath)/(1000*unit);//单位高度多少时长
                int l=(int)duration+1;
                Bitmap[] bitmaps=new Bitmap[l];
                for (int i = 0; i < bitmaps.length; i++) {
                    bitmaps[i]= mDefaultBitmap;
                }
                TrackUtil.getInstance().getVideoBitmap(trackBean.id,trackBean.srcPath, mTrackHeight,l,unit, new TrackUtil.OnFrameLoadListener() {
                    @Override
                    public void onFrameLoadFinish(String trackId,int index,Bitmap bitmap) {
                        Bitmap tmp=null;
                        if (trackBean!=null){
                            bitmaps[index]=bitmap;
                            tmp=TrackUtil.getInstance().add2Bitmap(bitmaps, mTrackHeight, mTrackHeight);
                            shaderBitmaps.put(trackId,tmp);
                        }
                        if (trackBean!=null&&tmp!=null){
                            trackBean.shaderBitmap= tmp;
                        }
                        parentView.post(() -> parentView.invalidate());
                    }
                });
            }else {
                trackBean.shaderBitmap =shaderBitmaps.get(track.getTrackId());
            }

            if (trackBean.shaderBitmap==null){
                trackBean.shaderBitmap=mDefaultBitmap;
            }
        }
        trackBean.duration = track.duration();
        trackBean.mLevel = level;


        trackBean.track = track;

        trackBean.durationWidth = TrackUtil.getInstance().getDurationWidth(track.duration());
        trackBean.mStartX = TrackConfig.SCREEN_WIDTH_HALF+TrackUtil.getInstance().getDurationWidth(track.inPoint());
//        trackBean.mStartX = TrackConfig.SCREEN_WIDTH_HALF+TrackUtil.getInstance().getMainTrackScrollX();//临时测试使用这个方法
        trackBean.mStartFrameIndex=track.inPoint() - track.startTime();
        trackBean.touchPoint = 0;
        trackBean.mEndX = trackBean.mStartX + trackBean.durationWidth;
        trackBean.mEndFrameIndex=trackBean.mStartFrameIndex+trackBean.duration;
        trackBean.mStartY=TrackConfig.TRACK_HEIGHT*trackBean.mLevel;
        trackBean.mLeftRect = new RectF(trackBean.mStartX - mLeftSeekBitmap.getWidth() , trackBean.mLevel*mTrackHeight, trackBean.mStartX, trackBean.mLevel*mTrackHeight+mTrackHeight);
        trackBean.mRightRect = new RectF(trackBean.mEndX, trackBean.mLevel*mTrackHeight, trackBean.mEndX + mLeftSeekBitmap.getWidth() , trackBean.mLevel*mTrackHeight+mTrackHeight);
        trackBeanList.add(trackBean);
        if (mSelectedTrack!=null&&mSelectedTrack.id.equals(trackBean.id)){
            mSelectedTrack=trackBean;
        }
        updateSelfMaxDuration(trackBean.id,TrackUtil.getInstance().getDurationByLength(trackBean.mEndX-TrackConfig.SCREEN_WIDTH_HALF));
    }


    @Override
    public void updateMaxDuration(double duration,boolean isActionUp) {
        if (mSelfMaxDuration!=duration&& duration > 0L&&isActionUp){
            mMaxDurationWidth = TrackUtil.getInstance().getDurationWidth(duration);
            parentView.requestLayout();
            parentView.invalidate();
        }
    }

    @Override
    public void updateScaleSize(float scale) {

    }

    @Override
    public boolean isClicked() {
        boolean isSelected = false;
        for (int i = 0; i < trackBeanList.size(); i++) {
            TrackBean v = trackBeanList.get(i);
            if (v.mStartX < clickX && v.mEndX > clickX
                    &&v.mStartY<clickY&&v.mStartY+mTrackHeight>clickY) {
                isSelected = true;
                boolean isSWitch=mSelectedTrack!=null;
                if (mSelectedTrack==null||!mSelectedTrack.id.equals(v.id)){
                    mSelectedTrack = v;
                    mSelectedTrackIndex = i;
                    mSelectedTrack.isSelected = true;
                    trackBeanList.get(i).isSelected = true;
                    TrackActionManager.getInstance().onTrackSelect( mSelectedTrack.track,isSWitch);
                }
                break;
            }
        }
        parentView.invalidate();
        return isSelected;
    }


    @Override
    public void cancelSelectTrackByClick() {
        if (mSelectedTrack!=null){
            TrackActionManager.getInstance().onTrackUnSelect(mSelectedTrack.track);
        }
        cancelSelectTrack();
    }

    @Override
    public void cancelSelectEffectByClick() {

    }


    private void cancelSelectTrack(){
        mSelectedTrack = null;
        parentView.invalidate();
    }


    @Override
    public void updateTrack(Track track) {

    }

    @Override
    public void updateMainTrack(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffect(Effect effect) {

    }

    @Override
    public void updateTrackDuration(Track track) {

    }

    @Override
    public void updateMainTrackDuration(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffectDuration(Effect effect) {

    }

    @Override
    public void removeTrack(Track track) {


    }

    @Override
    public void removeMainTrack(MediaTrack mediaTrack) {

    }


    @Override
    public void removeEffect(Effect effect) {

    }

    @Override
    public void addTrack(Track track) {
//        trackBeanList.clear();
//        List<EditData.PIPTrackGroup> pipGroups = groups.pipTrackGroups;
//        for (int i = 0; i < pipGroups.size(); i++) {
//            EditData.PIPTrackGroup group = pipGroups.get(i);
//            for (MediaTrack track : group.tracks) {
//                addMediaTracks(track, i);
//            }
//        }
    }

    @Override
    public void addMainTrack(List<MediaTrack> mainTracks) {

    }

    @Override
    public void addEffect(Effect effect) {

    }

    @Override
    public void addTrackGroup(List<TrackGroup> trackGroups, int trackType) {
        if (trackType== SXTrack.SX_TRACK_TYPE_MEDIA) {
            trackBeanList.clear();
            mDurationMap.clear();
            mSelfMaxDuration = mMaxDurationWidth = 0;
            int trackCount = 0;
            if (trackGroups != null && trackGroups.size() > 0) {
                for (int i = 0; i < trackGroups.size(); i++) {
                    TrackGroup group = trackGroups.get(i);
                    for (Track track : group.tracks()) {
                        addMediaTracks((MediaTrack) track, i);
                        trackCount++;
                    }
                }
            }
            if (trackCount == 0) {
                TrackUtil.getInstance().updateMaxTrackDuration(3,mSelfMaxDuration,true);
                parentView.requestLayout();
                parentView.invalidate();
            }
        }
    }

    @Override
    public void addEffectList(List<Effect> effectList, int trackType) {

    }

    @Override
    public int getMaxDurationWidth() {
        return mMaxDurationWidth;
    }


    private boolean canBeMove(int vtSelectedIndex,int eventX,int touchPoint, double dtmp){
        if (isOutSize(touchPoint, mSelectedTrack,eventX)){
            return false;
        }
        if (mSelectedTrack.mediaType==TrackBean.typePic) {
            if (touchPoint == 1) {
                if (vtSelectedIndex == 0
                        || trackBeanList.get(vtSelectedIndex - 1).mLevel < trackBeanList.get(vtSelectedIndex).mLevel) {
                    return eventX > TrackConfig.SCREEN_WIDTH_HALF;//每一层靠近零轴的最左边不能拖动了
                } else {
                    return eventX > trackBeanList.get(vtSelectedIndex - 1).mEndX;//同一层，往左拖动不能超过左边view
                }
            } else if (touchPoint == 2) {
                if (vtSelectedIndex + 1 == trackBeanList.size()
                        || (trackBeanList.get(vtSelectedIndex).mLevel < trackBeanList.get(vtSelectedIndex + 1).mLevel)) {//最后一个或者每行最后一个
                    return true;
                } else {
                    return eventX < trackBeanList.get(vtSelectedIndex + 1).mStartX;//同一层，不能超过后面view
                }
            }
        }else {
            if (touchPoint == 1) {
                return mSelectedTrack.mEndFrameIndex - dtmp >= 0;
            } else if (touchPoint == 2) {
                return mSelectedTrack.mStartFrameIndex + dtmp <= mSelectedTrack.origDuration;
            }

        }
        return false;
    }
    //拖动缩小过程中，不能将时长完全为0
    private boolean isOutSize(int touchPoint,TrackBean trackBean,int eventX){
        if (touchPoint==1){
            return eventX>=trackBean.mEndX;
        }else if (touchPoint==2){
            return eventX<=trackBean.mStartX;
        }
        return false;
    }


    @Override
    public void onMovingUp() {

    }

    @Override
    public void onMovingDown() {

    }

    @Override
    public void onTouchScroll() {

    }

    @Override
    public void onTrackSelect(Track track,boolean isSwitch) {
        int i=0;
        for (TrackBean trackBean:trackBeanList){
            if (track.getTrackId().equals(trackBean.id)){
                    mSelectedTrack = trackBean;
                    mSelectedTrackIndex = i;
                    mSelectedTrack.isSelected = true;
                    trackBeanList.get(i).isSelected = true;
                    parentView.requestLayout();
                    parentView.invalidate();
                    break;
            }
            i++;
        }
    }

    @Override
    public void onEffectSelect(Effect effect,boolean isSwitch) {

    }

    @Override
    public void onTrackUnSelect(Track track) {
        for (TrackBean trackBean:trackBeanList){
            if (track.getTrackId().equals(trackBean.id)){
                cancelSelectTrack();
            }
        }
    }

    @Override
    public void onEffectUnSelect(Effect effect) {

    }

    @Override
    public void onClickTransition(Track track) {

    }

    @Override
    public void hideDetailFragment() {

    }

    @Override
    public int getHeight() {
        int ret = 0;
        if (trackBeanList.size() > 0) {
            TrackBean trackBean = trackBeanList.get(trackBeanList.size() - 1);
            ret = trackBean.mStartY + mTrackHeight;
        }
        return ret;
    }

    @Override
    public boolean canVerticalScroll() {
        return mSelectedTrack == null;
    }
}
