package com.shixing.edit.widget;

import android.content.res.Resources;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.RectF;
import android.graphics.drawable.Drawable;

import com.shixing.edit.R;
import com.shixing.sxedit.internal.TextTrack;
import com.shixing.sxedit.internal.Track;

public class TextBoxView extends TrackFrame {
    private static final String TAG = "TextBoxView";

    private int width;
    private int height;
    private Drawable iconEdit;
    private Drawable iconCopy;
    private Drawable iconDelete;
    private Drawable iconRotate;
    private RectF frame = new RectF();
    private Matrix matrix = new Matrix();
    private Matrix iconMatrix = new Matrix();
    private float[] points = new float[8];
    private float[] mappedPoints = new float[8];

    private static final int INDEX_DELETE = 0;
    private static final int INDEX_EDIT = 1;
    private static final int INDEX_ROTATE = 2;
    private static final int INDEX_COPY = 3;

    private boolean editing = false;

    public void initIcon(Resources resources) {
        iconCopy = resources.getDrawable(R.drawable.icon_bianjifuzhi);
        iconDelete = resources.getDrawable(R.drawable.icon_bianjishanchu);
        iconEdit = resources.getDrawable(R.drawable.icon_text_edit);
        iconRotate = resources.getDrawable(R.drawable.icon_bianjixuanzhuan1);

        setIconBounds(iconDelete);
        setIconBounds(iconEdit);
        setIconBounds(iconRotate);
        setIconBounds(iconCopy);
    }

    public void setTrack(Track track) {
        this.track = track;
        super.setTrack(track);
        if (track != null) {
            updateTrackSize();
        }
    }

    public void updateTrackSize() {
        int[] trackSize = track.getTrackSize();
        width = trackSize[0];
        height = trackSize[1];
        if (trackSize[0] == 0 || trackSize[1] == 0) {
            return;
        }
        super.setTrack(track);

        frame.set(-width / 2f, -height / 2f, width / 2f, height / 2f);

        points[0] = frame.left;
        points[1] = frame.top;
        points[2] = frame.right;
        points[3] = frame.top;
        points[4] = frame.right;
        points[5] = frame.bottom;
        points[6] = frame.left;
        points[7] = frame.bottom;
    }

    private void setIconBounds(Drawable icon) {
        icon.setBounds(-icon.getIntrinsicWidth() / 2, -icon.getIntrinsicHeight() / 2,
                icon.getIntrinsicWidth() / 2, icon.getIntrinsicHeight() / 2);
    }

    public void draw(Canvas canvas, Paint framePaint) {
        if (track == null) return;
        if (width == 0 || height == 0) {
            updateTrackSize();
            if (width == 0 || height == 0) {
                return;
            }
        }

        if (!inTrackTimeRange())
            return;

        super.drawFrame(canvas, framePaint);

        float[] transform = track.transform();
        matrix.setValues(transform);

        matrix.mapPoints(mappedPoints, points);

//        canvas.drawLine(mappedPoints[0], mappedPoints[1], mappedPoints[2], mappedPoints[3], framePaint);
//        canvas.drawLine(mappedPoints[2], mappedPoints[3], mappedPoints[4], mappedPoints[5], framePaint);
//        canvas.drawLine(mappedPoints[4], mappedPoints[5], mappedPoints[6], mappedPoints[7], framePaint);
//        canvas.drawLine(mappedPoints[6], mappedPoints[7], mappedPoints[0], mappedPoints[1], framePaint);

        float rotation = track.rotation();
        iconMatrix.reset();
        iconMatrix.setRotate(rotation);

        drawIcon(canvas, INDEX_COPY, iconCopy);
        drawIcon(canvas, INDEX_DELETE, iconDelete);
        drawIcon(canvas, INDEX_EDIT, iconEdit);
        drawIcon(canvas, INDEX_ROTATE, iconRotate);
    }

    private void drawIcon(Canvas canvas, int index, Drawable icon) {
        if (editing && (index == INDEX_EDIT || index == INDEX_COPY)) {
            return;
        }

        iconMatrix.setTranslate(mappedPoints[index * 2], mappedPoints[index * 2 + 1]);
        canvas.save();
        canvas.concat(iconMatrix);
        icon.draw(canvas);
        canvas.restore();
    }

    private float[] mappedMidPoint = {0, 0};
    private float[] downPoint = {0, 0};
    private float downRotation = 0;
    private float[] downScale = {1, 1};

    public boolean isHandle(float x, float y) {
        boolean b = pointInIcon(x, y, 2);
        if (b) {
            downPoint[0] = x;
            downPoint[1] = y;

            float[] midPoint = {frame.centerX(), frame.centerY()};
            matrix.mapPoints(mappedMidPoint, midPoint);

            downRotation = track.rotation();
            downScale = track.scale();
        }
        return b;
    }

    private boolean pointInIcon(float x, float y, int index) {
        return Math.abs(mappedPoints[2 * index] - x) < 40 && Math.abs(mappedPoints[2 * index + 1] - y) < 40;
    }

    public void handleMove(float x, float y) {
        float scale = calculateScale(x, y);
        float rotation = calculateRotation(x, y);

        track.setScale(downScale[0] * scale, downScale[1] * scale);
        track.setRotation(downRotation + rotation);
    }

    private float calculateScale(float x, float y) {
        float deltaX = x - mappedMidPoint[0];
        float deltaY = y - mappedMidPoint[1];
        double distance = Math.sqrt(deltaX * deltaX + deltaY * deltaY);

        deltaX = downPoint[0] - mappedMidPoint[0];
        deltaY = downPoint[1] - mappedMidPoint[1];
        double old = Math.sqrt(deltaX * deltaX + deltaY * deltaY);

        return (float) (distance / old);
    }

    private float calculateRotation(float x, float y) {
        float dx = x - mappedMidPoint[0];
        float dy = y - mappedMidPoint[1];
        float degree = (float) Math.toDegrees(Math.atan2(dy, dx));

        dx = downPoint[0] - mappedMidPoint[0];
        dy = downPoint[1] - mappedMidPoint[1];
        float old = (float) Math.toDegrees(Math.atan2(dy, dx));

        return degree - old;
    }

    public boolean onSingleTap(float x, float y) {
        if (actionListener == null) {
            return false;
        }
        if (pointInIcon(x, y, INDEX_ROTATE)) {
            return true;
        } else if (!editing && pointInIcon(x, y, INDEX_EDIT)) {
            actionListener.onClickEdit(track);
            return true;
        } else if (pointInIcon(x, y, INDEX_DELETE)) {
            actionListener.onClickDelete(track);
            return true;
        } else if (!editing && pointInIcon(x, y, INDEX_COPY)) {
            actionListener.onClickCopy(track);
            return true;
        }
        return false;
    }

    private ActionListener actionListener;

    public void setActionListener(ActionListener actionListener) {
        this.actionListener = actionListener;
    }

    public void setEditing(boolean editing) {
        this.editing = editing;
    }

    public interface ActionListener {
        void onClickEdit(Track track);

        void onClickDelete(Track track);

        void onClickCopy(Track track);
    }
}
