package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.RectF;
import android.text.TextUtils;
import android.view.MotionEvent;

import com.shixing.edit.R;
import com.shixing.edit.data.EditData;
import com.shixing.edit.data.TrackBean;
import com.shixing.edit.data.TrackSourceObject;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.multitrack.TrackConfig;
import com.shixing.edit.multitrack.TrackUtil;
import com.shixing.edit.multitrack.TrackViewListener;
import com.shixing.edit.utils.ScreenUtil;
import com.shixing.edit.utils.Utils;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.TextTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;
import com.shixing.sxedit.track.SXTrack;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

public class TextOrStickerTrackInstance implements TrackGroupView.TrackView {

    private List<TrackBean> trackBeanList;
    private float pxSize ;
    private float mDefaultTextSize;
    public int mMaxDurationWidth;//最大长度
    public double mSelfMaxDuration;//自身最大时长
    Bitmap mDefaultBitmap, mLeftSeekBitmap, mRightSeekBitmap;
    private int trackHeight;
    public boolean isSelectMoving;//是否选中并拖动了
    public float eventX, eventY;//纠正后的点击位置
    private float clickX,clickY;
    private TrackBean mSelectedTrack = null;//当前选中的track
    private int mSelectedTrackIndex = 0;//被选中的位置
    private Paint pText=null;
    HashMap<String,Double> mDurationMap =new HashMap<>();
    Context context;
    private TrackGroupView parentView;
    public TextOrStickerTrackInstance(TrackGroupView view){
        init(view);
    }

    @Override
    public void init(TrackGroupView view) {
        parentView=view;
        context=view.getContext();

        pxSize = (float) ScreenUtil.INSTANCE.dp2px(1f);
        mDefaultTextSize = (float) ScreenUtil.INSTANCE.dp2px(13f);

        trackHeight = TrackConfig.TRACK_HEIGHT;
        trackBeanList = new ArrayList<>();
        Bitmap b1=BitmapFactory.decodeResource(context.getResources(), R.drawable.icon_default_pic);
        float  sc = trackHeight * 1f / b1.getHeight();
        mDefaultBitmap = Utils.scaleBitmap(b1,sc);
        Bitmap b2=BitmapFactory.decodeResource(context.getResources(), R.drawable.icon_tiaojiekuang_zuo);
        float  sc2 = trackHeight * 1f / b2.getHeight();
        mLeftSeekBitmap = Utils.scaleBitmap(b2,sc2);
        Bitmap b3=BitmapFactory.decodeResource(context.getResources(), R.drawable.icon_tiaojiekuang_you);
        float  sc3 = trackHeight * 1f / b3.getHeight();
        mRightSeekBitmap = Utils.scaleBitmap(b3,sc3);

        pText=new Paint();
        pText.setColor(context.getResources().getColor(R.color.text_white));
        pText.setTextSize(mDefaultTextSize);

    }

    @Override
    public void onDraw(Canvas canvas) {
        if (canvas != null && this.mMaxDurationWidth > 0) {
            Paint paint = new Paint();
            paint.setColor(context.getResources().getColor(R.color.bg_item_track_yellow));

            for (int i = 0; i < trackBeanList.size(); i++) {
                TrackBean trackBean = trackBeanList.get(i);
                RectF rectF = new RectF(trackBean.mStartX, trackBean.mStartY+ pxSize * 2, trackBean.mEndX, trackBean.mStartY+trackHeight - pxSize * 2);
                canvas.drawRoundRect(rectF,2* pxSize,2* pxSize,paint);
                canvas.save();
                canvas.clipRect(rectF);
                canvas.drawText(trackBean.text,trackBean.mStartX+8* pxSize,trackBean.mStartY+12* pxSize + mDefaultTextSize /2,pText);
                canvas.restore();
            }
            if (mSelectedTrack != null) {
                Paint p2 = new Paint();
                p2.setColor(context.getResources().getColor(R.color.bg_track_blake));
                RectF rectF;
                RectF rectF_top;
                RectF rectF_bottom;

                Matrix matrix = new Matrix();
                Matrix matrix_end = new Matrix();
                switch (mSelectedTrack.touchPoint) {
                    case 0:
                        matrix.postTranslate(mSelectedTrack.mStartX - mLeftSeekBitmap.getWidth(), mSelectedTrack.mStartY);
                        canvas.drawBitmap(mLeftSeekBitmap, matrix, p2);//画滑块
                        rectF_top = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY, mSelectedTrack.mEndX, mSelectedTrack.mStartY+ pxSize * 2);
                        rectF_bottom = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY+trackHeight - pxSize * 2, mSelectedTrack.mEndX, mSelectedTrack.mStartY+trackHeight);
                        p2.setColor(context.getResources().getColor(R.color.white));
                        canvas.drawRect(rectF_top, p2);
                        canvas.drawRect(rectF_bottom, p2);
                        matrix_end.postTranslate(mSelectedTrack.mEndX, mSelectedTrack.mStartY);
                        canvas.drawBitmap(mRightSeekBitmap, matrix_end, p2);
                        break;
                    case 1://按住左边滑动头
                        rectF = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY, eventX, mSelectedTrack.mStartY+trackHeight);
                        canvas.drawRect(rectF, p2);//画滑动的黑色背景
                        matrix.postTranslate(eventX - mLeftSeekBitmap.getWidth() , mSelectedTrack.mStartY);
                        canvas.drawBitmap(mLeftSeekBitmap, matrix, p2);//画滑块
                        rectF_top = new RectF(eventX, mSelectedTrack.mStartY, mSelectedTrack.mEndX, mSelectedTrack.mStartY+ pxSize * 2);
                        rectF_bottom = new RectF(eventX, mSelectedTrack.mStartY+trackHeight - pxSize * 2, mSelectedTrack.mEndX, mSelectedTrack.mStartY+trackHeight);
                        p2.setColor(context.getResources().getColor(R.color.white));
                        canvas.drawRect(rectF_top, p2);
                        canvas.drawRect(rectF_bottom, p2);
                        matrix_end.postTranslate(mSelectedTrack.mEndX, mSelectedTrack.mStartY);
                        canvas.drawBitmap(mRightSeekBitmap, matrix_end, p2);
                        break;
                    case 2://右边
                        rectF = new RectF(eventX, mSelectedTrack.mStartY, mSelectedTrack.mEndX , mSelectedTrack.mStartY+ trackHeight);
                        canvas.drawRect(rectF, p2);//画滑动的黑色背景
                        matrix.postTranslate(mSelectedTrack.mStartX - mLeftSeekBitmap.getWidth() , mSelectedTrack.mStartY);
                        canvas.drawBitmap(mLeftSeekBitmap, matrix, p2);//画滑块
                        rectF_top = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY, eventX, mSelectedTrack.mStartY+ pxSize * 2);
                        rectF_bottom = new RectF(mSelectedTrack.mStartX, mSelectedTrack.mStartY+trackHeight - pxSize * 2, eventX, mSelectedTrack.mStartY+trackHeight);
                        p2.setColor(context.getResources().getColor(R.color.white));
                        canvas.drawRect(rectF_top, p2);
                        canvas.drawRect(rectF_bottom, p2);
                        matrix_end.postTranslate(eventX, mSelectedTrack.mStartY);
                        canvas.drawBitmap(mRightSeekBitmap, matrix_end, p2);
                        break;
                }

            }

        }
    }


    @Override
    public boolean onTouchEvent(MotionEvent event) {
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                clickX = event.getX();
                clickY=event.getY();
                if (mSelectedTrack != null) {
                    if (mSelectedTrack.mLeftRect.contains(event.getX(), event.getY())) {
                        mSelectedTrack.difX = mSelectedTrack.mLeftRect.right - event.getX();
                        TrackActionManager.getInstance().onMovingDown();
                        parentView.setCanVerticalScroll(false);
                        eventX = event.getX() + mSelectedTrack.difX;
                        eventY = event.getY();
                        mSelectedTrack.touchPoint = 1;
                        isSelectMoving = true;
                        return true;
                    } else if (mSelectedTrack.mRightRect.contains(event.getX(), event.getY())) {
                        mSelectedTrack.difX = mSelectedTrack.mRightRect.left - event.getX();
                        TrackActionManager.getInstance().onMovingDown();
                        parentView.setCanVerticalScroll(false);
                        eventX = event.getX() + mSelectedTrack.difX;
                        eventY = event.getY();
                        mSelectedTrack.touchPoint = 2;
                        isSelectMoving = true;
                        return true;
                    } else {
                        mSelectedTrack.touchPoint = 0;
                    }
                }
                break;
            case MotionEvent.ACTION_MOVE:
                if (isSelectMoving) {
                    if (canBeMove(mSelectedTrackIndex, (int)(event.getX() + mSelectedTrack.difX), mSelectedTrack.touchPoint)){
                        eventX = event.getX() + mSelectedTrack.difX;
                        eventY = event.getY();
                        if (mSelectedTrack.touchPoint == 1) {
                            trackBeanList.get(mSelectedTrackIndex).duration = TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX - eventX);
                        } else if (mSelectedTrack.touchPoint == 2) {
                            trackBeanList.get(mSelectedTrackIndex).duration = TrackUtil.getInstance().getDurationByLength(eventX - mSelectedTrack.mStartX);
                            updateSelfMaxDuration(mSelectedTrack.id,TrackUtil.getInstance().getDurationByLength(eventX-TrackConfig.SCREEN_WIDTH_HALF));
                        }
                    }
                    parentView.invalidate();
                }
                break;
            case MotionEvent.ACTION_UP:
                TrackActionManager.getInstance().onMovingUp();
                double durationTmp = 0;
                if (isSelectMoving) {
                    int touchPoint=mSelectedTrack.touchPoint;
                    switch (mSelectedTrack.touchPoint) {
                        case 1:
                            durationTmp = TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX - eventX);
                            trackBeanList.get(mSelectedTrackIndex).duration = durationTmp;
                            trackBeanList.get(mSelectedTrackIndex).mStartFrameIndex = trackBeanList.get(mSelectedTrackIndex).mEndFrameIndex - durationTmp;
                            trackBeanList.get(mSelectedTrackIndex).mStartX=(int)eventX;
                            trackBeanList.get(mSelectedTrackIndex).mLeftRect=new RectF();
                            trackBeanList.get(mSelectedTrackIndex).mLeftRect = new RectF(trackBeanList.get(mSelectedTrackIndex).mStartX - mLeftSeekBitmap.getWidth() , trackBeanList.get(mSelectedTrackIndex).mStartY, trackBeanList.get(mSelectedTrackIndex).mStartX, trackBeanList.get(mSelectedTrackIndex).mStartY+trackHeight);
                            trackBeanList.get(mSelectedTrackIndex).mRightRect = new RectF(trackBeanList.get(mSelectedTrackIndex).mEndX, trackBeanList.get(mSelectedTrackIndex).mStartY, trackBeanList.get(mSelectedTrackIndex).mEndX + mLeftSeekBitmap.getWidth() , trackBeanList.get(mSelectedTrackIndex).mStartY+trackHeight);
                            break;
                        case 2:
                            durationTmp = TrackUtil.getInstance().getDurationByLength(eventX - mSelectedTrack.mStartX);
                            trackBeanList.get(mSelectedTrackIndex).duration = durationTmp;
                            trackBeanList.get(mSelectedTrackIndex).mEndFrameIndex = trackBeanList.get(mSelectedTrackIndex).mStartFrameIndex + durationTmp;
                            trackBeanList.get(mSelectedTrackIndex).mEndX=(int)eventX;
                            trackBeanList.get(mSelectedTrackIndex).mLeftRect = new RectF(trackBeanList.get(mSelectedTrackIndex).mStartX - mLeftSeekBitmap.getWidth() , trackBeanList.get(mSelectedTrackIndex).mStartY, trackBeanList.get(mSelectedTrackIndex).mStartX, trackBeanList.get(mSelectedTrackIndex).mStartY+trackHeight);
                            trackBeanList.get(mSelectedTrackIndex).mRightRect = new RectF(trackBeanList.get(mSelectedTrackIndex).mEndX, trackBeanList.get(mSelectedTrackIndex).mStartY, trackBeanList.get(mSelectedTrackIndex).mEndX + mLeftSeekBitmap.getWidth() ,trackBeanList.get(mSelectedTrackIndex).mStartY+ trackHeight);
                            break;
                    }
                    mSelectedTrack = trackBeanList.get(mSelectedTrackIndex);
                    TrackActionManager.getInstance().cutTrackOrEffect(mSelectedTrack,touchPoint);
                    updateSelfMaxDuration(mSelectedTrack.id,TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX-TrackConfig.SCREEN_WIDTH_HALF),true);
                }
                isSelectMoving = false;

                break;
        }

        return parentView.getOnTouchEvent(event);
    }
    private void updateSelfMaxDuration(String from,double duration){
        updateSelfMaxDuration(from,duration,false);
    }
    private void updateSelfMaxDuration(String from,double duration,boolean isActionUp){
        if (duration < 0L){
            return;
        }
        //计算textTrackView中的最大时长
        mDurationMap.put(from,duration);
        Collection<Double> c = mDurationMap.values();
        mSelfMaxDuration= Collections.max(c);
        int w=TrackUtil.getInstance().getDurationWidth(mSelfMaxDuration);
        if (mMaxDurationWidth<w){
            mMaxDurationWidth=w;
        }
        TrackUtil.getInstance().updateMaxTrackDuration(3,mSelfMaxDuration,isActionUp);
        parentView.requestLayout();
        parentView.invalidate();
    }

    @Override
    public void addTrackBeanSource(TrackSourceObject object) {
        trackBeanList.clear();
        List<EditData.TextTrackGroup> groups=object.textTrackGroups;
        for (int i = 0; i < groups.size(); i++) {
            EditData.TextTrackGroup group = groups.get(i);
            for (Track track : group.tracks) {
                addTextTrackSource(track, i);
            }
        }
    }

    @Override
    public void onProgressChanged(double time) {

    }

    @Override
    public boolean canBeAddEffect(Effect effect) {
        return false;
    }


    @Override
    public void updateMaxDuration(double duration,boolean isActionUp) {
        if (mSelfMaxDuration!=duration&& duration > 0L&&isActionUp){
            mMaxDurationWidth = TrackUtil.getInstance().getDurationWidth(duration);
            parentView.requestLayout();
            parentView.invalidate();
        }
    }

    @Override
    public void updateScaleSize(float scale) {

    }

    @Override
    public boolean isClicked() {
        return setSelectTrack();
    }

    @Override
    public void cancelSelectTrackByClick() {
            if (mSelectedTrack!=null){
                TrackActionManager.getInstance().onTrackUnSelect(mSelectedTrack.track);
            }
        cancelSelectTrack();
    }

    @Override
    public void cancelSelectEffectByClick() {

    }


    @Override
    public void updateTrack(Track track) {

    }

    @Override
    public void updateMainTrack(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffect(Effect effect) {

    }

    @Override
    public void updateTrackDuration(Track track) {

    }

    @Override
    public void updateMainTrackDuration(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffectDuration(Effect effect) {

    }

    @Override
    public void removeTrack(Track track) {
        if (track!=null){
           for (TrackBean trackBean:trackBeanList){
               if (trackBean.id.equals(track.getTrackId())){
                   trackBeanList.remove(trackBean);
                   break;
               }
           }
           parentView.requestLayout();
           parentView.invalidate();
        }
    }

    @Override
    public void removeMainTrack(MediaTrack mediaTrack) {

    }


    @Override
    public void removeEffect(Effect effect) {

    }

    @Override
    public void addTrack(Track track) {

    }

    @Override
    public void addMainTrack(List<MediaTrack> mainTracks) {

    }

    @Override
    public void addEffect(Effect effect) {

    }

    @Override
    public void addTrackGroup(List<TrackGroup> trackGroups,int trackType) {
        if (trackType== SXTrack.SX_TRACK_TYPE_TEXT||trackType==SXTrack.SX_TRACK_TYPE_STICKER){
            trackBeanList.clear();
            mDurationMap.clear();
            mSelfMaxDuration = mMaxDurationWidth = 0;
            for (int i = 0; i < trackGroups.size(); i++) {
                TrackGroup group = trackGroups.get(i);
                for (Track track : group.tracks()) {
                    addTextTrackSource(track, i);
                }
            }
            TrackUtil.getInstance().updateMaxTrackDuration(3, mSelfMaxDuration, true);
            parentView.requestLayout();
            parentView.invalidate();
        }

    }

    @Override
    public void addEffectList(List<Effect> effectList, int trackType) {

    }

    private void  cancelSelectTrack(){
        mSelectedTrack = null;
        parentView.invalidate();
    }
    public boolean setSelectTrack() {
        boolean isSelected=false;
        for (int i = 0; i < trackBeanList.size(); i++) {
            TrackBean v = trackBeanList.get(i);
            if (v.mStartX < clickX && v.mEndX > clickX
                    &&v.mStartY<clickY&&v.mStartY+trackHeight>clickY) {
                isSelected = true;
                boolean isSWitch=mSelectedTrack!=null;
                if (mSelectedTrack==null||!mSelectedTrack.id.equals(v.id)){
                    mSelectedTrack = v;
                    mSelectedTrackIndex = i;
                    mSelectedTrack.isSelected = true;
                    trackBeanList.get(i).isSelected = true;
                    TrackActionManager.getInstance().onTrackSelect( mSelectedTrack.track,isSWitch);
                }
                break;
            }

        }
        parentView.invalidate();
        return isSelected;
    }

    @Override
    public int getMaxDurationWidth() {
        return mMaxDurationWidth;
    }


    public void addTextTrackSource(Track track, int level) {
        TrackBean trackBean = new TrackBean();
        trackBean.track = track;
        trackBean.id=track.getTrackId();
        trackBean.origDuration = track.duration();
        trackBean.duration = trackBean.origDuration;
        trackBean.durationWidth=TrackUtil.getInstance().getDurationWidth(trackBean.duration);
        if (track instanceof TextTrack) {
            trackBean.text = ((TextTrack) track).content();
            trackBean.mediaType = TrackBean.typeText;
        } else {
            trackBean.mediaType = TrackBean.typeSticker;
            trackBean.text = "贴纸";
        }
        trackBean.mStartFrameIndex = 0;
        trackBean.mEndFrameIndex = trackBean.origDuration;
        trackBean.mStartX = TrackConfig.SCREEN_WIDTH_HALF + TrackUtil.getInstance().getDurationWidth(track.inPoint());
        trackBean.touchPoint = 0;
        trackBean.mEndX = trackBean.mStartX + trackBean.durationWidth;

        trackBean.mLevel=level;
        trackBean.mStartY=TrackConfig.TRACK_HEIGHT*trackBean.mLevel;
        trackBean.mLeftRect = new RectF(trackBean.mStartX - mLeftSeekBitmap.getWidth() , trackBean.mStartY, trackBean.mStartX, trackBean.mStartY+trackHeight);
        trackBean.mRightRect = new RectF(trackBean.mEndX, trackBean.mStartY, trackBean.mEndX + mLeftSeekBitmap.getWidth() , trackBean.mStartY+trackHeight);

        trackBeanList.add(trackBean);

        updateSelfMaxDuration(trackBean.id,TrackUtil.getInstance().getDurationByLength(trackBean.mEndX-TrackConfig.SCREEN_WIDTH_HALF));
        parentView.requestLayout();
        parentView.invalidate();

    }

    //text track
    private boolean canBeMove(int vtSelectedIndex,int eventX,int touchPoint){
        if (isOutSize(touchPoint, mSelectedTrack,eventX)){
            return false;
        }

        if (touchPoint==1) {
            if (vtSelectedIndex == 0
                    || trackBeanList.get(vtSelectedIndex - 1).mLevel < trackBeanList.get(vtSelectedIndex).mLevel) {
                return eventX > TrackConfig.SCREEN_WIDTH_HALF;//每一层靠近零轴的最左边不能拖动了
            } else {
                return eventX > trackBeanList.get(vtSelectedIndex - 1).mEndX;//同一层，往左拖动不能超过左边view
            }
        }else if (touchPoint==2){
            if (vtSelectedIndex+1==trackBeanList.size()
                    || (trackBeanList.get(vtSelectedIndex).mLevel<trackBeanList.get(vtSelectedIndex+1).mLevel)){//最后一个或者每行最后一个
                return true;
            }else {
                return eventX < trackBeanList.get(vtSelectedIndex + 1).mStartX;//同一层，不能超过后面view
            }
        }
        return false;
    }
    //拖动缩小过程中，不能将时长完全为0
    private boolean isOutSize(int touchPoint,TrackBean trackBean,int eventX){
        if (touchPoint==1){
            return eventX>=trackBean.mEndX;
        }else if (touchPoint==2){
            return eventX<=trackBean.mStartX;
        }
        return false;
    }

    private void setTrackText(String trackId, String text) {
        if (TextUtils.isEmpty(trackId) || text == null) return;

        for (TrackBean trackBean : trackBeanList) {
            if (trackId.equals(trackBean.id)) {
                trackBean.text = text;
                parentView.postInvalidate();
            }
        }
    }

    @Override
    public void onMovingUp() {

    }

    @Override
    public void onMovingDown() {

    }

    @Override
    public void onTouchScroll() {

    }

    @Override
    public void onTrackSelect(Track track,boolean isSwitch) {
        int i=0;
        for (TrackBean trackBean:trackBeanList){
            if (trackBean.id.equals(track.getTrackId())){
                    mSelectedTrack = trackBean;
                    mSelectedTrackIndex = i;
                    mSelectedTrack.isSelected = true;
                    trackBeanList.get(i).isSelected = true;
                    parentView.requestLayout();
                    parentView.invalidate();
                    break;
            }
            i++;
        }
    }

    @Override
    public void onEffectSelect(Effect effect,boolean isSwitch) {

    }

    @Override
    public void onTrackUnSelect(Track track) {
        int i=0;
        for (TrackBean trackBean:trackBeanList){
            if (trackBean.id.equals(track.getTrackId())){
                cancelSelectTrack();
                break;
            }
            i++;
        }
    }

    @Override
    public void onEffectUnSelect(Effect effect) {

    }

    @Override
    public void onClickTransition(Track track) {

    }

    @Override
    public void hideDetailFragment() {

    }

    @Override
    public void onTrackTextChanged(String trackId, String text) {
        setTrackText(trackId, text);
    }

    @Override
    public int getHeight() {
        int ret = 0;
        if (trackBeanList.size() > 0) {
            TrackBean trackBean = trackBeanList.get(trackBeanList.size() - 1);
            ret = trackBean.mStartY + trackHeight;
        }
        return ret;
    }
}
