package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.util.Log;
import android.view.View;
import androidx.annotation.Nullable;
import com.shixing.edit.Myapplication;
import com.shixing.edit.data.EditData;
import com.shixing.edit.data.TrackSourceObject;
import com.shixing.edit.multitrack.TrackConfig;
import com.shixing.edit.multitrack.TrackUtil;
import com.shixing.edit.multitrack.TrackViewListener;
import com.shixing.edit.utils.ScreenUtil;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;

import java.util.List;


public class TrackFlexibleRuler extends View implements TrackViewListener {

    //屏幕中间距离
    private static  int mHalfScreenWidth = ScreenUtil.INSTANCE.getScreenWidth(Myapplication.getInstance())/2;;
//    private  List<Float> mylist= Arrays.asList(new Float[]{((float)0.033334f), ((float)0.05f), ((float)0.083334f), ((float)0.166667f), ((float)0.25f), ((float)0.5f), ((float)1f), ((float)1.5f), ((float)2.5f), ((float)5f), ((float)10f), ((float)15f), ((float)30f), ((float)60f), ((float)90f), ((float)150f), ((float)300f), ((float)600f), ((float)900f)});
    private  float pointSize =(float)ScreenUtil.INSTANCE.dp2px(1f);
    private  int mColor= -1;

    private   int mAlpha = 0x80;
    private   float mTextSize=(float)ScreenUtil.INSTANCE.dp2px(9f);
    private   int screenWidth;
    private   Paint paint;
    private float j;
    private float viewHeightHalf;
    private float unitSize;//单位步长
    private float m;
    private int n;
    private int startX;
    private int endX;
    private int mDurationWidth;
    private double mDurationTime;
    private float textHalfWidth;//刻度的一半
    OnDurationWidthResizeListener onDurationWidthResizeListener;

    public TrackFlexibleRuler(Context context) {
        this(context,null);
    }

    public TrackFlexibleRuler(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);

    }

    public TrackFlexibleRuler(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        pointSize =(float)ScreenUtil.INSTANCE.dp2px(1f);
        this.mColor = -1;
        this.mAlpha = 0x80;
        this.mTextSize = (float)ScreenUtil.INSTANCE.dp2px(9f);
        this.screenWidth =ScreenUtil.INSTANCE.getScreenWidth(Myapplication.getInstance());
        paint= new Paint(1);
        paint.setColor(mColor);
        this.paint.setTextSize(this.mTextSize);
        this.paint.setAlpha(this.mAlpha);
        this.j = (this.paint.ascent() + this.paint.descent())/2f;
        this.textHalfWidth=paint.measureText("00:00")/2;
        this.unitSize= TrackConfig.THUMB_WIDTH +this.textHalfWidth*2;
    }

    //时刻转换成00:01格式字符串
    private  String getTrackStrFromInt(int arg18) {
        int v1 = arg18 / 60;
        int v0 = arg18 % 60;

        String v1_1 = v1 >= 10 ? ""+v1: "0" + v1;
        return v0 >= 10 ? v1_1 + ":" + v0 : v1_1 + ":" + ("0" + v0);
    }

    public int getDurationWidth(){
        return mDurationWidth;
    }

    @Override
    public void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        int v8=0;
        if(canvas != null && this.mDurationWidth > 0 && this.mDurationTime > 0L) {
//            this.startX = this.parentView == null ? 0 : this.parentView.getScrollX();
            this.endX = (int)(((double)this.startX) + ((double)this.screenWidth) * 1.5);
            int v0_1 = this.endX;
            int v1 = this.mDurationWidth;
            int v2 = mHalfScreenWidth;
            if(v0_1 < v1 + v2) {//不满半屏，以实际长度为准
                v0_1 = v1 + v2;
            }

            this.endX = v0_1;//终点
            int v0_2 = this.startX;//起点
            int v1_1 = mHalfScreenWidth;
            if(v0_2 > v1_1) {
                v8 = (int)(((float)(v0_2 - v1_1)) / this.unitSize);
            }

            this.n = v8;
            this.m = ((float) mHalfScreenWidth) + ((float)this.n) * this.unitSize-this.textHalfWidth;
            while(true) {
                float p_x = this.m;
                if(p_x >= ((float)this.endX)) {//超出终点
                    return;
                }

                int t = this.n;
                if(t % 2 == 0) {
                    String str = this.getTrackStrFromInt(t);
                    canvas.drawText(str, p_x  , this.viewHeightHalf - this.j, paint);
                    this.m =this.m+ TrackConfig.THUMB_WIDTH/2+this.paint.measureText(str);
                }
                else {
                    canvas.drawCircle(p_x, this.viewHeightHalf, pointSize, this.paint);
                    this.m += TrackConfig.THUMB_WIDTH/2;
                }

                ++this.n;
            }
        }
    }


    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        this.setDurationWidth(TrackUtil.getInstance().getDurationWidth(mDurationTime));
        this.setMeasuredDimension(this.mDurationWidth + mHalfScreenWidth * 2, View.MeasureSpec.getSize(heightMeasureSpec));
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        super.onSizeChanged(w, h, oldw, oldh);
        this.viewHeightHalf = (float)(h / 2);
    }

    private final void setDurationWidth(int durationWidth) {
        if(this.mDurationWidth != durationWidth && durationWidth > 0) {
            this.mDurationWidth = durationWidth;
            this.requestLayout();
            this.invalidate();
            onDurationWidthResizeListener.OnDurationWidthResize(durationWidth);
        }
    }

    public void setOnDurationWidthResizeListener(OnDurationWidthResizeListener listener){
        this.onDurationWidthResizeListener=listener;
    }


    @Override
    public void updateMaxDuration(double duration,boolean isActionUp) {
        if(this.mDurationTime != duration && duration > 0L) {
            this.mDurationTime = duration;
            this.requestLayout();
            this.invalidate();
        }

    }

    @Override
    public void updateScaleSize(float scale) {

    }


    @Override
    public boolean isClicked() {
        return false;
    }

    @Override
    public void cancelSelectTrackByClick() {

    }

    @Override
    public void cancelSelectEffectByClick() {

    }

    @Override
    public void updateTrack(Track track) {

    }

    @Override
    public void updateMainTrack(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffect(Effect effect) {

    }

    @Override
    public void updateTrackDuration(Track track) {

    }

    @Override
    public void updateMainTrackDuration(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffectDuration(Effect effect) {

    }

    @Override
    public void removeTrack(Track track) {

    }

    @Override
    public void removeMainTrack(MediaTrack mediaTrack) {

    }


    @Override
    public void removeEffect(Effect effect) {

    }

    @Override
    public void addTrack(Track track) {

    }

    @Override
    public void addMainTrack(List<MediaTrack> mainTracks) {

    }

    @Override
    public void addEffect(Effect effect) {

    }

    @Override
    public void addTrackGroup(List<TrackGroup> trackGroups, int trackType) {

    }

    @Override
    public void addEffectList(List<Effect> effectList, int trackType) {

    }


    @Override
    public void addTrackBeanSource(TrackSourceObject sourceObject) {

    }

    @Override
    public void onProgressChanged(double time) {

    }

    @Override
    public boolean canBeAddEffect(Effect effect) {
        return false;
    }


    public interface OnDurationWidthResizeListener{
        public void OnDurationWidthResize(int size);
    }

}
