package com.shixing.edit.widget;

import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.RectF;

import com.shixing.sxedit.internal.Track;

public class TrackFrame {
    protected Track track;

    private RectF frame = new RectF();
    private Path path = new Path();
    private Path drawPath = new Path();
    private Matrix transform = new Matrix();

    private float[] position;
    private float rotation;
    private float[] scale;

    protected double currentTime = 0;

    public Track getTrack() {
        return track;
    }

    public void setTrack(Track track) {
        this.track = track;
        if (track != null) {
            int[] size = track.getTrackSize();
            if (size == null) return;

            frame.set(-size[0] / 2f, -size[1] / 2f, size[0] / 2f, size[1] / 2f);
            path.reset();
            path.addRect(frame, Path.Direction.CCW);

            position = track.position();
            rotation = track.rotation();
            scale = track.scale();
            transform.setValues(track.transform());
        }
    }

    public void translate(float x, float y) {
        if (track != null) {
            position[0] += x;
            position[1] += y;

            track.setPosition(position);
        }
    }

    public void scale(float sx, float sy) {
        if (track != null) {
            scale[0] *= sx;
            scale[1] *= sy;

            track.setScale(scale[0], scale[1]);
        }
    }

    public void rotate(float degree) {
        if (track != null) {
            rotation = track.rotation();
            this.rotation += degree;
            track.setRotation(this.rotation);
        }
    }

    public void drawFrame(Canvas canvas, Paint paint) {
        if (track != null && inTrackTimeRange()) {
            transform.setValues(track.transform());
            path.transform(transform, drawPath);
            canvas.drawPath(drawPath, paint);
        }
    }

    protected boolean inTrackTimeRange() {
        return currentTime >= track.inPoint() && currentTime < track.inPoint() + track.duration();
    }

    public void setCurrentTime(double time) {
        currentTime = time;
    }
}
