package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Canvas;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewConfiguration;

import androidx.annotation.Nullable;

import com.shixing.edit.data.TrackSourceObject;
import com.shixing.edit.manager.ActionListener;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionListener;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.multitrack.TrackConfig;
import com.shixing.edit.multitrack.TrackUtil;
import com.shixing.edit.multitrack.TrackViewListener;
import com.shixing.sxedit.internal.AudioItem;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;

import java.util.List;


public class TrackGroupView extends View implements TrackViewListener,TrackActionListener {

    private TrackView mPipTrackInstance;
    private TrackView mTextOrStickerTrackInstance;
    private TrackView mEffectTrackInstance;
    private TrackView mFilterOrAdjustTrackInstance;
    private TrackView mMusicTrackInstance;
    public static final String TRACK_GROUP_PIP="pip";
    public static final String TRACK_GROUP_TEXT_STICKER="textOrSticker";
    public static final String TRACK_GROUP_EFFECT="effect";
    public static final String TRACK_GROUP_FILTER_ADJUST="filterOrAdjust";
    public static final String TRACK_GROUP_MUSIC="music";
    private TrackView mCurrentTrackInstance;

    public TrackGroupView(Context context) {
        this(context, null);
    }

    public TrackGroupView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public TrackGroupView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        mCurrentTrackInstance=new DefaultTrackView();

        mPipTrackInstance =new PipTrackInstance(this);
        mTextOrStickerTrackInstance =new TextOrStickerTrackInstance(this);
        mEffectTrackInstance =new EffectTrackInstance(this);
        mFilterOrAdjustTrackInstance =new FilterOrAdjustTrackInstance(this);
        mMusicTrackInstance=new MusicTrackInstance(this);
    }
    //切换trackGroup
    public void turnTrackGroup(String trackGroup){
        mTransY = 0;
        if (TRACK_GROUP_TEXT_STICKER.equals(trackGroup)){
            mCurrentTrackInstance=mTextOrStickerTrackInstance;
        }else  if (TRACK_GROUP_PIP.equals(trackGroup)){
            mCurrentTrackInstance=mPipTrackInstance;
        }else  if (TRACK_GROUP_FILTER_ADJUST.equals(trackGroup)){
            mCurrentTrackInstance= mFilterOrAdjustTrackInstance;
        }else  if (TRACK_GROUP_EFFECT.equals(trackGroup)){
            mCurrentTrackInstance=mEffectTrackInstance;
        }else  if (TRACK_GROUP_MUSIC.equals(trackGroup)){
            mCurrentTrackInstance=mMusicTrackInstance;
        }
        requestLayout();
        invalidate();
    }

    private boolean canVerticalScroll = false;
    private boolean dragging = false;
    private float mDownY = 0;
    private float mLastY = 0;
    private float mTransY = 0;
    boolean notClick = false;

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        event.offsetLocation(0, -mTransY);
        notClick = false;
        switch (event.getActionMasked()) {
            case MotionEvent.ACTION_DOWN:
                mDownY = event.getRawY();
                if (/*mCurrentTrackInstance.canVerticalScroll() && */mCurrentTrackInstance.getHeight() > getHeight() || mTransY < 0) {
                    canVerticalScroll = true;
                    mLastY = event.getRawY();
                } else {
                    canVerticalScroll = false;
                }
                break;
            case MotionEvent.ACTION_MOVE:
                if (canVerticalScroll) {
                    float y = event.getRawY();
                    float delta = y - mLastY;
                    mLastY = y;

                    if (!dragging && Math.abs(delta) > ViewConfiguration.getTouchSlop()) {
                        dragging = true;
                        MotionEvent e = MotionEvent.obtain(event);
                        e.setAction(MotionEvent.ACTION_CANCEL);
                        mCurrentTrackInstance.onTouchEvent(e);
                        TrackActionManager.getInstance().onMovingDown();
                        Log.e("TAG", "onTouchEvent: dragging");
                        e.recycle();
                    }

                    if (dragging) {
                        mTransY += delta;
                        if (mTransY < getHeight()-mCurrentTrackInstance.getHeight()) {
                            mTransY = getHeight()-mCurrentTrackInstance.getHeight();
                        }
                        if (mTransY > 0) {
                            mTransY = 0;
                        }
                        invalidate();
                        return true;
                    }
                }
                break;
            case MotionEvent.ACTION_UP:
                TrackActionManager.getInstance().onMovingUp();
                dragging = false;
                if (event.getEventTime() - event.getDownTime() > 500 || Math.abs(event.getRawY() - mDownY) > 20) {
                    notClick = true;
                }
                break;
        }
        return mCurrentTrackInstance.onTouchEvent(event);

    }

    public void setCanVerticalScroll(boolean canVerticalScroll) {
        this.canVerticalScroll = canVerticalScroll;
    }

    public boolean getOnTouchEvent(MotionEvent event){
        if (notClick)
            return true;
        return super.onTouchEvent(event);
    }

    @Override
    public void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        canvas.save();
        canvas.translate(0, mTransY);
        mCurrentTrackInstance.onDraw(canvas);
        canvas.save();
    }


    @Override
    public void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        int width = 0;
        ActionListener listener = ActionManager.getInstance().getListener();
        if (listener != null) {
            double duration = listener.getEditManager().editDuration();
            width = TrackUtil.getInstance().getDurationWidth(duration);
        }
        this.setMeasuredDimension(width + TrackConfig.SCREEN_WIDTH_HALF * 2, MeasureSpec.getSize(heightMeasureSpec));
    }


    public void addTrackBeanSource(TrackSourceObject groups) {
        if (groups.sourceType!=TrackSourceObject.SourceType_MainTrack){
            mCurrentTrackInstance.addTrackBeanSource(groups);
        }
    }

    @Override
    public void onProgressChanged(double time) {

    }

    @Override
    public boolean canBeAddEffect(Effect effect) {
        return  mCurrentTrackInstance.canBeAddEffect(effect);
    }


    @Override
    public void cancelSelectTrackByClick() {
        mCurrentTrackInstance.cancelSelectTrackByClick();
    }

    @Override
    public void cancelSelectEffectByClick() {
        mCurrentTrackInstance.cancelSelectEffectByClick();
    }

    @Override
    public void cancelSelectAudioByClick() {
        mCurrentTrackInstance.cancelSelectAudioByClick();
    }

    @Override
    public void updateTrack(Track track) {
        mCurrentTrackInstance.updateTrack(track);
    }

    @Override
    public void updateMainTrack(MediaTrack mainTrack) {
        mCurrentTrackInstance.updateMainTrack(mainTrack);
    }


    @Override
    public void updateEffect(Effect effect) {
        mCurrentTrackInstance.updateEffect(effect);
    }

    @Override
    public void updateTrackDuration(Track track) {
        mCurrentTrackInstance.updateTrackDuration(track);
    }

    @Override
    public void updateMainTrackDuration(MediaTrack mainTrack) {
        mCurrentTrackInstance.updateMainTrackDuration(mainTrack);
    }


    @Override
    public void updateEffectDuration(Effect effect) {
        mCurrentTrackInstance.updateEffectDuration(effect);
    }

    @Override
    public void removeTrack(Track track) {
        mCurrentTrackInstance.removeTrack(track);
    }

    @Override
    public void removeMainTrack(MediaTrack mediaTrack) {
        mCurrentTrackInstance.removeMainTrack(mediaTrack);
    }


    @Override
    public void removeEffect(Effect effect) {
        mCurrentTrackInstance.removeEffect(effect);
    }

    @Override
    public void addTrack(Track track) {
        mCurrentTrackInstance.addTrack(track);
    }

    @Override
    public void addMainTrack(List<MediaTrack> mainTracks) {
        mCurrentTrackInstance.addMainTrack(mainTracks);
    }

    @Override
    public void addEffect(Effect effect) {
        mCurrentTrackInstance.addEffect(effect);
    }

    @Override
    public void addTrackGroup(List<TrackGroup> trackGroups, int trackType) {
        mCurrentTrackInstance.addTrackGroup(trackGroups,trackType);
    }

    @Override
    public void setAudioGroup(List<List<AudioItem>> groups) {
        mCurrentTrackInstance.setAudioGroup(groups);
    }

    @Override
    public void addEffectList(List<Effect> effectList, int trackType) {
        mCurrentTrackInstance.addEffectList(effectList,trackType);
    }


    @Override
    public void updateMaxDuration(double duration,boolean isActionUp) {
        mCurrentTrackInstance.updateMaxDuration(duration,isActionUp);

    }

    @Override
    public void updateScaleSize(float scale) {
        mCurrentTrackInstance.updateScaleSize(scale);
    }

    @Override
    public boolean isClicked() {
        return mCurrentTrackInstance.isClicked();
    }

    @Override
    public void onMovingUp() {
        mCurrentTrackInstance.onMovingUp();
    }

    @Override
    public void onMovingDown() {
        mCurrentTrackInstance.onMovingDown();
    }

    @Override
    public void onTouchScroll() {
        mCurrentTrackInstance.onTouchScroll();
    }

    @Override
    public void onTrackSelect(Track track,boolean isSwitch) {
        mCurrentTrackInstance.onTrackSelect(track,isSwitch);
    }

    @Override
    public void onAudioSelect(AudioItem audioItem, boolean isSWitch) {
        mCurrentTrackInstance.onAudioSelect(audioItem, isSWitch);
    }

    @Override
    public void onEffectSelect(Effect effect,boolean isSwitch) {
        mCurrentTrackInstance.onEffectSelect(effect,isSwitch);
    }

    @Override
    public void onAudioUnSelect(AudioItem audioItem) {
        mCurrentTrackInstance.onAudioUnSelect(audioItem);
    }

    @Override
    public void onTrackUnSelect(Track track) {
        mCurrentTrackInstance.onTrackUnSelect(track);
    }

    @Override
    public void onEffectUnSelect(Effect effect) {
        mCurrentTrackInstance.onEffectUnSelect(effect);
    }

    @Override
    public void onClickTransition(Track track) {
        mCurrentTrackInstance.onClickTransition(track);
    }

    @Override
    public void hideDetailFragment() {
        mCurrentTrackInstance.hideDetailFragment();
    }

    @Override
    public void onAudioThumbChanged(String audioId, int[] data) {
        mMusicTrackInstance.onAudioThumbChanged(audioId, data);
    }

    @Override
    public void onTrackTextChanged(String trackId, String text) {
        mCurrentTrackInstance.onTrackTextChanged(trackId, text);
    }

    public interface TrackView extends TrackViewListener, TrackActionListener {
        public void init(TrackGroupView view);
        public void onDraw(Canvas canvas);
        public boolean onTouchEvent(MotionEvent event );
        public void addTrackBeanSource(TrackSourceObject groups);
        public void cancelSelectTrackByClick();
        public int getMaxDurationWidth();
        default int getHeight() {
            return 0;
        }
        default boolean canVerticalScroll() {
            return false;
        }
    }
    public class DefaultTrackView implements TrackView
    {

        @Override
        public void init(TrackGroupView view) {

        }

        @Override
        public void onDraw(Canvas canvas) {

        }

        @Override
        public boolean onTouchEvent(MotionEvent event) {
            return false;
        }

        @Override
        public void addTrackBeanSource(TrackSourceObject groups) {

        }

        @Override
        public void onProgressChanged(double time) {

        }

        @Override
        public boolean canBeAddEffect(Effect effect) {
            return false;
        }


        @Override
        public void updateMaxDuration(double duration,boolean isActionUp) {

        }

        @Override
        public void updateScaleSize(float scale) {

        }

        @Override
        public boolean isClicked() {
            return false;
        }

        @Override
        public void cancelSelectTrackByClick() {

        }

        @Override
        public void cancelSelectEffectByClick() {

        }


        @Override
        public void updateTrack(Track track) {

        }

        @Override
        public void updateMainTrack(MediaTrack mainTrack) {

        }


        @Override
        public void updateEffect(Effect effect) {

        }

        @Override
        public void updateTrackDuration(Track track) {

        }

        @Override
        public void updateMainTrackDuration(MediaTrack mainTrack) {

        }


        @Override
        public void updateEffectDuration(Effect effect) {

        }

        @Override
        public void removeTrack(Track track) {

        }

        @Override
        public void removeMainTrack(MediaTrack mediaTrack) {

        }


        @Override
        public void removeEffect(Effect effect) {

        }

        @Override
        public void addTrack(Track track) {

        }

        @Override
        public void addMainTrack(List<MediaTrack> mainTracks) {

        }

        @Override
        public void addEffect(Effect effect) {

        }

        @Override
        public void addTrackGroup(List<TrackGroup> trackGroups, int trackType) {

        }

        @Override
        public void addEffectList(List<Effect> effectList, int trackType) {

        }




        @Override
        public int getMaxDurationWidth() {
            return 0;
        }

        @Override
        public void onMovingUp() {

        }

        @Override
        public void onMovingDown() {

        }

        @Override
        public void onTouchScroll() {

        }

        @Override
        public void onTrackSelect(Track track, boolean isSwitch) {

        }


        @Override
        public void onEffectSelect(Effect effect,boolean isSwitch) {

        }

        @Override
        public void onTrackUnSelect(Track track) {

        }

        @Override
        public void onEffectUnSelect(Effect effect) {

        }

        @Override
        public void onClickTransition(Track track) {

        }

        @Override
        public void hideDetailFragment() {

        }

        @Override
        public int getHeight() {
            return 0;
        }
    }
}
