package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.BitmapShader;
import android.graphics.Canvas;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.RectF;
import android.graphics.Shader;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

import androidx.annotation.Nullable;

import com.shixing.edit.R;
import com.shixing.edit.data.TrackBean;
import com.shixing.edit.data.TrackSourceObject;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionListener;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.multitrack.TrackConfig;
import com.shixing.edit.multitrack.TrackUtil;
import com.shixing.edit.multitrack.TrackViewListener;
import com.shixing.edit.utils.ScreenUtil;
import com.shixing.edit.utils.Utils;
import com.shixing.sxedit.effect.SXEffect;
import com.shixing.sxedit.internal.AnimationEffect;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.MainTrackGroup;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;
import com.shixing.sxedit.track.SXTrack;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;


public class VideoTrackView extends View implements TrackViewListener, TrackActionListener {
    private static final String TAG = "VideoTrackView";
    private List<TrackBean> videoTrackBeanList;
    private float pxSize ;
    private float mTextSize;
    private int mDurationWidth;
    Bitmap mDefaultBitmap, mDefaultTransBitmap,mTranslationBitmap, mLeftSeekBitmap, mRightSeekBitmap;
    private int mTrackHeight;
    public boolean isSelectMoving;//是否选中并拖动了
    public float eventX, eventY;
    private float clickX,clickY;
    TrackBean mSelectedTrack = null;
    private int mSelectedTrackIndex = 0;

    private boolean mShowAnimationMask = true;
    private Paint mAnimationMaskPaint;
    private RectF mAnimationMaskRect = new RectF();

    public VideoTrackView(Context context) {
        this(context, null);

    }

    public VideoTrackView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);

    }

    public VideoTrackView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        pxSize = (float) ScreenUtil.INSTANCE.dp2px(1f);
        this.mTextSize = (float) ScreenUtil.INSTANCE.dp2px(9f);

        mTrackHeight = ScreenUtil.INSTANCE.dp2px(50);
        videoTrackBeanList = new ArrayList<>();
        Bitmap b1=BitmapFactory.decodeResource(context.getResources(), R.drawable.icon_default_pic);
        float  sc = mTrackHeight * 1f / b1.getHeight();
        mDefaultBitmap = Utils.scaleBitmap(b1,sc);
        Bitmap b2 = BitmapFactory.decodeResource(getResources(), R.drawable.icon_jiaojiefangkuang);
        float  sc2 = mTrackHeight /2 * 1f / b2.getHeight();
        mDefaultTransBitmap = Utils.scaleBitmap(b2,sc2);

        Bitmap b6 = BitmapFactory.decodeResource(getResources(), R.drawable.icon_zhuanchangkuang);
        float  sc6 = mTrackHeight /2 * 1f / b6.getHeight();
        mTranslationBitmap = Utils.scaleBitmap(b6,sc6);
        Bitmap b3=BitmapFactory.decodeResource(context.getResources(), R.drawable.icon_tiaojiekuang_zuo);
        float  sc3 = mTrackHeight * 1f / b3.getHeight();
        mLeftSeekBitmap = Utils.scaleBitmap(b3,sc3);
        Bitmap b4=BitmapFactory.decodeResource(context.getResources(), R.drawable.icon_tiaojiekuang_you);
        float  sc4 = mTrackHeight * 1f / b4.getHeight();
        mRightSeekBitmap = Utils.scaleBitmap(b4,sc4);


        mAnimationMaskPaint = new Paint();
        mAnimationMaskPaint.setColor(getResources().getColor(R.color.mask_animation));
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {

        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                clickX = event.getX();
                clickY=event.getY();
                if (mSelectedTrack != null) {
                    if (mSelectedTrack.mLeftRect.contains(event.getX(), event.getY())) {
                        mSelectedTrack.difX = mSelectedTrack.mLeftRect.right - event.getX();
                        TrackActionManager.getInstance().onMovingDown();
                        eventX = event.getX() + mSelectedTrack.difX;
                        eventY = event.getY();
                        mSelectedTrack.touchPoint = 1;
                        isSelectMoving = true;
                        return true;
                    } else if (mSelectedTrack.mRightRect.contains(event.getX(), event.getY())) {
                        mSelectedTrack.difX = mSelectedTrack.mRightRect.left - event.getX();
                        TrackActionManager.getInstance().onMovingDown();
                        eventX = event.getX() + mSelectedTrack.difX;
                        eventY = event.getY();
                        mSelectedTrack.touchPoint = 2;
                        isSelectMoving = true;
                        return true;
                    } else {
                        mSelectedTrack.touchPoint = 0;
                    }
                }
                break;
            case MotionEvent.ACTION_MOVE:
                if (isSelectMoving) {
                    double dtmp = 0;
                    switch (mSelectedTrack.touchPoint) {
                        case 1:
                            dtmp = TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX - (event.getX() + mSelectedTrack.difX));
                            if (canBeMove(mSelectedTrackIndex, (int) (event.getX() + mSelectedTrack.difX), mSelectedTrack.touchPoint, dtmp)) {
                                eventX = event.getX() + mSelectedTrack.difX;
                                eventY = event.getY();
                                videoTrackBeanList.get(mSelectedTrackIndex).duration = TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX - eventX);

                                updateSelfMaxDuration(videoTrackBeanList.get(mSelectedTrackIndex).track,  videoTrackBeanList.get(mSelectedTrackIndex).duration);
                            }
                            break;
                        case 2:
                            dtmp = TrackUtil.getInstance().getDurationByLength((event.getX() + mSelectedTrack.difX) - mSelectedTrack.mStartX);
                            if (canBeMove(mSelectedTrackIndex, (int) (event.getX() + mSelectedTrack.difX), mSelectedTrack.touchPoint, dtmp)) {
                                eventX = event.getX() + mSelectedTrack.difX;
                                eventY = event.getY();
                                videoTrackBeanList.get(mSelectedTrackIndex).duration = TrackUtil.getInstance().getDurationByLength(eventX - mSelectedTrack.mStartX);
                                updateSelfMaxDuration(mSelectedTrack.track,videoTrackBeanList.get(mSelectedTrackIndex).duration);
                            }
                            break;
                    }
                    this.invalidate();

                }
                break;
            case MotionEvent.ACTION_UP:
                double durationTmp = 0;
                if (isSelectMoving) {
                    int touchPoint=mSelectedTrack.touchPoint;
                    switch (mSelectedTrack.touchPoint) {
                        case 1:
                            durationTmp = TrackUtil.getInstance().getDurationByLength(mSelectedTrack.mEndX - eventX);
                            videoTrackBeanList.get(mSelectedTrackIndex).duration = durationTmp;
                            videoTrackBeanList.get(mSelectedTrackIndex).mStartFrameIndex = videoTrackBeanList.get(mSelectedTrackIndex).mEndFrameIndex - durationTmp;
                            mSelectedTrack.mStartX=(int)eventX;
                            mSelectedTrack.duration=durationTmp;
                            mSelectedTrack.mStartFrameIndex=videoTrackBeanList.get(mSelectedTrackIndex).mStartFrameIndex;
                            break;
                        case 2:
                            durationTmp = TrackUtil.getInstance().getDurationByLength(eventX - mSelectedTrack.mStartX);
                            videoTrackBeanList.get(mSelectedTrackIndex).duration = durationTmp;
                            videoTrackBeanList.get(mSelectedTrackIndex).mEndFrameIndex = videoTrackBeanList.get(mSelectedTrackIndex).mStartFrameIndex + durationTmp;
                            mSelectedTrack.mEndX=(int)eventX;
                            mSelectedTrack.duration=durationTmp;
                            mSelectedTrack.mEndFrameIndex=videoTrackBeanList.get(mSelectedTrackIndex).mEndFrameIndex;
                            break;
                    }
                    TrackActionManager.getInstance().cutTrackOrEffect(mSelectedTrack,touchPoint);
                    updateVideoTrack(true);
                    mSelectedTrack = videoTrackBeanList.get(mSelectedTrackIndex);
                }
                isSelectMoving = false;
                invalidate();
                TrackActionManager.getInstance().onMovingUp();
                break;
        }

        return super.onTouchEvent(event);

    }

    @Override
    public void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        if (canvas != null && this.mDurationWidth > 0) {
            Paint pbg = new Paint();
            pbg.setColor(getResources().getColor(R.color.bg_track_blake));
            canvas.drawRect(TrackConfig.SCREEN_WIDTH_HALF, 0, mDurationWidth, mTrackHeight, pbg);
            Paint paint = new Paint();
            Paint p=new Paint();
            p.setColor(getResources().getColor(R.color.white));
            for (int i = 0; i < videoTrackBeanList.size(); i++) {
                TrackBean videoTrackBean = videoTrackBeanList.get(i);
                BitmapShader bitmapShader = new BitmapShader(videoTrackBean.shaderBitmap, Shader.TileMode.REPEAT, Shader.TileMode.REPEAT);
                Matrix matrix=new Matrix();
                int xtmp=-TrackUtil.getInstance().getDurationWidth(videoTrackBean.mStartFrameIndex)+videoTrackBean.mStartX;
                matrix.postTranslate(xtmp,0);
                bitmapShader.setLocalMatrix(matrix);
                paint.setShader(bitmapShader);
                MediaTrack mediaTrack=(MediaTrack)  videoTrackBean.track;

                //animation
                AnimationEffect animationEffect = null;
                double animDuration = 0;
                SXEffect.SXEffectTimeFollowType followType = null;
                if (mShowAnimationMask) {
                    List<AnimationEffect> animations = mediaTrack.getAnimations();
                    if (animations != null && animations.size() > 0) {
                        animationEffect = animations.get(0);
                        followType = animationEffect.followType();
                        animDuration = animationEffect.duration();
                    }
                }

                RectF rectF;
                if (i==0){
                    if (mediaTrack.hasTransition()){
                        MainTrackGroup mainTrackGroup=ActionManager.getInstance().getListener().getMainGroup();
                        double endts=mainTrackGroup.getTransitionDuration(videoTrackBean.track.getTrackId());
                        rectF = new RectF(videoTrackBean.mStartX, pxSize * 2, videoTrackBean.mEndX-TrackUtil.getInstance().getDurationWidth(endts)/2, mTrackHeight - pxSize * 2);

                        if (mShowAnimationMask && animationEffect != null) {
                            mAnimationMaskRect.set(rectF);
                            if (followType == SXEffect.SXEffectTimeFollowType.FollowStart) {
                                mAnimationMaskRect.right = Math.max(mAnimationMaskRect.left, mAnimationMaskRect.left + TrackUtil.getInstance().getDurationWidth(animDuration));
                            } else if (followType == SXEffect.SXEffectTimeFollowType.FollowEnd) {
                                mAnimationMaskRect.left = mAnimationMaskRect.right - TrackUtil.getInstance().getDurationWidth(Math.max(0, animDuration - endts));
                            }
                        }
                    }else {
                        rectF = new RectF(videoTrackBean.mStartX, pxSize * 2, videoTrackBean.mEndX, mTrackHeight - pxSize * 2);
                        if (mShowAnimationMask && animationEffect != null) {
                            mAnimationMaskRect.set(rectF);
                            if (followType == SXEffect.SXEffectTimeFollowType.FollowStart) {
                                mAnimationMaskRect.right = mAnimationMaskRect.left + TrackUtil.getInstance().getDurationWidth(animDuration);
                            } else if (followType == SXEffect.SXEffectTimeFollowType.FollowEnd) {
                                mAnimationMaskRect.left = mAnimationMaskRect.right - TrackUtil.getInstance().getDurationWidth(animDuration);
                            }
                        }
                    }
                }else {
                    double endts=0;
                    double startts=0;
                    MainTrackGroup mainTrackGroup=ActionManager.getInstance().getListener().getMainGroup();
                    if (mediaTrack.hasTransition()){
                        endts=mainTrackGroup.getTransitionDuration(videoTrackBean.track.getTrackId());
                    }

                    TrackBean lastTrack=videoTrackBeanList.get(i-1);
                    if (((MediaTrack)lastTrack.track).hasTransition()){
                        startts=mainTrackGroup.getTransitionDuration(lastTrack.track.getTrackId());
                    }

                    rectF = new RectF(videoTrackBean.mStartX+TrackUtil.getInstance().getDurationWidth(startts)/2, pxSize * 2, videoTrackBean.mEndX-TrackUtil.getInstance().getDurationWidth(endts)/2, mTrackHeight - pxSize * 2);

                    if (mShowAnimationMask && animationEffect != null) {
                        mAnimationMaskRect.set(rectF);
                        if (followType == SXEffect.SXEffectTimeFollowType.FollowStart) {
                            mAnimationMaskRect.right = Math.min(mAnimationMaskRect.right, mAnimationMaskRect.left + TrackUtil.getInstance().getDurationWidth(Math.max(0, animDuration - startts)));
                        } else if (followType == SXEffect.SXEffectTimeFollowType.FollowEnd) {
                            mAnimationMaskRect.left = Math.max(mAnimationMaskRect.left, mAnimationMaskRect.right - TrackUtil.getInstance().getDurationWidth(Math.max(0, animDuration - endts)));
                        }
                    }
                }

                canvas.drawRect(rectF, paint);
                videoTrackBean.showingEnd = rectF.right;

                if (mShowAnimationMask && animationEffect != null) {
                    canvas.drawRect(mAnimationMaskRect, mAnimationMaskPaint);
                }
            }
            if (mSelectedTrack != null) {
                Paint p2 = new Paint();
                p2.setColor(getResources().getColor(R.color.bg_track_blake));
                RectF rectF;
                RectF rectF_top;
                RectF rectF_bottom;

                Matrix matrix = new Matrix();
                Matrix matrix_end = new Matrix();

                switch (mSelectedTrack.touchPoint) {
                    case 0:
                        matrix.postTranslate(mSelectedTrack.mStartX - mLeftSeekBitmap.getWidth() , 0);
                        canvas.drawBitmap(mLeftSeekBitmap, matrix, p2);//画滑块
                        rectF_top = new RectF(mSelectedTrack.mStartX, 0, mSelectedTrack.mEndX, pxSize * 2);
                        rectF_bottom = new RectF(mSelectedTrack.mStartX, mTrackHeight - pxSize * 2, mSelectedTrack.mEndX, mTrackHeight);
                        p2.setColor(getResources().getColor(R.color.white));
                        canvas.drawRect(rectF_top, p2);
                        canvas.drawRect(rectF_bottom, p2);
                        matrix_end.postTranslate(mSelectedTrack.mEndX, 0);
                        canvas.drawBitmap(mRightSeekBitmap, matrix_end, p2);
                        break;
                    case 1://按住左边滑动头
                        rectF = new RectF(mSelectedTrack.mStartX, 0, eventX, mTrackHeight);
                        canvas.drawRect(rectF, p2);//画滑动的黑色背景
                        matrix.postTranslate(eventX - mLeftSeekBitmap.getWidth() , 0);
                        canvas.drawBitmap(mLeftSeekBitmap, matrix, p2);//画滑块
                        rectF_top = new RectF(eventX, 0, mSelectedTrack.mEndX, pxSize * 2);
                        rectF_bottom = new RectF(eventX, mTrackHeight - pxSize * 2, mSelectedTrack.mEndX, mTrackHeight);
                        p2.setColor(getResources().getColor(R.color.white));
                        canvas.drawRect(rectF_top, p2);
                        canvas.drawRect(rectF_bottom, p2);
                        matrix_end.postTranslate(mSelectedTrack.mEndX, 0);
                        canvas.drawBitmap(mRightSeekBitmap, matrix_end, p2);
                        break;
                    case 2://右边
                        rectF = new RectF(eventX, 0, mSelectedTrack.mEndX , mTrackHeight);
                        canvas.drawRect(rectF, p2);//画滑动的黑色背景
                        matrix.postTranslate(mSelectedTrack.mStartX - mLeftSeekBitmap.getWidth() , 0);
                        canvas.drawBitmap(mLeftSeekBitmap, matrix, p2);//画滑块
                        rectF_top = new RectF(mSelectedTrack.mStartX, 0, eventX, pxSize * 2);
                        rectF_bottom = new RectF(mSelectedTrack.mStartX, mTrackHeight - pxSize * 2, eventX, mTrackHeight);
                        p2.setColor(getResources().getColor(R.color.white));
                        canvas.drawRect(rectF_top, p2);
                        canvas.drawRect(rectF_bottom, p2);
                        matrix_end.postTranslate(eventX, 0);
                        canvas.drawBitmap(mRightSeekBitmap, matrix_end, p2);
                        break;
                }
                Paint ptxt = new Paint();
                ptxt.setColor(getResources().getColor(R.color.white));
                ptxt.setTextSize(mTextSize);
                String duration = String.format("%.1f", mSelectedTrack.duration) + "s";
                canvas.drawText(duration, mSelectedTrack.mStartX + pxSize * 8, pxSize * 12, ptxt);
            }
            for (int i = 0; i < videoTrackBeanList.size()-1; i++) {
                TrackBean trackBean=videoTrackBeanList.get(i);
                if (mSelectedTrack ==null||(mSelectedTrack != null && i != mSelectedTrackIndex - 1 && i != mSelectedTrackIndex)) {
                    if (((MediaTrack)trackBean.track).hasTransition()){
                        MainTrackGroup mainTrackGroup=ActionManager.getInstance().getListener().getMainGroup();
                        double ts=mainTrackGroup.getTransitionDuration(trackBean.track.getTrackId());
                        canvas.drawBitmap(mTranslationBitmap,trackBean.mEndX-mTranslationBitmap.getWidth()/2-TrackUtil.getInstance().getDurationWidth(ts)/2, mTrackHeight / 2-mTranslationBitmap.getHeight()/2,p);
                    }else {
                        canvas.drawBitmap(mDefaultTransBitmap,trackBean.mEndX-mDefaultTransBitmap.getWidth()/2, mTrackHeight / 2-mDefaultTransBitmap.getHeight()/2,p);
                    }
                    trackBean.isShowTransitionRect=true;
                }
            }
        }
    }


    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        this.setMeasuredDimension(this.mDurationWidth + TrackConfig.SCREEN_WIDTH_HALF * 2, MeasureSpec.getSize(heightMeasureSpec));
        Log.e(TAG, "onMeasure: " + (mDurationWidth + TrackConfig.SCREEN_WIDTH_HALF * 2));
    }

    HashMap<String, Double> durationMap = new HashMap<>();
    double mSelfMaxDuration;
    //清空重新计算
    private void updateSelfMaxDuration(List<TrackBean> trackBeanList,boolean isActionUp){
        durationMap.clear();
        mSelfMaxDuration=0;
        MainTrackGroup mainTrackGroup=ActionManager.getInstance().getListener().getMainGroup();
        for (TrackBean tb:trackBeanList){
            durationMap.put(tb.id, tb.duration);
            if (tb.duration>0) {
                if (tb.track instanceof MediaTrack){
                    MediaTrack mediaTrack=((MediaTrack)tb.track);
                    if (mediaTrack.hasTransition()){
                        double ts=mainTrackGroup.getTransitionDuration(mediaTrack.getTrackId());
                        mSelfMaxDuration=mSelfMaxDuration-ts;
                    }
                }
                mSelfMaxDuration = mSelfMaxDuration + tb.duration;
            }
        }
        updateTotalDuration(mSelfMaxDuration,isActionUp);
    }
    private void updateSelfMaxDuration(Track from, double duration) {
        MainTrackGroup mainTrackGroup=ActionManager.getInstance().getListener().getMainGroup();
        if (from instanceof MediaTrack){
            MediaTrack mediaTrack=((MediaTrack)from);
            if (mediaTrack.hasTransition()){
                double ts=mainTrackGroup.getTransitionDuration(mediaTrack.getTrackId());
                duration=duration-ts;
            }
        }
        updateSelfMaxDuration(from.getTrackId(),duration,false);
    }

    private void updateSelfMaxDuration(String from, double duration,boolean isActionUp) {
        if (duration < 0L) {
            return;
        }
        //计算textTrackView中的最大时长
        durationMap.put(from, duration);
        mSelfMaxDuration=0;
        Iterator<HashMap.Entry<String, Double>> iterator = durationMap.entrySet().iterator();
        while (iterator.hasNext()) {
            HashMap.Entry<String, Double> entry = iterator.next();
            mSelfMaxDuration=mSelfMaxDuration+entry.getValue();
        }
        updateTotalDuration(mSelfMaxDuration,isActionUp);
    }

    private void updateTotalDuration(double mSelfMaxDuration){
        updateTotalDuration(mSelfMaxDuration,false);
    }
    private void updateTotalDuration(double mSelfMaxDuration,boolean isActionUp){
        int w = TrackUtil.getInstance().getDurationWidth(mSelfMaxDuration);
//        if (mDurationWidth < w) {
            mDurationWidth = w;
//        }
        TrackUtil.getInstance().updateMaxTrackDuration(1, mSelfMaxDuration,isActionUp);
        requestLayout();
        invalidate();
    }

    public boolean isSelectTransition(TrackBean trackBean){
        if (trackBean.isShowTransitionRect){
            RectF rectF=new RectF(trackBean.showingEnd-25* pxSize,0,trackBean.showingEnd+25* pxSize, mTrackHeight);
            return rectF.contains(clickX,clickY);
        }
        return false;
    }

    private HashMap<String,Bitmap> shaderBitmaps=new HashMap<>();
    private void addVideoTracks(List<MediaTrack> tracks) {
        videoTrackBeanList.clear();
        for (MediaTrack track : tracks) {
            TrackBean trackBean = new TrackBean();
            if (track.resourceDuration() <= 0){
                trackBean.mediaType=TrackBean.typePic;
                trackBean.origDuration =TrackConfig.PIC_TRACK_DURATION_DEFAULT;
                trackBean.srcPath = track.getFilePath();
                Bitmap bitmap = BitmapFactory.decodeFile(trackBean.srcPath);
                if (bitmap == null) bitmap = mDefaultBitmap;
                trackBean.shaderBitmap = Utils.centerSquareScaleBitmap(bitmap, mTrackHeight);
            }else {
                trackBean.mediaType=TrackBean.typeVideo;
                trackBean.mSpeed=ActionManager.getInstance().getListener().getSpeed(track);
                trackBean.origDuration=track.resourceDuration()/trackBean.mSpeed;
                trackBean.srcPath = track.getFilePath();


                if (!shaderBitmaps.containsKey(track.getFilePath())){
                    double unit=TrackUtil.getInstance().getDurationByLength(mTrackHeight);
                    double duration=TrackUtil.getInstance().getLocalVideoDuration(trackBean.srcPath)/(1000*unit);//单位高度多少时长
                    int l=(int)duration+1;
                    Bitmap[] bitmaps=new Bitmap[l];
                    for (int i = 0; i < bitmaps.length; i++) {
                        bitmaps[i]= mDefaultBitmap;
                    }
                    TrackUtil.getInstance().getVideoBitmap(trackBean.id,trackBean.srcPath, mTrackHeight,l, unit, new TrackUtil.OnFrameLoadListener() {
                        @Override
                        public void onFrameLoadFinish(String trackId,int index,Bitmap bitmap) {
                            Bitmap tmp=null;
                            if (track!=null){
                                bitmaps[index]=bitmap;
                                tmp=TrackUtil.getInstance().add2Bitmap(bitmaps, mTrackHeight, mTrackHeight);
                                shaderBitmaps.put(track.getFilePath(),tmp);
                            }
                            if (trackBean!=null&&tmp!=null){
                                trackBean.shaderBitmap= tmp;
                            }
                            post(() -> invalidate());
                        }
                    });
                }else {
                    trackBean.shaderBitmap =shaderBitmaps.get(track.getFilePath());
                }

                if (trackBean.shaderBitmap==null){
                    trackBean.shaderBitmap= mDefaultBitmap;
                }
            }

            trackBean.duration = track.duration() ;
            trackBean.id = track.getTrackId();
            trackBean.track=track;

            videoTrackBeanList.add(trackBean);
        }
        updateVideoTrack(true);
        this.requestLayout();
        this.invalidate();
    }
    public void updateVideoTrack(boolean isActionUp) {

        ArrayList<TrackBean> vlist = new ArrayList();
        for (int i = 0; i < videoTrackBeanList.size(); i++) {
            TrackBean videoTrackBean = videoTrackBeanList.get(i);
            videoTrackBean.durationWidth = TrackUtil.getInstance().getDurationWidth(videoTrackBean.duration);
            if (i == 0) {
                videoTrackBean.mStartX = TrackConfig.SCREEN_WIDTH_HALF;
            } else {


               MediaTrack mediaTrack=(MediaTrack)  vlist.get(i - 1).track;
               if (mediaTrack!=null&&mediaTrack.hasTransition()){
                   MainTrackGroup mainTrackGroup=ActionManager.getInstance().getListener().getMainGroup();
                   double ts=mainTrackGroup.getTransitionDuration(mediaTrack.getTrackId());
                   videoTrackBean.mStartX = vlist.get(i - 1).mEndX-TrackUtil.getInstance().getDurationWidth(ts);//有转场的，向前排1.5s
               }else {
                   videoTrackBean.mStartX = vlist.get(i - 1).mEndX;
               }


            }
            videoTrackBean.touchPoint = 0;
            videoTrackBean.mEndX = videoTrackBean.mStartX + videoTrackBean.durationWidth;
            videoTrackBean.mEndFrameIndex=videoTrackBean.mStartFrameIndex+videoTrackBean.duration;
            videoTrackBean.mLeftRect = new RectF(videoTrackBean.mStartX - mLeftSeekBitmap.getWidth() , 0, videoTrackBean.mStartX, mTrackHeight);
            videoTrackBean.mRightRect = new RectF(videoTrackBean.mEndX, 0, videoTrackBean.mEndX + mLeftSeekBitmap.getWidth() , mTrackHeight);
            vlist.add(videoTrackBean);
            if (mSelectedTrack!=null&&mSelectedTrack.id.equals(videoTrackBean.id)){
                mSelectedTrack=videoTrackBean;
            }
        }
        videoTrackBeanList.clear();
        videoTrackBeanList.addAll(vlist);

        updateSelfMaxDuration(videoTrackBeanList,isActionUp);
        this.requestLayout();
        this.invalidate();

    }


    //text track
    private boolean canBeMove(int vtSelectedIndex, int eventX, int touchPoint, double dtmp) {
        if (isOutSize(touchPoint, mSelectedTrack,eventX)){
            return false;
        }
        if (mSelectedTrack.mediaType == TrackBean.typePic) {
            if (touchPoint == 1) {
                if (vtSelectedIndex == 0
                        || videoTrackBeanList.get(vtSelectedIndex - 1).mLevel < videoTrackBeanList.get(vtSelectedIndex).mLevel) {
                    return eventX > TrackConfig.SCREEN_WIDTH_HALF;//每一层靠近零轴的最左边不能拖动了
                } else {
                    MediaTrack lastTrack=(MediaTrack) videoTrackBeanList.get(vtSelectedIndex-1).track;
                    MainTrackGroup mainTrackGroup=ActionManager.getInstance().getListener().getMainGroup();
                    double ts=mainTrackGroup.getTransitionDuration(lastTrack.getTrackId());
                    if (lastTrack.hasTransition()){
                        return eventX > videoTrackBeanList.get(vtSelectedIndex - 1).mEndX-TrackUtil.getInstance().getDurationWidth(ts);//同一层，往左拖动不能超过左边view
                    }else {
                        return eventX > videoTrackBeanList.get(vtSelectedIndex - 1).mEndX;//同一层，往左拖动不能超过左边view
                    }
                }
            } else if (touchPoint == 2) {

                return true;//图片的右边无限制
            }
        } else {
            if (touchPoint == 1) {
                return mSelectedTrack.mEndFrameIndex - dtmp >= 0;
            } else if (touchPoint == 2) {
                return mSelectedTrack.mStartFrameIndex + dtmp <= mSelectedTrack.origDuration;
            }

        }
        return false;
    }

    //拖动缩小过程中，不能将时长完全为0
    private boolean isOutSize(int touchPoint,TrackBean trackBean,int eventX){
        if (touchPoint==1){
            return eventX>=trackBean.mEndX;
        }else if (touchPoint==2){
            return eventX<=trackBean.mStartX;
        }
        return false;
    }

    @Override
    public void updateMaxDuration(double duration, boolean isActionUp) {
        if (mSelfMaxDuration != duration && duration > 0L&&isActionUp) {
            mDurationWidth = TrackUtil.getInstance().getDurationWidth(duration);
            requestLayout();
            invalidate();
        }
    }

    @Override
    public void updateScaleSize(float scale) {

    }

    @Override
    public boolean isClicked() {
        boolean isSelected = false;
        for (int i = 0; i < videoTrackBeanList.size(); i++) {
            TrackBean v = videoTrackBeanList.get(i);
            if (isSelectTransition(v)){
                TrackActionManager.getInstance().onClickTransition(v.track);
                return true;
            }
            if (v.mStartX < clickX && v.mEndX > clickX) {
                isSelected = true;
                boolean isSWitch=mSelectedTrack!=null;
                if (mSelectedTrack==null||!mSelectedTrack.id.equals(v.id)){
                    mSelectedTrack = v;
                    mSelectedTrackIndex = i;
                    mSelectedTrack.isSelected = true;
                    videoTrackBeanList.get(i).isSelected = true;
                    TrackActionManager.getInstance().onTrackSelect( mSelectedTrack.track,isSWitch);
                }
                break;
            }
        }
        invalidate();
        return isSelected;
    }

    @Override
    public void cancelSelectTrackByClick() {
        if (mSelectedTrack !=null){
            TrackActionManager.getInstance().onTrackUnSelect( mSelectedTrack.track);
        }
        cancelSelectTrack();
    }

    @Override
    public void cancelSelectEffectByClick() {

    }

    private void cancelSelectTrack(){
        mSelectedTrack = null;
        mSelectedTrackIndex=0;
        invalidate();
    }


    @Override
    public void updateTrack(Track track) {

    }

    @Override
    public void updateMainTrack(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffect(Effect effect) {

    }

    @Override
    public void updateTrackDuration(Track track) {

    }

    @Override
    public void updateMainTrackDuration(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffectDuration(Effect effect) {

    }

    @Override
    public void removeTrack(Track track) {

    }

    @Override
    public void removeMainTrack(MediaTrack mediaTrack) {

    }


    @Override
    public void removeEffect(Effect effect) {

    }

    @Override
    public void addTrack(Track track) {

    }

    @Override
    public void addMainTrack(List<MediaTrack> mainTracks) {
        addVideoTracks(mainTracks);
    }

    @Override
    public void addEffect(Effect effect) {

    }

    @Override
    public void addTrackGroup(List<TrackGroup> trackGroups, int trackType) {

    }

    @Override
    public void addEffectList(List<Effect> effectList, int trackType) {

    }


    @Override
    public void addTrackBeanSource(TrackSourceObject sourceObject) {

    }

    @Override
    public void onProgressChanged(double time) {

    }

    @Override
    public boolean canBeAddEffect(Effect effect) {
        return false;
    }


    public Track getCursorSelectMainTrack(int currentCursorX) {
      for (TrackBean trackBean:videoTrackBeanList){
          if (trackBean.mStartX<currentCursorX&&trackBean.mEndX>currentCursorX){
              return trackBean.track;
          }
      }
        return videoTrackBeanList.get(0).track;
    }

    @Override
    public void onMovingUp() {

    }

    @Override
    public void onMovingDown() {

    }

    @Override
    public void onTouchScroll() {

    }

    @Override
    public void onTrackSelect(Track track,boolean isSwitch) {
        int i=0;
        for (TrackBean trackBean:videoTrackBeanList){
            if (track.getTrackId().equals(trackBean.id)){
                    mSelectedTrack = trackBean;
                    mSelectedTrackIndex = i;
                    mSelectedTrack.isSelected = true;
                    videoTrackBeanList.get(i).isSelected = true;
                    requestLayout();
                    invalidate();
                break;
            }
            i++;
        }
    }

    @Override
    public void onEffectSelect(Effect effect,boolean isSwitch) {

    }

    @Override
    public void onTrackUnSelect(Track track) {
            if (track!=null&&track.type()== SXTrack.SX_TRACK_TYPE_MEDIA){
                for (TrackBean trackBean:videoTrackBeanList){
                    if (trackBean.id.equals(track.getTrackId())){
                        cancelSelectTrack();
                    }
                }
            }
    }

    @Override
    public void onEffectUnSelect(Effect effect) {

    }

    @Override
    public void onClickTransition(Track track) {

    }

    @Override
    public void hideDetailFragment() {

    }

    @Override
    public void updateTrackAnimation() {
        invalidate();
    }

    @Override
    public void showTrackAnimationMask(boolean show) {
        mShowAnimationMask = show;
        invalidate();
    }
}
